@@ -264,10 +264,9 @@ rowindex OrderVector;
 OrderVector=(long *)calloc(m_size+1, sizeof *OrderVector);
 DDInit(m_size, n_size, A, InitRays, OrderVector);
 FindInitialRays(m_size, n_size, A, OrderVector, 
-    InitialHyperplanes, InitRays, InitRayIndex, &found);
+                InitialHyperplanes, InitRays, InitRayIndex, &found);
 
-/* removed unecessary stuff */
-flag=1;
+flag = 1;
 if (found) 
   {
   InitialDataSetup(m_size, n_size, A, InitRays, InitRayIndex, OrderVector);
@@ -279,9 +278,14 @@ if (found)
     save_adj(data, A);
     save_facet(data);
     init_facet(data);
+    flag = 0;
     }
-  FreeDDMemory(OrderVector);
   } 
+
+/* sdf - memory leak fix */
+FreeDDMemory(OrderVector);
+free_Bmatrix(n_size, InitRays);
+
 return(flag);
 }
 
