@@ -18,6 +18,7 @@
 #include <time.h>
 #include <math.h>
 #include <string.h>
+#include <glib.h>
 
 /* FIXME put in cdd.h */
 extern long mprev;
@@ -349,14 +350,12 @@ void UpdateEdges(rowrange m_size, colrange n_size,
 _L99:;  
 }
 
-/* CURRENT - original author forgot to NULLify all sets */
-/* this causes core dumps when GDIS calls cdd a second time */
-/* and cdd tries to free non NULL pointers or fails to init properly */
+/* sdf - NULLify data to avoid core dumps when GDIS calls cdd a second time */
 void FreeDDMemory(rowindex ordervec)
 {
 RayRecord *Ptr, *PrevPtr;
 long count;
-  
+
 PrevPtr=ArtificialRay;
 count=0;
 for (Ptr=ArtificialRay->Next; Ptr!=NULL; Ptr=Ptr->Next)
@@ -367,26 +366,33 @@ for (Ptr=ArtificialRay->Next; Ptr!=NULL; Ptr=Ptr->Next)
   count++;
   PrevPtr=Ptr;
   }
+
+/* sdf - memory leak fix */
+free(LastRay->Ray);
+free(LastRay->ZeroSet);
+free(LastRay);
+
 LastRay=NULL;
 FirstRay=NULL;
 ArtificialRay=NULL;
-  
+ 
 set_free(InitialHyperplanes);
 set_free(AddedHyperplanes);
+set_free(WeaklyAddedHyperplanes);
 set_free(GroundSet);
 set_free(Face);
 set_free(Face1);
 free(ordervec);
 
-/* CURRENT - author forgot about these... see SetInequalitySets() */
+/* sdf - memory leak fix */
 set_free(EqualitySet);
 set_free(NonequalitySet);
 GroundSet=NULL;
 EqualitySet=NULL;
 NonequalitySet=NULL;
+
 /* These don't matter (I think) */
 /*
-free(EqualityIndex);
 InitialHyperplanes=NULL;
 AddedHyperplanes=NULL;
 Face=NULL;
@@ -619,21 +625,18 @@ void GausianColumnPivot2(rowrange m_size, colrange n_size,
 
 void InitializeBmatrix(colrange n_size, Bmatrix T)
 {
-  colrange j;
-  double x;
+colrange j;
 
-  for (j = 0; j < n_size; j++) {
-    T[j]=(double *)calloc(n_size, sizeof x);
-  }
+for (j=n_size ; j-- ; )
+  T[j] = g_malloc(n_size * sizeof(double));
 }
 
 void free_Bmatrix(colrange n_size, Bmatrix T)
 {
-  colrange j;
+colrange j;
 
-  for (j = 0; j < n_size; j++) {
-    free(T[j]);
-  }
+for (j=n_size ; j-- ; )
+  g_free(T[j]);
 }
 
 void SetToIdentity(colrange n_size, Bmatrix T)
@@ -805,6 +808,7 @@ void ComputeRank(rowrange m_size, colrange n_size,
   set_initialize(&NoPivotRow, m_size);
   set_compl(NoPivotRow,TargetRows);
   set_initialize(&ColSelected, n_size);
+printf("Btemp\n");
   InitializeBmatrix(n_size, Btemp);
   SetToIdentity(n_size, Btemp);
   do {   /* Find a set of rows for a basis */
@@ -891,8 +895,10 @@ void FindBasis(rowrange m_size, colrange n_size, Amatrix A1,
       if (*rank==n_size) stop = TRUE;
   } while (!stop);
   set_free(ColSelected);
-}
 
+/* sdf - memory leak fix */
+set_free(NopivotRow);
+}
 
 
 void FindInitialRays(rowrange m_size, colrange n_size, Amatrix A, rowindex ordervec,
@@ -926,12 +932,16 @@ void FindInitialRays(rowrange m_size, colrange n_size, Amatrix A, rowindex order
     return;
   }
   *found = TRUE;
-  /* free_Bmatrix(n_size, InitRays);  */
-  CopyBmatrix(n_size, BInverse,InitRays);
+
+/* sdf - memory leak fix */
+  free_Bmatrix(n_size, InitRays);
+  CopyBmatrix(n_size, BInverse, InitRays);
+
   set_free(CandidateRows);
   if (HyperplaneOrder==MaxCutoff||HyperplaneOrder==MinCutoff||HyperplaneOrder==MixCutoff){
     PreOrderedRun=FALSE;
   } else PreOrderedRun=TRUE;
+
 }
 
 void CheckEquality(colrange n_size, RayRecord **RP1, RayRecord **RP2, boolean *equal)
