@@ -25,6 +25,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include <strings.h>
 #include <stdlib.h>
 #include <math.h>
+#include <GL/gl.h>
 
 #include "gdis.h"
 #include "coords.h"
@@ -36,293 +37,53 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "morph.h"
 #include "opengl.h"
 #include "select.h"
+#include "zone.h"
 
 /* data structures */
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
-/*****************************/
-/* get the core's zone index */
-/*****************************/
-/* FIXME - if out of bounds -> need a re-zone & then return the index as well */
-#define DEBUG_GET_ZI 0
-gint get_zone_index(struct core_pak *core, struct model_pak *model)
+/***********************************************/
+/* construct a list of bonded atoms for a core */
+/***********************************************/
+GSList *con_core_bond_list(struct core_pak *core)
 {
-gint i, zi, grid[3];
-gdouble x[3];
+GSList *blist, *clist=NULL;
+struct bond_pak *bond;
 
-/* pbc constrain */
-ARR3SET(x, core->x);
-
-#if DEBUG_GET_ZI
-P3VEC("x1: ", x);
-#endif
-
-/* NB: grid is just used as a dummy variable here */
-fractional_clamp(x, grid, model->periodic);
-
-#if DEBUG_GET_ZI
-P3VEC("x2: ", x);
-#endif
-
-/* get the grid index */
-for (i=0 ; i<3 ; i++)
-  grid[i] = (x[i]-model->zone_min[i])/model->zone_dx[i];
-
-#if DEBUG_GET_ZI
-printf(" g: %d %d %d\n", grid[0], grid[1], grid[2]);
-#endif
-
-/* get the zone index */
-zi = grid[2]*model->zone_div[1]*model->zone_div[0]; 
-zi += grid[1]*model->zone_div[0]; 
-zi += grid[0];
-
-return(zi);
-}
-
-/*****************************/
-/* get the core's zone index */
-/*****************************/
-struct zone_pak *get_zone(struct core_pak *core, struct model_pak *model)
-{
-gint zi;
-
-if (!model->zones)
-  return(NULL);
-
-/* if out of bounds -> re-zoning needed */
-zi = get_zone_index(core, model);
-if (zi < 0 || zi >= model->num_zones)
+for (blist=core->bonds ; blist ; blist=g_slist_next(blist))
   {
-  con_init_zones(model);
-  zi = get_zone_index(core, model);
-  if (zi < 0 || zi >= model->num_zones)
-    g_assert_not_reached();
-  }
+  bond = (struct bond_pak *) blist->data;
 
-return(model->zones[zi]);
+  if (bond->atom1 == core)
+    clist = g_slist_prepend(clist, bond->atom2);
+  else
+    clist = g_slist_prepend(clist, bond->atom1);
+  }
+return(clist);
 }
 
-/******************/
-/* zone debugging */
-/******************/
-void con_info_zones(struct model_pak *model)
+/*************************************/
+/* determine if two cores are bonded */
+/*************************************/
+gint con_cores_bonded(struct core_pak *core1, struct core_pak *core2)
 {
-gint i, j;
-struct zone_pak *zone;
-
-printf("con_info_zones(%s)\n", model->filename);
-
-for (i=0 ; i<3 ; i++)
-  printf("[%d] : %11.4f - %11.4f  (%f)(%d)\n",
-          i, model->zone_min[i], model->zone_max[i],
-          model->zone_dx[i], model->zone_div[i]);
-
-for (i=0 ; i<model->num_zones ; i++)
-  {
-  zone = (struct zone_pak *) model->zones[i];
-  j = g_slist_length(zone->cores);
-  if (j)
-    printf("zone %d: %d cores.\n", i, j);
-  }
-}
-
-/********************************/
-/* free space partitioning data */
-/********************************/
-void con_free_zones(struct model_pak *model)
-{
-gint i;
-struct zone_pak *zone;
-
-/* free zone array core lists */
-for (i=0 ; i<model->num_zones ; i++)
-  {
-  zone = model->zones[i];
-
-  g_slist_free(zone->cores);
-  }
-
-/* free the zone array */
-if (model->zones)
-  g_free(model->zones);
-
-model->zones = NULL;
-model->num_zones = 0;
-}
-
-/*********************************/
-/* initialize space partitioning */
-/*********************************/
-/* after this distance - connectivity is no longer considered (worst case) */
-#define ZONE_SIZE 5.0
-#define DEBUG_ZONE_INIT 0
-void con_init_zones(struct model_pak *model)
-{
-gint i, j, k, zi, num_zones;
-gint div[3];
-gdouble dx[3], min[3], max[3];
 GSList *list;
-struct core_pak *core;
-struct zone_pak *zone;
+struct bond_pak *bond;
 
-#if DEBUG_ZONE_INIT
-printf("con_init_zones(%s) [%d D]\n", model->filename, model->periodic);
-#endif
-
-con_free_zones(model);
-
-/* FIXME - will there be problem (for periodic models) */
-/* if atoms are not constrained ie in the range [0-1) */
-/* TODO - allow zones "outside" pbc, but when comparing -> clamp to within */
-
-/* get the (possibly mixed) frac/cart coord limits */
-VEC3SET(min, 0.0, 0.0, 0.0);
-VEC3SET(max, 0.0, 0.0, 0.0);
-cor_calc_xlimits(min, max, model->cores);
-
-/* set cartesian number of divisions */
-for (i=model->periodic ; i<3 ; i++)
+for (list=core1->bonds ; list ; list=g_slist_next(list))
   {
-/* safety margin */
-  max[i] += 0.5;
-  min[i] -= 0.5;
+  bond = (struct bond_pak *) list->data;
 
-  dx[i] = max[i] - min[i];
-  div[i] = 0.5 + dx[i]/ZONE_SIZE;
+  if (bond->atom1 == core2 || bond->atom2 == core2)
+    return(TRUE);
   }
-
-/* set fractional number of divisions */
-for (i=model->periodic ; i-- ; )
-  {
-  min[i] = 0.0;
-  max[i] = 1.0;
-  dx[i] = model->pbc[i] / ZONE_SIZE;
-  div[i] =  dx[i];
-  dx[i] = 1.0;
-  }
-
-/* division sizes */
-for (i=3; i--; )
-  {
-  if (!div[i])
-    div[i] = 1;
-  dx[i] /=  div[i];
-  }
-
-#if DEBUG_ZONE_INIT
-for (i=0 ; i<3 ; i++)
-  printf("[%d] : %11.4f - %11.4f  (%f)(%d)\n", i, min[i], max[i], dx[i], div[i]);
-#endif
-
-/* TODO - if periodic - extend zone by 1 in all directions (unfragment) */
-/* ie div +=2 , min -= dx, max += dx */
-/* TODO - OR (better) if we can just constrain to get the equivalent existing zone */
-
-ARR3SET(model->zone_min, min);
-ARR3SET(model->zone_max, max);
-ARR3SET(model->zone_div, div);
-ARR3SET(model->zone_dx, dx);
-
-num_zones = div[0] * div[1] * div[2];
-model->num_zones = num_zones;
-
-#if DEBUG_ZONE_INIT
-printf("Allocating %dx%dx%d = %d zones...\n", div[0], div[1], div[2], num_zones);
-#endif
-
-model->zones = g_malloc(num_zones * sizeof(struct zone_pak *));
-
-#if DEBUG_ZONE_INIT
-printf("Initializing %d zones...\n", num_zones);
-#endif
-
-for (k=0 ; k<div[2] ; k++)
-  {
-  for (j=0 ; j<div[1] ; j++)
-    {
-    for (i=0 ; i<div[0] ; i++)
-      {
-      zi = k*div[1]*div[0]; 
-      zi += j*div[0]; 
-      zi += i;
-
-      model->zones[zi] = g_malloc(sizeof(struct zone_pak));
-
-      zone = (struct zone_pak *) model->zones[zi];
-      zone->cores = NULL;
-      VEC3SET(zone->grid, i, j, k);
-      }
-    }
-  }
-
-/* TODO - check if no zonal division is required */
-/* NB: only for isolated molecules, as periodic will */
-/* always have zones to make unfragmenting easier */
-
-#if DEBUG_ZONE_INIT
-printf("Partioning %d cores...\n", g_slist_length(model->cores));
-#endif
-
-/* loop over all cores */
-for (list=model->cores ; list ; list=g_slist_next(list))
-  {
-  core = list->data;
-
-/* zone index */
-  zi = get_zone_index(core, model);
-  if (zi < 0 || zi >= model->num_zones)
-    g_assert_not_reached();
-
-/* add to the list */
-  zone = (struct zone_pak *) model->zones[zi];
-  zone->cores = g_slist_prepend(zone->cores, core);
-
-#if DEBUG_ZONE_INIT
-/* region colour by zone */
-core->region = zi;
-/*
-printf("grid coords: %d %d %d  ", grid[0], grid[1], grid[2]);
-printf("region: %d\n", zi); 
-*/
-#endif
-  }
-
-for (i=0 ; i<num_zones ; i++)
-  {
-  zone = (struct zone_pak *) model->zones[i];
-#if DEBUG_ZONE_INIT
-  j = g_slist_length(zone->cores);
-  if (j)
-    printf("zone %d: %d cores.\n", i, j);
-#endif
-  }
-}
-
-/************************************/
-/* get surrounding zones for a core */
-/************************************/
-/* TODO - also specify an optional buffer */
-/* TODO - what about periodic images (need an xlat returned as well */
-GSList *con_core_zones(struct core_pak *core, struct model_pak *model)
-{
-GSList *zlist=NULL;
-
-/*
-for (list=model->zone_list ; list ; list=g_slist_next(list))
-  {
-  zone[i] = (core->x[i] - min[i]) / dx[i];
-  }
-*/
-
-return(zlist);
+return(FALSE);
 }
 
 /****************************/
 /* display hydrogen bonding */
 /****************************/
-/* FIXME - this only need be called when check box is toggled */
 void build_hbonds(struct model_pak *model)
 {
 gint status;
@@ -395,40 +156,41 @@ for (list=data->cores ; list ; list=g_slist_next(list))
 /* build polyhedral style connectivity */
 /***************************************/
 #define DEBUG_BUILD_POLYHEDRAL 0
-void build_polyhedral(struct model_pak *data)
+void create_polyhedra(struct model_pak *data)
 {
 gint a, b, c, k, n, flag;
 gdouble angle, vec1[3], vec2[3], vec3[3], vec4[3];
-GSList *list1, *list2, *clist, *slist;
+GSList *list1, *list2, *clist, *slist, *mlist;
 struct spatial_pak *spatial;
-struct vec_pak *p1, *p2, *p3;
+struct vec_pak *p1, *p2, *p3, *v1, *v2, *v3;
 struct core_pak *core1;
 struct bond_pak *bond;
 
-for (list1=data->cores ; list1 ; list1=g_slist_next(list1))
+/* if selection, apply only to those cores, else all cores */
+if (data->selection)
+  mlist = data->selection;
+else
+  mlist = data->cores;
+
+/* iterate over candidate center atoms */
+for (list1=mlist ; list1 ; list1=g_slist_next(list1))
   {
   core1 = (struct core_pak *) list1->data;
-
-  switch (core1->atom_code)
-    {
-/* allowed center atoms */
-    case 13:
-    case 14:
-    case 25:
-      break;
-    default:
-      continue;
-    }
   ARR3SET(vec1, core1->x);
 
-/* search for surrounding O's */
+/* search for bonds */
   n=0;
   clist=NULL;
   for (list2=core1->bonds ; list2 ; list2=g_slist_next(list2))
     {
     bond = (struct bond_pak *) list2->data;
 
-    if ((bond->atom1)->atom_code == 8)
+/* NEW - ignore bonds between same atom types (both candidate center atoms) */
+    if ((bond->atom1)->atom_code == (bond->atom2)->atom_code)
+      continue;
+
+/* find the candidate polyhedral vertex */
+    if (core1 == bond->atom2)
       {
       p1 = g_malloc(sizeof(struct vec_pak));
       ARR3SET(p1->x, (bond->atom1)->x);
@@ -441,9 +203,10 @@ for (list1=data->cores ; list1 ; list1=g_slist_next(list1))
       clist = g_slist_prepend(clist, p1);
       n++;
       }
-
-    if ((bond->atom2)->atom_code == 8)
+    else
       {
+g_assert(core1 == bond->atom1);
+
       p1 = g_malloc(sizeof(struct vec_pak));
       ARR3SET(p1->x, (bond->atom2)->x);
 
@@ -477,7 +240,8 @@ printf("This [%s] has %d candidates.\n", core1->label, n);
 
 /* spatial */
       spatial = g_malloc(sizeof(struct spatial_pak));
-      spatial->type = SPATIAL_POLYGON;
+      spatial->type = SPATIAL_POLYHEDRA;
+      spatial->method = GL_POLYGON;
 /* central atom's colour */
       ARR3SET(p1->colour, core1->colour);
       VEC3MUL(p1->colour, 1.0/65535.0);
@@ -555,21 +319,37 @@ printf("This [%s] has %d candidates.\n", core1->label, n);
       ARR3SUB(vec2, p1->x);
       VEC3MUL(vec2, -1.0);
       angle = via(vec2, vec4, 3);
+
+/* copy the vertices (since they may be free'd individually later) */
+/* TODO - implement copy/duplicate etc. vector method */
+      v1 = g_malloc(sizeof(struct vec_pak));
+      v2 = g_malloc(sizeof(struct vec_pak));
+      v3 = g_malloc(sizeof(struct vec_pak));
+      ARR3SET(v1->x, p1->x);
+      ARR3SET(v2->x, p2->x);
+      ARR3SET(v3->x, p3->x);
+      ARR3SET(v1->n, p1->n);
+      ARR3SET(v2->n, p2->n);
+      ARR3SET(v3->n, p3->n);
+      ARR3SET(v1->colour, p1->colour);
+      ARR3SET(v2->colour, p2->colour);
+      ARR3SET(v3->colour, p3->colour);
+
 /* enforce an outwards pointing clockwise normal */
       if (angle < PI/2.0)
         {
-        spatial->data = g_slist_append(spatial->data, p1);
-        spatial->data = g_slist_append(spatial->data, p2);
-        spatial->data = g_slist_append(spatial->data, p3);
+        spatial->data = g_slist_append(spatial->data, v1);
+        spatial->data = g_slist_append(spatial->data, v2);
+        spatial->data = g_slist_append(spatial->data, v3);
         }
       else
         {
-        spatial->data = g_slist_append(spatial->data, p1);
-        spatial->data = g_slist_append(spatial->data, p3);
-        spatial->data = g_slist_append(spatial->data, p2);
-VEC3MUL(p1->n, -1.0);
-VEC3MUL(p2->n, -1.0);
-VEC3MUL(p3->n, -1.0);
+        spatial->data = g_slist_append(spatial->data, v1);
+        spatial->data = g_slist_append(spatial->data, v3);
+        spatial->data = g_slist_append(spatial->data, v2);
+        VEC3MUL(v1->n, -1.0);
+        VEC3MUL(v2->n, -1.0);
+        VEC3MUL(v3->n, -1.0);
         }
       }
 
@@ -580,16 +360,8 @@ VEC3MUL(p3->n, -1.0);
     else
       free_slist(slist);
     }
-  else
-    {
-/* not used, so free completely */
-    free_slist(clist);
-    }
-
-/* FIXME - this can cause a core dump (poly bonds on bad_models/test.gin) */
-/*
+/* free all vertices */
   g_slist_free(clist);
-*/
   }
 }
 
@@ -748,7 +520,10 @@ gdouble x1[3], x2[3], pic[3], xlat[3];
 GSList *list;
 struct bond_pak *bond;
 struct core_pak *comp;
+/*
 struct zone_pak *zone, *zone2;
+*/
+gpointer zone, zone2;
 
 /* allow connectivity to be turned off */
 if (!data->build_molecules)
@@ -758,18 +533,20 @@ if (!data->build_molecules)
 ri = core->bond_cutoff;
 
 /* get the core's zone */
-zone = get_zone(core, data);
+zone = zone_ptr(core, data);
 g_assert(zone != NULL);
 
 #if DEBUG_ATOM_BONDS
+/*
 printf("current core [%s] [%f %f %f] :  zone = [%d %d %d]\n",
 core->label, core->x[0], core->x[1], core->x[2],
 zone->grid[0], zone->grid[1], zone->grid[2]);
+*/
 #endif
 
 /* setup scan limits */
-ARR3SET(start, zone->grid);
-ARR3SET(stop, zone->grid);
+ARR3SET(start, zone_grid(zone));
+ARR3SET(stop, start);
 /* standard - add a single zone around the edge */
 VEC3SUB(start, 1, 1, 1);
 VEC3ADD(stop, 1, 1, 1);
@@ -836,7 +613,7 @@ g_assert(zi < data->num_zones);
 
 /* loop over cores in the corresponding zone */
       zone2 = data->zones[zi];
-      for (list=zone2->cores ; list ; list=g_slist_next(list))
+      for (list=zone_cores(zone2) ; list ; list=g_slist_next(list))
         {
 
     comp = (struct core_pak *) list->data;
@@ -860,9 +637,10 @@ g_assert(zi < data->num_zones);
     fractional_clamp(x2, m2, data->periodic);
 
 #if DEBUG_ATOM_BONDS
-printf("m1 [%d %d %d]", m1[0], m1[1], m1[2]);
-printf("m2 [%d %d %d]", m2[0], m2[1], m2[2]);
-printf("x [%f %f %f]\n", pic[0], pic[1], pic[2]);
+printf("[%s ?< %f] : ", comp->label, r2cut);
+printf("m1 [%d %d %d] ", m1[0], m1[1], m1[2]);
+printf("m2 [%d %d %d] ", m2[0], m2[1], m2[2]);
+printf("x [%f %f %f] : ", pic[0], pic[1], pic[2]);
 #endif
 
 /* compare core positions */
@@ -873,8 +651,6 @@ printf("x [%f %f %f]\n", pic[0], pic[1], pic[2]);
     vecmat(data->latmat, x2);
     r2 = VEC3MAGSQ(x2);
 
-    if (r2 < 6.25)
-      {
       if (r2 < POSITION_TOLERANCE)
         {
 #if DEBUG_ATOM_BONDS
@@ -906,7 +682,8 @@ printf("Ignoring atoms that are too close!\n");
               case 7:
               case 8:
               case 9:
-                flag++;
+                if (r2 < 6.25)
+                  flag++;
                 break;
               }
             }
@@ -917,7 +694,8 @@ printf("Ignoring atoms that are too close!\n");
               case 7:
               case 8:
               case 9:
-                flag++;
+                if (r2 < 6.25)
+                  flag++;
                 break;
               }
             }
@@ -957,7 +735,6 @@ printf("Ignoring atoms that are too close!\n");
           comp->bonds = g_slist_prepend(comp->bonds, bond);
           }
         }
-      }
         }
       }
     }
@@ -1180,7 +957,6 @@ void wipe_bonds(struct model_pak *data)
 GSList *list;
 struct bond_pak *bond;
 struct core_pak *core1, *core2;
-struct spatial_pak *spatial;
 
 list = data->bonds;
 while (list)
@@ -1197,20 +973,6 @@ while (list)
   g_free(bond);
   }
 data->bonds=NULL;
-
-/* clear all polygons */
-/* FIXME - what if they're NOT part of a polyhedral bond construct */
-list = data->spatial;
-while (list)
-  {
-  spatial = (struct spatial_pak *) list->data;
-  list = g_slist_next(list);
-  if (spatial->type == SPATIAL_POLYGON)
-    {
-    g_slist_free(spatial->data);
-    data->spatial = g_slist_remove(data->spatial, spatial);
-    }
-  }
 }
 
 /*********/
@@ -1227,8 +989,10 @@ printf("calc_bonds(%d)\n", data->cur_frame);
 printf("calc_bonds(%d)\n", data->build_molecules);
 #endif
 
-/* TODO - have a flag that indicates if this needs doing */
-con_init_zones(data);
+g_assert(data->zones != NULL);
+
+if (data->anim_fix)
+  return;
 
 /* TODO - delete polyhedra */
 /* redo atom connectivity */
@@ -1251,8 +1015,10 @@ build_hbonds(data);
 if (data->build_zeolite)
   build_zeolite(data);
 
+/*
 if (data->build_polyhedra)
   build_polyhedral(data);
+*/
 
 #if DEBUG_CALC_BONDS
 dump_bonds(data);
@@ -1308,18 +1074,19 @@ data->moles=NULL;
 #define DEBUG_CALC_MOL_CENT 0
 void calc_mol_centroids(struct model_pak *data)
 {
-gint m, n;
+gint n;
+gdouble scale;
 GSList *list, *list2;
 struct mol_pak *mol;
 struct core_pak *core;
 
 /* centroid calc & core->mol reference */
-m=n=0;
 for (list=data->moles ; list ; list=g_slist_next(list))
   {
   mol = (struct mol_pak *) list->data;
   VEC3SET(mol->centroid, 0.0, 0.0, 0.0);
   
+  n=0;
   for (list2=mol->cores ; list2 ; list2=g_slist_next(list2))
     {
     core = (struct core_pak *) list2->data;
@@ -1328,20 +1095,47 @@ for (list=data->moles ; list ; list=g_slist_next(list))
 
 #if DEBUG_CALC_MOL_CENT
     printf("[%2d]  [%4s]  [%10.4f %10.4f %10.4f]\n",
-           m, core->label, core->x[0], core->x[1], core->x[2]);
+           core->label, core->x[0], core->x[1], core->x[2]);
 #endif
 
     ARR3ADD(mol->centroid, core->x);
     n++;
     }
 
-  VEC3MUL(mol->centroid, 1.0/g_slist_length(mol->cores));
-  m++;
+  if (n)
+    {
+    scale = 1.0 / (gdouble) n;
+    VEC3MUL(mol->centroid, scale);
+    }
   }
+}
 
-if (n != g_slist_length(data->cores))
-  printf("calc_mols() error: core # mismatch %d:%d\n",
-                      n, g_slist_length(data->cores));
+/*********************************/
+/* sort molecule cores primitive */
+/*********************************/
+gint sort_core_order(gpointer ptr_core1, gpointer ptr_core2)
+{
+struct core_pak *core1 = ptr_core1, *core2 = ptr_core2;
+
+if (core1->atom_order < core2->atom_order)
+  return(-1);
+return(1);
+}
+
+/*********************************************************************/
+/* sort cores in molecules into the same order as the main core list */
+/*********************************************************************/
+void sort_mol_cores(struct model_pak *model)
+{
+GSList *list;
+struct mol_pak *mol;
+
+for (list=model->moles ; list ; list=g_slist_next(list))
+  {
+  mol = list->data;
+
+  mol->cores = g_slist_sort(mol->cores, (gpointer) sort_core_order);
+  }
 }
 
 /**************************/
@@ -1351,6 +1145,7 @@ if (n != g_slist_length(data->cores))
 void calc_mols(struct model_pak *data)
 {
 gint n=0;
+guint i=0;
 GSList *list, *blist, *mlist, *list1, *list2;
 struct core_pak *core, *core2;
 struct bond_pak *bond;
@@ -1361,10 +1156,13 @@ g_assert(data != NULL);
 /* clean */
 free_mol_list(data);
 
+/* init */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
   core->status &= ~PRUNED;
+/* NEW - record core order in list */
+  core->atom_order = i++;
   }
 
 /* build mol lists */
@@ -1440,7 +1238,6 @@ for (list=data->cores ; list ; list=g_slist_next(list))
   data->moles = g_slist_prepend(data->moles, mol);
 
 /* label the cores as belonging to the mol */
-/* NB: don't do this as unfragment() requires fragments to be in separate molecule */
   for (list1=mol->cores ; list1 ; list1=g_slist_next(list1))
     {
     core = (struct core_pak *) list1->data;
@@ -1454,7 +1251,8 @@ dump_mols(data);
 
 calc_mol_centroids(data);
 
-return;
+/* NEW - ensure core ordering in molecules is the same as in the main list */
+sort_mol_cores(data);
 }
 
 /**********************************************/
@@ -1518,6 +1316,8 @@ for (list1=mol2->cores ; list1 ; list1=g_slist_next(list1))
     if (bond == bond2)
       continue;
 
+g_assert(bond2 != NULL);
+
 /* FIXME - strictly, we should test if this heals more than */
 /* just the current bond. would need to compare pics's to */
 /* test this tho' ... */
@@ -1579,7 +1379,7 @@ VEC3SET(bond->pic, 0.0, 0.0, 0.0);
 void unfragment(struct model_pak *data)
 {
 gint m, n, direction, merge;
-GSList *list1, *list2;
+GSList *list1, *list2, *broken_bonds;
 struct mol_pak *mol;
 struct bond_pak *bond;
 struct core_pak *core1, *core2;
@@ -1591,6 +1391,17 @@ dump_bonds(data);
 if (!data->periodic)
   return;
 
+
+/* preconstruct broken bond list */
+broken_bonds = NULL;
+for (list1=data->bonds ; list1 ; list1=g_slist_next(list1))
+  {
+  bond = (struct bond_pak *) list1->data;
+  if (bond->type != BOND_HBOND && bond->periodic == BOND_SPLIT)
+    broken_bonds = g_slist_prepend(broken_bonds, bond);  
+  }
+
+
 /* seek periodic bonds containing atoms in the current mol */
 list1 = data->moles;
 while (list1)
@@ -1599,17 +1410,23 @@ while (list1)
   merge = 0;
 
 /* search for a periodic bond that links to mol */
+/*
   list2 = data->bonds;
+*/
+
+  list2 = broken_bonds;
   while (list2)
     {
     bond = (struct bond_pak *) list2->data;
     list2 = g_slist_next(list2);
 
 /* ignore these */
+/*
     if (bond->type == BOND_HBOND)
       continue;
 
     if (bond->periodic == BOND_SPLIT)
+*/
       {
       core1 = bond->atom1;
       core2 = bond->atom2;
@@ -1710,11 +1527,11 @@ printf("--- unfrag end\n");
 dump_bonds(data);
 */
 
+sort_mol_cores(data);
+
 #if DEBUG_UNFRAGMENT
 printf("[*] molecules: %d\n", g_slist_length(data->moles));
 #endif
-
-return;
 }
 
 /*********************************/
@@ -1724,7 +1541,7 @@ void connect_refresh(void)
 {
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 
 calc_bonds(model);
 calc_mols(model);
@@ -1739,7 +1556,7 @@ void cb_connect_confine(GtkWidget *w, gint mode)
 {
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 
 switch (mode)
   {
@@ -1763,7 +1580,7 @@ void cb_connect_unfrag(void)
 {
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 
 unfragment(model);
 
@@ -1784,7 +1601,7 @@ GtkWidget *frame;
 struct model_pak *data;
 
 /* do we have to do anything? */
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -1803,17 +1620,19 @@ vbox = gtk_vbox_new(TRUE, PANEL_SPACING);
 gtk_container_add(GTK_CONTAINER(frame), vbox);
 gtk_container_set_border_width(GTK_CONTAINER(frame), PANEL_SPACING/2);
 
-
 gtksh_auto_check("Build molecules ",    connect_refresh, NULL,
                                          &data->build_molecules, vbox);
+
+/*
 gtksh_auto_check("Polyhedral bonding ", connect_refresh, NULL,
                                          &data->build_polyhedra, vbox);
+*/
+
 gtksh_auto_check("Zeolite bonding ",    connect_refresh, NULL,
                                          &data->build_zeolite, vbox);
 gtksh_auto_check("Hydrogen bonding ",   connect_refresh, NULL,
                                          &data->build_hydrogen, vbox);
 
-
 /* pane */
 vbox2 = gtk_vbox_new(FALSE, 0);
 gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
