@@ -28,14 +28,17 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "edit.h"
 #include "interface.h"
 #include "matrix.h"
+#include "measure.h"
 #include "spatial.h"
 #include "surface.h"
 #include "numeric.h"
 #include "morph.h"
 #include "opengl.h"
 #include "select.h"
+#include "zone.h"
 
 /* data structures */
 extern struct sysenv_pak sysenv;
@@ -88,7 +91,7 @@ if (i==4)
     {
     shell = (struct shel_pak *) core->shell;
 
-    printf("[%4s] %1d %1d  s %10.4f %10.4f %10.4f : %6.2f\n",
+    printf("[%4s] %1d %1d s %10.4f %10.4f %10.4f : %6.2f\n",
             shell->label, shell->primary, shell->orig,
             shell->x[0], shell->x[1], shell->x[2], shell->charge);
     }
@@ -200,23 +203,103 @@ VEC3SET(data->cell[6].x, 1.0, b, c);
 VEC3SET(data->cell[7].x, 1.0, 0.0, c);
 }
 
+
+/*********************************/
+/* construct core-shell linkages */
+/*********************************/
+#define DEBUG_SHELLS 0
+#define MAX_SHELL_DIST 0.6
+void shell_make_links(struct model_pak *model)
+{
+gint i, match, total;
+gchar *text;
+gdouble min, sep, vec[3];
+GSList *list1, *list2, *locality;
+struct core_pak *core;
+struct shel_pak *shel;
+
+/* checks */
+g_assert(model != NULL);
+g_assert(model->zones != NULL);
+
+/* enumerate all shells */
+total = 0;
+for (list1=model->shels ; list1 ; list1=g_slist_next(list1))
+  {
+  shel = (struct shel_pak *) list1->data;
+
+/* get neighbourhood core list for the shell */
+  i = zone_index(shel->x, model);
+  locality = zone_area_cores(1, model->zones[i], model);
+
+/* upper bound for core-shell link */
+  min = MAX_SHELL_DIST*MAX_SHELL_DIST;
+
+/* enumerate cores in the shell's vicinity */
+  match = 0;
+  for (list2=locality ; list2 ; list2=g_slist_next(list2))
+    {
+    core = (struct core_pak *) list2->data;
+
+    if (shel->atom_code == core->atom_code)
+      {
+/* get the minimum fractional separation */
+      ARR3SET(vec, core->x);
+      ARR3SUB(vec, shel->x);
+      fractional_minsq(vec, model->periodic);
+/* convert to cartesian & compare with cutoff */
+      vecmat(model->latmat, vec);
+      sep = VEC3MAGSQ(vec);
+      if (sep < min)
+        {
+/* FIXME - what to do with previous matches? eg set (core->shell)->shell = NULL ? */
+/* reference each other */
+        core->shell = shel;
+        shel->core = core;
+        match++;
+/* keep track of the minimum, so only the closest pair is recorded */
+        min = sep;
+        }
+      }
+    }
+/* record any shells with no linkages */
+  if (!match)
+    total++;
+/* free the constructed locality */
+  g_slist_free(locality);
+  }
+
+/* found some floating shells? */
+if (total)
+  {
+  text = g_strdup_printf("Warning: found %d shell(s) with no cores.\n", total);
+  show_text(WARNING, text);
+  g_free(text);
+  }
+}
+
 /**********/
 /* SHELLS */
 /**********/
-#define DEBUG_SHELLS 0
 void update_shells(struct model_pak *data)
 {
-gint tot_match, match, dummy[3];
-gdouble min, sep, dim[3], vec[3];
+gint tot_match, match;
+gdouble min, sep, vec[3];
 GSList *core_list, *shel_list;
 struct core_pak *core;
 struct shel_pak *shel;
 
-/* pbc calc with periodic image correction */
-dim[0] = data->image_limit[0] + data->image_limit[1];
-dim[1] = data->image_limit[2] + data->image_limit[3];
-dim[2] = data->image_limit[4] + data->image_limit[5];
 
+/* CURRENT */
+shell_make_links(data);
+return;
+
+
+
+/* checks */
+g_assert(data != NULL);
+
+/* find core-shell connections */
 tot_match = 0;
 for (core_list=data->cores ; core_list ; core_list=g_slist_next(core_list))
   {
@@ -235,28 +318,16 @@ for (core_list=data->cores ; core_list ; core_list=g_slist_next(core_list))
 /* get the absolute fractional separation */
       ARR3SET(vec, core->x);
       ARR3SUB(vec, shel->x);
-      VEC3ABS(vec);
 
-/* get the minimum fractional separation */
-fractional_clamp(vec, dummy, data->periodic);
-      switch(data->periodic)
-        {
-        case 3:
-          while(vec[2] > dim[2]/2.0)
-            vec[2] -= 1.0;
-        case 2:
-          while(vec[0] > dim[0]/2.0)
-            vec[0] -= 1.0;
-          while(vec[1] > dim[1]/2.0)
-            vec[1] -= 1.0;
-          break;
-        }
+/* NEW - get the minimum fractional separation */
+      fractional_minsq(vec, data->periodic);
 
 /* convert to cartesian & compare with cutoff */
       vecmat(data->latmat, vec);
       sep = VEC3MAGSQ(vec);
       if (sep < min)
         {
+/* FIXME - what to do with previous matches? eg set (core->shell)->shell = NULL ? */
 /* reference each other */
         core->shell = shel;
         shel->core = core;
@@ -271,184 +342,187 @@ fractional_clamp(vec, dummy, data->periodic);
   }
 
 #if DEBUG_SHELLS
-printf("update_shells(): found %d core-shell connections.\n", tot_match);
+printf("core-shell connections: %d/%d\n", tot_match, g_slist_length(data->shels));
 #endif
 
 /* do some shells float down here? */
 if (tot_match != g_slist_length(data->shels))
   {
 /* TODO - somehow only print this once for a model */
-show_text(WARNING, "update_shells() warning: some shell(s) have no cores.\n");
-  }
-}
-
-/*************************/
-/* core matching routine */
-/*************************/
-struct core_pak *seek_core(struct core_pak *core, gint frac, struct model_pak *model)
-{
-gint dummy[3];
-gdouble vec[3];
-GSList *list;
-struct core_pak *comp;
-struct zone_pak *zone;
-
-/* scan cores in the same zone */
-zone = get_zone(core, model);
-for (list=zone->cores ; list ; list=g_slist_next(list))
+  show_text(WARNING, "Warning: missing core-shell connection(s).\n");
+#if DEBUG_SHELLS
+for (shel_list=data->shels ; shel_list ; shel_list=g_slist_next(shel_list))
   {
-  comp = (struct core_pak *) list->data;
-  if (comp->status & DELETED)
-    continue;
-
-/* don't compare self */
-  if (core != comp)
+  shel = (struct shel_pak *) shel_list->data;
+  if (!shel->core)
     {
-    if (frac)
-      {
-      ARR3SET(vec, core->x);
-      ARR3SUB(vec, comp->x);
-/* NEW */
-      fractional_clamp(vec, dummy, model->periodic);
-
-      if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE)
-        return(comp);
-      }
-    else
-      {
-      ARR3SET(vec, core->rx);
-      ARR3SUB(vec, comp->rx);
-      if (VEC3MAGSQ(vec) < POSITION_TOLERANCE)
-        return(comp);
-      }
+/* NEW - also calculate cartesian values */
+    ARR3SET(shel->rx, shel->x);
+    vecmat(data->latmat, shel->rx);
+    ARR3ADD(shel->rx, data->centroid);
+    print_shell(shel); 
     }
   }
-return(NULL);
-}
-
-/**************************/
-/* shell matching routine */
-/**************************/
-struct shel_pak *seek_shel(struct shel_pak *shell, gint frac, struct model_pak *model)
-{
-gint dummy[3];
-gdouble vec[3];
-GSList *list;
-struct shel_pak *comp;
-
-for (list=model->shels ; list ; list=g_slist_next(list))
-  {
-  comp = (struct shel_pak *) list->data;
-  if (comp->status & DELETED)
-    continue;
-
-/* don't compare self */
-  if (shell != comp)
-    {
-    if (frac)
-      {
-      ARR3SET(vec, shell->x);
-      ARR3SUB(vec, comp->x);
-/* NEW */
-      fractional_clamp(vec, dummy, model->periodic);
-
-      if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE*FRACTION_TOLERANCE)
-        return(comp);
-      }
-    else
-      {
-      ARR3SET(vec, shell->rx);
-      ARR3SUB(vec, comp->rx);
-      if (VEC3MAGSQ(vec) < POSITION_TOLERANCE*POSITION_TOLERANCE)
-        return(comp);
-      }
-    }
+#endif
   }
-return(NULL);
 }
 
-/*******************************/
-/* coordinate matching routine */
-/*******************************/
-struct core_pak *seek_coord(gdouble *x, gint frac, struct model_pak *data)
+/*********************************/
+/* coordinate matching routine 1 */
+/*********************************/
+/* done via 3D world coordinate matching */
+#define DEBUG_SEEK_COORD3D 0
+struct core_pak *seek_coord3d(gdouble *x, struct model_pak *data)
 {
-gdouble vec[3];
-GSList *list=NULL;
-struct core_pak *core;
-
-list = data->cores;
-while (list)
-  {
-  core = (struct core_pak *) list->data;
-
-  if (frac)
-    {
-    ARR3SET(vec, core->x);
-    ARR3SUB(vec, x);
-    if (VEC3MAGSQ(vec) < FRACTION_TOLERANCE*FRACTION_TOLERANCE)
-      return(core);
-    }
-  else
-    {
-    ARR3SET(vec, core->rx);
-    ARR3SUB(vec, x);
-    if (VEC3MAGSQ(vec) < POSITION_TOLERANCE*POSITION_TOLERANCE)
-      return(core);
-    }
-
-  list = g_slist_next(list);
-  }
-return(NULL);
-}
-
-/*******************************/
-/* coordinate matching routine */
-/*******************************/
-#define DEBUG_SEEK_COORD2D 0
-struct core_pak *seek_coord2d(gdouble *x, struct model_pak *data)
-{
-gdouble d2, tol, vec[3];
-GSList *list=NULL;
+gdouble d2, tol, vec[3], piv[3];
+GSList *list=NULL, *ilist=NULL;
 struct core_pak *core, *best=NULL;
+struct image_pak *image;
 
 /* tolerance (angstroms squared)  */
 tol = data->rmax * 20.0*data->scale/sysenv.size;
 
-#if DEBUG_SEEK_COORD2D
+#if DEBUG_SEEK_COORD3D
 printf(" size = %d\n", sysenv.size);
 printf("scale = %f\n", data->scale);
 printf(" rmax = %f\n", data->rmax);
 printf("  tol = %f\n", tol);
 #endif
 
+do
+  {
+  if (ilist)
+    {
+/* retrieve periodic image vector */
+    image = (struct image_pak *) ilist->data;
+    ARR3SET(piv, image->rx);
+    ilist = g_slist_next(ilist);
+    }
+  else
+    {
+    VEC3SET(piv, 0.0, 0.0, 0.0);
+    ilist = data->images;
+    }
+
+/* attempt to match core coords */
+  for (list=data->cores ; list ; list=g_slist_next(list))
+    {
+    core = (struct core_pak *) list->data;
+
+    ARR3SET(vec, core->rx);
+    ARR3ADD(vec, piv);
+    VEC3MUL(vec, data->scale);
+    ARR3SUB(vec, x);
+    d2 = vec[0]*vec[0] + vec[1]*vec[1];
+    if (d2 < tol)
+      {
+      best = core;
+      tol = d2;
+      }
+    }
+  }
+while (ilist);
+
+return(best);
+}
+
+/*********************************/
+/* coordinate matching routine 2 */
+/*********************************/
+/* done via 2D window coordinate matching */
+#define DEBUG_SEEK_COORD2D 0
+struct core_pak *seek_coord2d(gint x, gint y, struct model_pak *data)
+{
+gint tol, d2, w[2];
+gdouble vec[3];
+GSList *list=NULL;
+struct core_pak *core, *best=NULL;
+
+/* tolerance (pixels squared) */
+tol = 49;
+
+#if DEBUG_SEEK_COORD2D
+printf("search: [%d,%d]\n", x, y);
+#endif
+
+/* TODO - periodic images */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
 
   ARR3SET(vec, core->rx);
   VEC3MUL(vec, data->scale);
-  ARR3SUB(vec, x);
-  d2 = vec[0]*vec[0] + vec[1]*vec[1];
+
+  gl_get_window_coords(vec, w);
+
+#if DEBUG_SEEK_COORD2D
+printf("  (%d, %d)\n", w[0], w[1]);
+#endif
+
+  w[0] -= x;
+  w[1] -= y;
+
+  d2 = w[0]*w[0] + w[1]*w[1];
   if (d2 < tol)
     {
     best = core;
     tol = d2;
     }
   }
-
 return(best);
 }
 
+/******************************/
+/* vertex coordinate extremes */
+/******************************/
+void vertex_limits(gdouble *min, gdouble *max, struct model_pak *model)
+{
+gint i;
+gdouble x[3];
+GSList *list;
+struct vertex_pak *v;
+
+g_assert(model != NULL);
+
+/* failsafe limit init */
+VEC3SET(min, 0.0, 0.0, 0.0);
+VEC3SET(max, 0.0, 0.0, 0.0);
+list = model->vertices;
+if (!list)
+  return;
+
+/* failsafe limit init */
+v = list->data;
+ARR3SET(x, v->x);
+ARR3SET(min, x);
+ARR3SET(max, x);
+
+/* scan list and update limits */
+while (list)
+  {
+  v = list->data;
+
+  ARR3SET(x, v->x);
+
+  for (i=3 ; i-- ; )
+    {
+    if (x[i] < min[i])
+      min[i] = x[i];
+    if (x[i] > max[i])
+      max[i] = x[i];
+    }
+  list = g_slist_next(list);
+  }
+}
+
 /**********/
 /* COORDS */
 /**********/
-/* NEW - global = true -> whole model, else selection (if non-empty) */
-/* TODO - if not global - need to ignore some objects (eg cell/spatials) */
 #define DEBUG_CENT 0
 gint cent_coords(gint global, struct model_pak *data)
 {
 gint i, n;
-gdouble vec[3], r, r2, rmax;
+gdouble min[3], max[3], vec[3], r, r2, rmax;
 GSList *list=NULL, *ilist, *clist;
 struct image_pak *image;
 struct spatial_pak *spatial;
@@ -466,19 +540,7 @@ clist = data->cores;
 if (!global && g_slist_length(data->selection))
   clist = data->selection;
 
-/* centroid is always the origin, so just calc. scaling */
-for (list=data->vertices ; list ; list=g_slist_next(list))
-  {
-  v = (struct vertex_pak *) list->data;
-
-  ARR3SET(vec, v->x);
-  vecmat(data->latmat, vec);
-  r = VEC3MAGSQ(vec);
-  if (r > r2)
-    r2 = r; 
-  }
-
-/* coord centroid calc */
+/* centroid calc */
 n=0;
 for (list=clist ; list ; list=g_slist_next(list))
   {
@@ -500,9 +562,17 @@ else
     {
     VEC3SET(data->centroid, 0.5, 0.5, 0.5);
     }
+  else
+    {
+/* morphology centroid (NB: won't be the origin if non-centrosymetric) */
+    vertex_limits(min, max, data);
+    ARR3SET(data->centroid, min);
+    ARR3ADD(data->centroid, max);
+    VEC3MUL(data->centroid, 0.5);
+    }
   }
 
-/* if centroid is substantially within pbc, make pbc/2 the centroid */
+/* if centroid is substantially within repeat cell, make pbc/2 the centroid */
 if (data->periodic)
   {
   for (i=data->periodic ; i-- ; )
@@ -521,7 +591,6 @@ for (list=data->images ; list ; list=g_slist_next(list))
   n++;
   }
 VEC3MUL(vec, 1.0 / (gdouble) n);
-
 ARR3ADD(data->centroid, vec);
 
 /* get distance of furtherest displayed item from centroid */
@@ -575,27 +644,38 @@ if (data->periodic)
     }
   }
 
-/* molecular surface contribution */
+/* check vertices */
+for (list=data->vertices ; list ; list=g_slist_next(list))
+  {
+  v = list->data;
+  ARR3SET(vec, v->x);
+  ARR3SUB(vec, data->centroid);
+  vecmat(data->latmat, vec);
+  r = VEC3MAGSQ(vec);
+  if (r > r2)
+    r2 = r; 
+  }
+
+/* check spatial objects */
 for (list=data->spatial ; list ; list=g_slist_next(list))
   {
-  spatial = (struct spatial_pak *) list->data;
+  spatial = list->data;
 
+/* molecular surface contribution */
   if (spatial->type == SPATIAL_MOLSURF)
     {
     p1 = g_slist_nth_data(spatial->data, 0);
 
     ARR3SET(vec, p1->x);
     ARR3SUB(vec, data->centroid);
-    vecmat(data->latmat,vec);  
+    vecmat(data->latmat, vec);  
 
-/* dist sq. */
     r = VEC3MAGSQ(vec);
     if (r > r2)
       r2 = r; 
     }
   }
-
-rmax = RMAX_SCALE * sqrt(r2);
+rmax = sqrt(r2);
 
 /* assign the calculated value, unless 0 */
 if (rmax)
@@ -620,17 +700,17 @@ return(0);
 gint sort_cores(struct core_pak *core1, struct core_pak *core2)
 {
 if (core1->x[2] < core2->x[2])
-  return(-1);
-if (core1->x[2] > core2->x[2])
   return(1);
+if (core1->x[2] > core2->x[2])
+  return(-1);
 return(0);
 }
 gint sort_shels(struct shel_pak *shel1, struct shel_pak *shel2)
 {
 if (shel1->x[2] < shel2->x[2])
-  return(-1);
-if (shel1->x[2] > shel2->x[2])
   return(1);
+if (shel1->x[2] > shel2->x[2])
+  return(-1);
 return(0);
 }
 void sort_coords(struct model_pak *data)
@@ -654,7 +734,7 @@ if (!list)
 
 /* init */
 core = list->data;
-for (i=3 ;i-- ; )
+for (i=3 ; i-- ; )
   min[i] = max[i] = core->x[i];
 
 /* NB: even tho' we check the "fractional" part - the values */
@@ -776,6 +856,8 @@ mat4mat(tmp4, mat4);
 P4MAT("mat4:", mat4);
 #endif
 
+memcpy(data->display_lattice, mat4, 16*sizeof(gdouble));
+
 /* update image translation vectors */
 for (list=data->images ; list ; list=g_slist_next(list))
   {
@@ -822,6 +904,9 @@ ARR3ADD(vec4, shell->offset);
   ARR3SET(shell->rx, vec4);
   }
 
+/* CURRENT */
+measure_update_global(data);
+
 /********/
 /* CELL */
 /********/
@@ -977,8 +1062,9 @@ if (data->id == MORPH)
         n++;
         }
 /* assign */
-      g_assert(n > 0);
-      VEC3MUL(vec, 1.0/(gdouble) n);
+/* NB: n=0 can sometimes occur with a bad (ie non-closed) polyhedron */
+      if (n)
+        VEC3MUL(vec, 1.0/(gdouble) n);
       ARR3SET(plane->rx, vec);
       }
     }
@@ -1050,8 +1136,9 @@ gchar *txt;
 
 g_assert(shell != NULL);
 
-txt = g_strdup_printf("%4s shel at (%9.4f,%9.4f,%9.4f).\n",
-                      shell->label, shell->x[0], shell->x[1], shell->x[2]);
+txt = g_strdup_printf("%4s shel at (%9.4f,%9.4f,%9.4f) [%9.4f, %9.4f, %9.4f]\n",
+                      shell->label, shell->x[0], shell->x[1], shell->x[2],
+                      shell->rx[0], shell->rx[1], shell->rx[2]);
 show_text(STANDARD, txt);
 g_free(txt);
 }
@@ -1114,27 +1201,37 @@ if (n == 7 || n == 15)
   }
 }
 
+/***********************/
+/* core free primitive */
+/***********************/
+void core_free(gpointer data)
+{
+struct core_pak *core = data;
+
+g_free(core->atom_label);
+g_free(core->atom_type);
+g_free(core->res_name);
+g_slist_free(core->bonds);
+
+free_slist(core->vibx_list);
+free_slist(core->viby_list);
+free_slist(core->vibz_list);
+
+g_free(core);
+}
+
 /****************************************************/
 /* free all lists that reference cores or core data */
 /****************************************************/
 void free_core_list(struct model_pak *data)
 {
 GSList *list;
-struct core_pak *core;
+
+g_assert(data != NULL);
 
 /* free cores */
 for (list=data->cores ; list ; list=g_slist_next(list))
-  {
-  core = (struct core_pak *) list->data;
-
-  g_slist_free(core->bonds);
-
-  free_slist(core->vibx_list);
-  free_slist(core->viby_list);
-  free_slist(core->vibz_list);
-
-  g_free(core);
-  }
+  core_free(list->data);
 g_slist_free(data->cores);
 
 /* free shells */
