@@ -20,17 +20,25 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-/* dialog control */
-gint request_dialog(gint, gint);
-gint dialog_active(gint);
-gint dialog_seek(gint, gint);
-void close_dialog(gint);
-void close_dialog_type(gint);
-void event_close_dialog(GtkWidget *, gint);
-void close_model_dialogs(gint);
-void colour_dialog(gchar *, GtkWidget *, gpointer, gpointer, gpointer);
-void modify_colour_dialog(GtkWidget *w, gdouble *colour);
+/* dialogs */
+gpointer dialog_request(gint, gchar *, gpointer, gpointer, struct model_pak *);
+gpointer dialog_window(gpointer);
+gpointer dialog_model(gpointer);
+gint dialog_exists(gint, struct model_pak *);
+void dialog_refresh_type(gint);
+void dialog_refresh_model(struct model_pak *);
+void dialog_close_model(struct model_pak *);
+void dialog_close(gint, struct model_pak *);
+void dialog_destroy(GtkWidget *, gpointer);
+void dialog_destroy_type(gint);
+void dialog_destroy_model(struct model_pak *);
+void dialog_destroy_single(gint, struct model_pak *);
 
+/* dialog control */
+void colour_dialog(gchar *, GtkWidget *, gpointer, gpointer, gpointer);
+void modify_colour_dialog(GtkWidget *, gdouble *);
+
+/* deprec */
 /* cleanup control */
 void file_cleanup(void);
 void meas_cleanup(void);
@@ -38,3 +46,10 @@ void render_cleanup(void);
 void symmetry_cleanup(void);
 void task_cleanup(void);
 
+/* individual dialogs */
+void periodicity_dialog(void);
+void symmetry_dialog(void);
+void surface_dialog(void);
+void energetics_dialog(void);
+
+
