@@ -84,7 +84,8 @@ while(!fgetline(fp,line))
           if (fgetline(fp, line))
             break;
           list = get_keywords(line);
-          buff = get_tokens(line, 11);
+          buff = tokenize(line, &num_tokens);
+
 /* process 1st keyword, and subsequent data (if any) */
           if (list)
             {
@@ -93,32 +94,40 @@ while(!fgetline(fp,line))
               {
 /* these shouldn't be modified if not default */
               case SYMBOL:
+                g_assert(num_tokens > 1);
                 elem.symbol = g_strdup(*(buff+1));
                 break; 
               case NAME:
+                g_assert(num_tokens > 1);
                 elem.name = g_strdup(*(buff+1));
                 break; 
               case NUMBER:
+                g_assert(num_tokens > 1);
                 elem.number = (gint) str_to_float(*(buff+1));
                 set[key]++;
                 break; 
               case WEIGHT:
+                g_assert(num_tokens > 1);
                 elem.weight = str_to_float(*(buff+1));
                 set[key]++;
                 break; 
               case COVALENT:
+                g_assert(num_tokens > 1);
                 elem.cova = str_to_float(*(buff+1));
                 set[key]++;
                 break; 
               case VDW:
+                g_assert(num_tokens > 1);
                 elem.vdw = str_to_float(*(buff+1));
                 set[key]++;
                 break; 
               case CHARGE:
+                g_assert(num_tokens > 1);
                 elem.charge = str_to_float(*(buff+1));
                 set[key]++;
                 break; 
               case COLOUR:
+                g_assert(num_tokens > 3);
                 elem.colour[0] = (gint) str_to_float(*(buff+1));
                 elem.colour[1] = (gint) str_to_float(*(buff+2));
                 elem.colour[2] = (gint) str_to_float(*(buff+3));
@@ -193,7 +202,7 @@ if (num_tokens != 12)
   break;
   }
       symbol = g_strdup(*(buff+0));
-      if (elem_type(symbol))
+      if (elem_symbol_test(symbol))
         {
         list = NULL;
         for (i=1 ; i<num_tokens ; i++)
@@ -274,7 +283,7 @@ fprintf(fp, "%%gdis_end\n");
 /* find all unique elements in a model */
 /***************************************/
 #define DEBUG_FIND_UNIQUE 0
-GSList *find_unique(gint mode, struct model_pak *data)
+GSList *find_unique(gint mode, struct model_pak *model)
 {
 gint found;
 GSList *list, *types, *unique;
@@ -283,7 +292,7 @@ struct core_pak *core;
 unique = NULL;
 
 /* go through all atoms */
-for (list=data->cores ; list ; list=g_slist_next(list))
+for (list=model->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
 
@@ -357,11 +366,11 @@ return(unique);
 /***********************/
 /* set the atom colour */
 /***********************/
-void init_atom_colour(struct core_pak *core, struct model_pak *mdata)
+void init_atom_colour(struct core_pak *core, struct model_pak *model)
 {
 struct elem_pak elem_data;
 
-get_elem_data(core->atom_code, &elem_data, mdata);
+get_elem_data(core->atom_code, &elem_data, model);
 ARR3SET(core->colour, elem_data.colour);
 if (core->ghost)
   core->colour[3] = 0.5;
@@ -372,13 +381,13 @@ else
 /***********************/
 /* set the atom charge */
 /***********************/
-void init_atom_charge(struct core_pak *core, struct model_pak *mdata)
+void init_atom_charge(struct core_pak *core, struct model_pak *model)
 {
 struct elem_pak elem_data;
 
 if (core->lookup_charge)
   {
-  get_elem_data(core->atom_code, &elem_data, mdata);
+  get_elem_data(core->atom_code, &elem_data, model);
   core->charge = elem_data.charge;
   }
 }
@@ -386,22 +395,101 @@ if (core->lookup_charge)
 /************************/
 /* set the atom charges */
 /************************/
-void init_model_charges(struct model_pak *mdata)
+void init_model_charges(struct model_pak *model)
 {
 GSList *list;
 
-for (list=mdata->cores ; list ; list=g_slist_next(list))
-  init_atom_charge(list->data, mdata);
+for (list=model->cores ; list ; list=g_slist_next(list))
+  init_atom_charge(list->data, model);
 }
 
-/*********************************************/
-/* does a given string represent an element? */
-/*********************************************/
+/*********************************************************/
+/* compute net charge and dipole (if model is a surface) */
+/*********************************************************/
+#define DEBUG_CALC_EMP 0
+void calc_emp(struct model_pak *data)
+{
+gdouble zdip, qsum, x[3];
+GSList *list;
+struct core_pak *core;
+struct shel_pak *shell;
+
+/* checks */
+g_assert(data != NULL);
+
+/* TODO - speed up by getting to a local var the elem data for unique atoms */
+/* cores & shell dipole calc */
+zdip=qsum=0.0;
+for (list=data->cores ; list ; list=g_slist_next(list))
+  {
+  core = list->data;
+/*
+  if (core->region != REGION1A)
+    continue;
+*/
+  if (core->status & DELETED)
+    continue;
+
+/* core contribution */
+  ARR3SET(x, core->x);
+  vecmat(data->latmat, x);
+
+  zdip += core->charge * x[2];
+  qsum += core->charge;
+
+/* NB: add shell constribution at the core's z location */
+  if (core->shell)
+    {
+    shell = core->shell;
+
+    ARR3SET(x, shell->x);
+    vecmat(data->latmat, x);
+    zdip += shell->charge * x[2];
+    qsum += shell->charge;
+    }
+  }
+
+#if DEBUG_CALC_EMP
+printf("surface: %f %f %f  (%f)\n", 
+data->surface.miller[0], data->surface.miller[1], data->surface.miller[2], 
+data->surface.shift);
+printf("net dipole: %e\n", zdip);
+printf("sum charge: %e\n", qsum);
+if (qsum*qsum > FRACTION_TOLERANCE)
+  printf("Warning: your model has a net charge = %f\n", qsum);
+#endif
+
+/* only surfaces get the z dipole */
+if (data->periodic == 2)
+  data->gulp.sdipole = zdip;
+
+data->gulp.qsum = qsum;
+}
+
+/***************************************************/
+/* does a given string represent an atomic number? */
+/***************************************************/
+gint elem_number_test(const gchar *input)
+{
+gint n=0;
+
+if (str_is_float(input))
+  {
+  n = str_to_float(input);
+  if (n<0 || n>MAX_ELEMENTS-1)
+    n=0; 
+  }
+return(n);
+}
+
+/***************************************************/
+/* does a given string represent an element symbol */
+/***************************************************/
 #define DEBUG_ELEM_TYPE 0
-gint elem_type(const gchar *input)
+gint elem_symbol_test(const gchar *input)
 {
 gint i, j, m, n;
-gchar *elem1, *elem2, **buff;
+gchar *elem1, **buff;
 
 /* checks */
 if (input == NULL)
@@ -442,18 +530,16 @@ j=0;
 /* FIXME - elim dependence on const */
 for(i=0 ; i<sysenv.num_elements ; i++)
   {
-/* duplicate for manipulation */
-  elem2 = g_strdup(elements[i].symbol);
-  g_strstrip(elem2);
-  n = strlen(elem2);
-/* only compare if stripped lengths match */
+/* only compare if lengths match */
+  n = strlen(elements[i].symbol);
   if (n == m)
-    if (g_ascii_strncasecmp(elem1, elem2, m) == 0)
+    {
+    if (g_ascii_strncasecmp(elem1, elements[i].symbol, m) == 0)
       {
       j = i;
       break;
       }
-  g_free(elem2);
+    }
   }
 
 #if DEBUG_ELEM_TYPE
@@ -468,6 +554,20 @@ g_free(elem1);
 return(j);
 }
 
+/*************************************************************/
+/* does a given string represent an element symbol or number */
+/*************************************************************/
+gint elem_test(const gchar *input)
+{
+gint n;
+
+n = elem_symbol_test(input);
+if (!n)
+  n = elem_number_test(input);
+
+return(n);
+}
+
 /*********************************/
 /* PDB hack for element matching */
 /*********************************/
@@ -501,13 +601,13 @@ for (i=0 ; i<len ; i++)
     case 'h':
     case 'H':
       tmp = g_strndup(input, i+1);
-      j = elem_type(tmp);
+      j = elem_symbol_test(tmp);
       g_free(tmp);
       return(j);
     }
   }
 /* default element matching */
-return(elem_type(input));
+return(elem_symbol_test(input));
 }
 
 /*************************/
@@ -515,7 +615,7 @@ return(elem_type(input));
 /*************************/
 /* will return non-default data (if it exists) otherwise the default */
 #define DEBUG_GET_ELEM 0
-gint get_elem_data(gint code, struct elem_pak *elem, struct model_pak *data)
+gint get_elem_data(gint code, struct elem_pak *elem, struct model_pak *model)
 {
 gint i;
 struct elem_pak *elem_data;
@@ -524,16 +624,16 @@ GSList *list;
 /* TODO - seek in sysenv (cope with data == NULL) */
 
 /* search the model exceptions database */
-if (data)
+if (model)
   {
 #if DEBUG_GET_ELEM
-printf("Searching %d model items items for code = %d.\n",data->num_elem,code);
+printf("Searching %d model items items for code = %d.\n",model->num_elem,code);
 #endif
-  for (i=data->num_elem ; i-- ; )
+  for (i=model->num_elem ; i-- ; )
     {
-    if (code == (data->elements+i)->number)
+    if (code == (model->elements+i)->number)
       {
-      memcpy(elem, data->elements+i, sizeof(struct elem_pak));
+      memcpy(elem, model->elements+i, sizeof(struct elem_pak));
       return(0);
       }
     }
@@ -582,7 +682,7 @@ return(1);
 /* store non-default element data */
 /**********************************/
 #define DEBUG_PUT_ELEM_DATA 0
-void put_elem_data(struct elem_pak *elem, struct model_pak *data)
+void put_elem_data(struct elem_pak *elem, struct model_pak *model)
 {
 gint i, ix=0, flag=0;
 GSList *list;
@@ -612,7 +712,7 @@ if (VEC3MAGSQ(elem->colour) < 3.1)
   }
 
 /* if data = NULL (ie no model) -> global -> put in sysenv list */
-if (data == NULL)
+if (model == NULL)
   {
 /* search for existing item */
   list = sysenv.elements;
@@ -650,16 +750,16 @@ printf("Modifying local database.\n");
 /* replace flag (as opposed to add new) */
 ix=0;
 /* alloc if non-existent */
-if (!data->num_elem)
+if (!model->num_elem)
   {
-  data->elements = g_malloc(sizeof(struct elem_pak));
-  data->num_elem++;
+  model->elements = g_malloc(sizeof(struct elem_pak));
+  model->num_elem++;
   flag++;
   }
 else
   {
 /* search for existing data */
-  for (i=data->num_elem ; i-- ; )
+  for (i=model->num_elem ; i-- ; )
     {
     if (elem->number == i)
       {
@@ -672,16 +772,16 @@ else
 /* TODO - a linked list would be easier... */
 /* replace or re-alloc & add? */
   if (flag)
-    memcpy(data->elements+ix, elem, sizeof(struct elem_pak));
+    memcpy(model->elements+ix, elem, sizeof(struct elem_pak));
   else
     {
-    data->elements = g_renew(struct elem_pak, data->elements, data->num_elem+1);
-    memcpy(data->elements+data->num_elem, elem, sizeof(struct elem_pak));
-    data->num_elem++;
+    model->elements = g_renew(struct elem_pak, model->elements, model->num_elem+1);
+    memcpy(model->elements+model->num_elem, elem, sizeof(struct elem_pak));
+    model->num_elem++;
     }
 
 /* update atom bond_cutoff */
-  for (list=data->cores ; list ; list=g_slist_next(list))
+  for (list=model->cores ; list ; list=g_slist_next(list))
     {
     core = (struct core_pak *) list->data;
     if (core->atom_code == elem->number)
@@ -693,11 +793,11 @@ else
 /******************************************/
 /* lookup and restore core default values */
 /******************************************/
-void init_elem(struct core_pak *core, gint type, struct model_pak *data)
+void init_elem(struct core_pak *core, gint type, struct model_pak *model)
 {
 struct elem_pak elem;
 
-get_elem_data(core->atom_code, &elem, data);
+get_elem_data(core->atom_code, &elem, model);
 
 switch (type)
   {
@@ -710,130 +810,8 @@ switch (type)
   }
 }
 
-/****************************************/
-/* transfer model specific element data */
-/****************************************/
-gint copy_elem_data(struct model_pak *src, struct model_pak *dest)
-{
-/* free old */
-if (dest->num_elem)
-  g_free(dest->elements);
-
-/* alloc and copy new */
-dest->elements = g_malloc(src->num_elem * sizeof(struct elem_pak));
-memcpy(dest->elements, src->elements, src->num_elem*sizeof(struct elem_pak));
-dest->num_elem = src->num_elem;
-
-return(0);
-}
-
-/**********************/
-/* transfer gulp data */
-/**********************/
-gint copy_gulp_data(struct model_pak *src, struct model_pak *dest)
-{
-gint i;
-
-/* free any old data in the destination model */
-free_gulp_data(dest);
-
-/* copy keywords */
-dest->gulp.no_exec = src->gulp.no_exec;
-dest->gulp.run = src->gulp.run;
-dest->gulp.method = src->gulp.method;
-dest->gulp.optimiser = src->gulp.optimiser;
-dest->gulp.optimiser2 = src->gulp.optimiser2;
-dest->gulp.switch_type = src->gulp.switch_type;
-dest->gulp.switch_value = src->gulp.switch_value;
-dest->gulp.free = src->gulp.free;
-dest->gulp.qeq = src->gulp.qeq;
-dest->gulp.zsisa = src->gulp.zsisa;
-dest->gulp.compare = src->gulp.compare;
-dest->gulp.nosym = src->gulp.nosym;
-dest->gulp.fix = src->gulp.fix;
-dest->gulp.cycles = src->gulp.cycles;
-dest->gulp.ensemble = src->gulp.ensemble;
-dest->gulp.coulomb = src->gulp.coulomb;
-dest->gulp.maxcyc = src->gulp.maxcyc;
-
-/* copy surface related data */
-if (dest->periodic == 2)
-  dest->gulp.sbulkenergy = src->gulp.sbulkenergy;
-if (dest->periodic == 2 && src->periodic == 2)
-  dest->surface.dspacing = src->surface.dspacing;
-
-/* copy potentials data */
-if (src->gulp.num_potentials)
-  {
-  dest->gulp.num_potentials = src->gulp.num_potentials;
-  dest->gulp.pot = g_malloc(src->gulp.num_potentials*sizeof(gchar *));
-  for (i=0 ; i<src->gulp.num_potentials ; i++)
-    *(dest->gulp.pot+i) = g_strdup(*(src->gulp.pot+i));
-  }
-/* copy element data */
-if (src->gulp.num_elem)
-  {
-  dest->gulp.num_elem = src->gulp.num_elem;
-  dest->gulp.elem = g_malloc(src->gulp.num_elem*sizeof(gchar *));
-  for (i=0 ; i<src->gulp.num_elem ; i++)
-    *(dest->gulp.elem+i) = g_strdup(*(src->gulp.elem+i));
-  }
-/* copy species data */
-if (src->gulp.num_species)
-  {
-  dest->gulp.num_species = src->gulp.num_species;
-  dest->gulp.species = g_malloc(src->gulp.num_species*sizeof(gchar *));
-  for (i=0 ; i<src->gulp.num_species ; i++)
-    *(dest->gulp.species+i) = g_strdup(*(src->gulp.species+i));
-  }
-
-return(0);
-}
-
-gint copy_gulp_extra(struct model_pak *src, struct model_pak *dest)
-{
-/* copy unrecognized gulp options */
-/* FIXME - if dest is 2D don't copy all this stuff, eg skip elastic consts */
-if (src->gulp.extra)
-  dest->gulp.extra = g_strdup(src->gulp.extra);
-return(0);
-}
-
-/**********************************/
-/* free memory used for gulp data */
-/**********************************/
-#define DEBUG_FREE_GULP_DATA 0
-gint free_gulp_data(struct model_pak *data)
-{
-gint i;
-
-#if DEBUG_FREE_GULP_DATA
-printf("freeing gulp data...\n");
-#endif
-for (i=0 ; i<data->gulp.num_potentials ; i++)
-  g_free(*(data->gulp.pot+i));
-for (i=0 ; i<data->gulp.num_elem ; i++)
-  g_free(*(data->gulp.elem+i));
-for (i=0 ; i<data->gulp.num_species ; i++)
-  g_free(*(data->gulp.species+i));
-
-if (data->gulp.srcfile)
-  g_free(data->gulp.srcfile);
-if (data->gulp.extra)
-  g_free(data->gulp.extra);
-
-data->gulp.num_potentials = 0;
-data->gulp.num_elem = 0;
-data->gulp.num_species = 0;
-
-#if DEBUG_FREE_GULP_DATA
-printf("done.\n");
-#endif
-return(0);
-}
-
 /* globals for the atom properties dialog */
-GtkWidget *apd_label, *apd_charge, *apd_x, *apd_y, *apd_z;
+GtkWidget *apd_label, *apd_type, *apd_charge, *apd_x, *apd_y, *apd_z;
 struct model_pak *apd_data=NULL;
 struct core_pak *apd_core=NULL;
 
@@ -913,14 +891,14 @@ VEC3MUL(colour, 65535.0);
 /****************************************/
 /* change the colour scheme for an atom */
 /****************************************/
-void atom_colour_scheme(gint type, struct core_pak *core, struct model_pak *mdata)
+void atom_colour_scheme(gint type, struct core_pak *core, struct model_pak *model)
 {
 struct elem_pak edata;
 
 switch (type)
   {
   case ELEM:
-    get_elem_data(core->atom_code, &edata, mdata);
+    get_elem_data(core->atom_code, &edata, model);
     break;
 
   case MOL:
@@ -942,16 +920,16 @@ ARR3SET(core->colour, edata.colour);
 /****************************************/
 /* change the colour scheme for a model */
 /****************************************/
-void model_colour_scheme(gint type, struct model_pak *mdata)
+void model_colour_scheme(gint type, struct model_pak *model)
 {
 GSList *list;
 struct core_pak *core;
 
-mdata->colour_scheme = type;
-for (list=mdata->cores ; list ; list=g_slist_next(list))
+model->colour_scheme = type;
+for (list=model->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
-  atom_colour_scheme(type, core, mdata);
+  atom_colour_scheme(type, core, model);
   }
 }
 
@@ -970,7 +948,7 @@ struct elem_pak edata;
 struct model_pak *model;
 struct core_pak *core;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 if (!model)
   return;
 if (!model->selection)
@@ -980,7 +958,7 @@ switch (type)
   {
   case NAME:
     text = gtk_entry_get_text(GTK_ENTRY(apd_label));
-    n = elem_type(text);
+    n = elem_symbol_test(text);
     if (n)
       {
       get_elem_data(n, &edata, apd_data);
@@ -1055,7 +1033,7 @@ switch(type)
 /* make sure we alow enough space for the string and the \0 */
     n = (LABEL_SIZE-1 > strlen(text)) ? strlen(text) : LABEL_SIZE-1;
     g_snprintf(apd_core->label,n+1,"%s",text);
-    n = elem_type(text);
+    n = elem_symbol_test(text);
 
 /* if recognized -> update */
     if (n)
@@ -1080,26 +1058,43 @@ switch(type)
       }
     break;
 
+/* FIXME - not the best name, it's really the FF type */
+  case LABEL:
+    text = gtk_entry_get_text(GTK_ENTRY(apd_type));
+    if (apd_core->atom_type)
+      g_free(apd_core->atom_type);
+    apd_core->atom_type = g_strdup(text);
+    redraw_canvas(SINGLE);
+    break;
+
   case CHARGE:
     text = gtk_entry_get_text(GTK_ENTRY(apd_charge));
     apd_core->charge = str_to_float(text);
     apd_core->lookup_charge = FALSE;
     calc_emp(apd_data);
-    if (apd_data->show_charges)
+    if (apd_data->show_charge || apd_data->show_atom_charges)
       redraw_canvas(SINGLE);
     break;
 
   case COORD_X:
     text = gtk_entry_get_text(GTK_ENTRY(apd_x));
     apd_core->x[0] = str_to_float(text);
+    calc_coords(REFRESH, apd_data);
+    redraw_canvas(SINGLE);
     break;
+
   case COORD_Y:
     text = gtk_entry_get_text(GTK_ENTRY(apd_y));
     apd_core->x[1] = str_to_float(text);
+    calc_coords(REFRESH, apd_data);
+    redraw_canvas(SINGLE);
     break;
+
   case COORD_Z:
     text = gtk_entry_get_text(GTK_ENTRY(apd_z));
     apd_core->x[2] = str_to_float(text);
+    calc_coords(REFRESH, apd_data);
+    redraw_canvas(SINGLE);
     break;
 
   default:
@@ -1110,18 +1105,23 @@ switch(type)
 /*************************************/
 /* updates the dialog for a new atom */
 /*************************************/
-void atom_properties_update(struct core_pak *core, struct model_pak *data)
+void atom_properties_update(struct core_pak *core, struct model_pak *model)
 {
 gdouble q;
-gchar *element, *label, *charge, *x, *y, *z;
+gchar *element, *label, *type, *charge, *x, *y, *z;
 struct shel_pak *shell;
 
-if (core && data)
+if (core && model)
   {
 /* data available */
   element = g_strdup(elements[core->atom_code].symbol);
   label = g_strdup(core->label);
 
+  if (core->atom_type)
+    type = g_strdup(core->atom_type);
+  else
+    type = g_strdup("?");
+
   q = core->charge;
   if (core->shell)
     {
@@ -1141,6 +1141,7 @@ else
 /* otherwise defaults */
   element = g_strdup("");
   label = g_strdup("");
+  type = g_strdup("");
   charge = g_strdup("");
   x = g_strdup("");
   y = g_strdup("");
@@ -1152,6 +1153,7 @@ apd_data = NULL;
 
 /* entry updates */
 gtk_entry_set_text(GTK_ENTRY(apd_label), label);
+gtk_entry_set_text(GTK_ENTRY(apd_type), type);
 gtk_entry_set_text(GTK_ENTRY(apd_charge), charge);
 gtk_entry_set_text(GTK_ENTRY(apd_x), x);
 gtk_entry_set_text(GTK_ENTRY(apd_y), y);
@@ -1159,11 +1161,12 @@ gtk_entry_set_text(GTK_ENTRY(apd_z), z);
 
 /* TODO : colour? SOF? */
 
-apd_data = data;
+apd_data = model;
 
 /* cleanup */
 g_free(element);
 g_free(label);
+g_free(type);
 g_free(charge);
 g_free(x);
 g_free(y);
@@ -1175,17 +1178,14 @@ g_free(z);
 /****************************************/
 void atom_properties_box(GtkWidget *box)
 {
-GtkWidget *frame, *hbox, *vbox, *entry;
+GtkWidget *hbox, *vbox, *entry;
 
 /* checks */
 g_return_if_fail(box != NULL);
 
-frame = gtk_frame_new(NULL);
-gtk_box_pack_start(GTK_BOX(box), frame, TRUE, FALSE, 0);
-
 /* two column element data display */
 hbox = gtk_hbox_new(FALSE, 0);
-gtk_container_add(GTK_CONTAINER(frame), hbox);
+gtk_box_pack_start(GTK_BOX(box), hbox, TRUE, TRUE, 0);
 
 /* left vbox - titles */
 vbox = gtk_vbox_new(TRUE, 0);
@@ -1198,6 +1198,11 @@ gtk_entry_set_text(GTK_ENTRY(entry), "Label");
 gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
 gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
 
+entry = gtk_entry_new();
+gtk_entry_set_text(GTK_ENTRY(entry), "FF Type");
+gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
+gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
+
 entry = gtk_entry_new();
 gtk_entry_set_text(GTK_ENTRY(entry), "Charge");
 gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
@@ -1230,6 +1235,9 @@ gtk_box_pack_end(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 apd_label = gtk_entry_new();
 gtk_box_pack_start(GTK_BOX(vbox), apd_label, FALSE, FALSE, 0);
 
+apd_type = gtk_entry_new();
+gtk_box_pack_start(GTK_BOX(vbox), apd_type, FALSE, FALSE, 0);
+
 apd_charge = gtk_entry_new();
 gtk_box_pack_start(GTK_BOX(vbox), apd_charge, FALSE, FALSE, 0);
 
@@ -1248,6 +1256,8 @@ atom_properties_update(NULL, NULL);
 /* attach callbacks (NB: set initial data first) */
 g_signal_connect(GTK_OBJECT(apd_label), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(NAME));
+g_signal_connect(GTK_OBJECT(apd_type), "activate",
+                 GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(LABEL));
 g_signal_connect(GTK_OBJECT(apd_charge), "activate",
                  GTK_SIGNAL_FUNC(atom_properties_change), GINT_TO_POINTER(CHARGE));
 g_signal_connect(GTK_OBJECT(apd_x), "activate",
