@@ -47,7 +47,10 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "file.h"
 #include "parse.h"
 #include "matrix.h"
+#include "model.h"
 #include "space.h"
+#include "render.h"
+#include "select.h"
 #include "gtkshorts.h"
 #include "interface.h"
 #include "dialog.h"
@@ -60,24 +63,76 @@ The GNU GPL can also be found at http://www.gnu.org
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
+/***************************************/
+/* setup the recognized file type list */
+/***************************************/
+#define DEBUG_FILE_INIT 0
 void file_init(void)
 {
+GSList *list, *rlist=NULL;
 struct file_pak *file_data;
 
-/* TODO - include calling routines for read/writing */
-/* TODO - include the rest of the (babel) types */
+/* NEW - build a recognized image read format list */
+/* also used in picture writing as it avoids overlap */
+/* problems when getting the file_data structure by extension */ 
+#define PICTURE_SUPPORT 0
+#if PICTURE_SUPPORT
+for (list=gdk_pixbuf_get_formats() ; list ; list=g_slist_next(list))
+  {
+  gint i;
+  gchar **ext;
 
+  ext = gdk_pixbuf_format_get_extensions(list->data);
+
+  i = 0;
+  while (*(ext+i))
+    {
+/*
+    if (gdk_pixbuf_format_is_writable(list->data))
+      wlist = g_slist_prepend(wlist, g_strdup(*(ext+i)));
+*/
+    rlist = g_slist_prepend(rlist, g_strdup(*(ext+i)));
+    i++;
+    }
+  }
+#endif
+
+#if DEBUG_FILE_INIT
+printf("read: ");
+for (list=rlist ; list ; list=g_slist_next(list))
+  {
+  printf("[%s] ", (gchar *) list->data);
+  }
+printf("\n");
+#endif
+
+/* build the recognized file type list */
 sysenv.file_list = NULL;
 
 /* supported file type */
 file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
 file_data->id = DATA;                           /* unique identifier */ 
 file_data->group = DATA;                        /* used to group inp/out types */
-file_data->menu = TRUE;                         /* include in file menu listing? */
+file_data->menu = TRUE;                         /* include in menu listing */
 file_data->label = g_strdup("All known types"); /* text info for the user */
 file_data->ext = NULL;                          /* extension matching */
 file_data->write_file = NULL;                   /* file creation */
 file_data->read_file = NULL;                    /* file reading */
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = ABINIT_OUT;
+file_data->group = ABINIT;
+file_data->menu = FALSE;
+file_data->label = g_strdup("ABINIT output");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "about");
+file_data->ext = g_slist_prepend(file_data->ext, "abot");
+file_data->write_file = NULL;
+file_data->read_file = read_about;
+file_data->read_frame = read_about_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -92,6 +147,7 @@ file_data->ext = g_slist_prepend(file_data->ext, "cor");
 file_data->ext = g_slist_prepend(file_data->ext, "arc");
 file_data->write_file = write_arc;
 file_data->read_file = read_arc;
+file_data->read_frame = read_arc_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -104,155 +160,20 @@ file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "cif");
 file_data->write_file = write_cif;
 file_data->read_file = read_cif;
+file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
 file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = FDF;
-file_data->group = FDF;
+file_data->id = DIFFAX_INP;
+file_data->group = DIFFAX_INP;
 file_data->menu = TRUE;
-file_data->label = g_strdup("SIESTA");
+file_data->label = g_strdup("DIFFaX");
 file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "fdf");
-file_data->write_file = write_fdf;
-file_data->read_file = read_fdf;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = SIESTA_OUT;
-file_data->group = FDF;
-file_data->menu = FALSE;
-file_data->label = g_strdup("SIESTA Output");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "sout");
-file_data->ext = g_slist_prepend(file_data->ext, "sot");
-file_data->write_file = NULL;
-file_data->read_file = read_sout;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = GULP;
-file_data->group = GULP;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Gulp");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "gin");
-file_data->ext = g_slist_prepend(file_data->ext, "res");
-file_data->write_file = write_gulp;
-file_data->read_file = read_gulp;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = GULPOUT;
-file_data->group = GULP;
-file_data->menu = FALSE;
-file_data->label = g_strdup("Gulp output");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "got");
-file_data->ext = g_slist_prepend(file_data->ext, "gout");
-file_data->write_file = NULL;
-file_data->read_file = read_gulp_output;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = MARVIN;
-file_data->group = MARVIN;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Marvin");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "mar");
-file_data->ext = g_slist_prepend(file_data->ext, "mvn");
-file_data->ext = g_slist_prepend(file_data->ext, "mvn-r");
-file_data->write_file = write_marvin;
-file_data->read_file = read_marvin;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = MVNOUT;
-file_data->group = MARVIN;
-file_data->menu = FALSE;
-file_data->label = g_strdup("Marvin output");
-file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "mot");
-file_data->ext = g_slist_prepend(file_data->ext, "mvout");
-file_data->ext = g_slist_prepend(file_data->ext, "mvnout");
-file_data->write_file = NULL;
-file_data->read_file = read_mvnout;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = MORPH;
-file_data->group = MORPH;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Morphology");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "gmf");
-file_data->write_file = write_gmf;
-file_data->read_file = read_gmf;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = XTL;
-file_data->group = XTL;
-file_data->menu = TRUE;
-file_data->label = g_strdup("XTL");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "xtl");
-file_data->write_file = write_xtl;
-file_data->read_file = read_xtl;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = XYZ;
-file_data->group = XYZ;
-file_data->menu = TRUE;
-file_data->label = g_strdup("XYZ");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "xyz");
-file_data->write_file = write_xyz;
-file_data->read_file = read_xyz;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = BABEL;
-file_data->group = DATA;
-file_data->menu = TRUE;
-file_data->label = g_strdup("Filterable types");
-file_data->ext = NULL;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = AUTO;
-file_data->group = BABEL;
-file_data->menu = TRUE;
-file_data->label = g_strdup("auto select filter");
-file_data->ext = NULL;
-file_data->write_file = NULL;
-file_data->read_file = NULL;
-sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
-
-/* supported file type */
-file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = PDB;
-file_data->group = PDB;
-file_data->menu = TRUE;
-file_data->label = g_strdup("PDB");
-file_data->ext = NULL;
-file_data->ext = g_slist_append(file_data->ext, "pdb");
-file_data->write_file = NULL;
-file_data->read_file = read_pdb;
+file_data->ext = g_slist_prepend(file_data->ext, "dfx");
+file_data->write_file = write_diffax;
+file_data->read_file = read_diffax;
+file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -265,6 +186,7 @@ file_data->ext = NULL;
 file_data->ext = g_slist_append(file_data->ext, "inp");
 file_data->write_file = write_gms;
 file_data->read_file = read_gms;
+file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -278,18 +200,248 @@ file_data->ext = g_slist_prepend(file_data->ext, "gmout");
 file_data->ext = g_slist_prepend(file_data->ext, "gmot");
 file_data->write_file = NULL;
 file_data->read_file = read_gms_out;
+file_data->read_frame = read_gms_out_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
 file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = DIFFAX_INP;
-file_data->group = DIFFAX_INP;
-file_data->menu = TRUE;
-file_data->label = g_strdup("DIFFaX");
+file_data->id = MD_ANALYSIS;
+file_data->group = MD_ANALYSIS;
+file_data->menu = FALSE;
+file_data->label = g_strdup("Analysis");
 file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "dfx");
-file_data->write_file = write_diffax;
-file_data->read_file = read_diffax;
+file_data->ext = NULL;
+/* import/export handled in analysis as a special case */
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = MORPH;
+file_data->group = MORPH;
+file_data->menu = TRUE;
+file_data->label = g_strdup("GDIS Morphology");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "gmf");
+file_data->write_file = write_gmf;
+file_data->read_file = read_gmf;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = GULP;
+file_data->group = GULP;
+file_data->menu = TRUE;
+file_data->label = g_strdup("GULP");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "gin");
+file_data->ext = g_slist_prepend(file_data->ext, "res");
+file_data->write_file = write_gulp;
+file_data->read_file = read_gulp;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = GULPOUT;
+file_data->group = GULP;
+file_data->menu = FALSE;
+file_data->label = g_strdup("GULP output");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "got");
+file_data->ext = g_slist_prepend(file_data->ext, "gout");
+file_data->write_file = NULL;
+file_data->read_file = read_gulp_output;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = MARVIN;
+file_data->group = MARVIN;
+file_data->menu = TRUE;
+file_data->label = g_strdup("MARVIN");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "mar");
+file_data->ext = g_slist_prepend(file_data->ext, "mvn");
+file_data->ext = g_slist_prepend(file_data->ext, "mvn-r");
+file_data->write_file = write_marvin;
+file_data->read_file = read_marvin;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = MVNOUT;
+file_data->group = MARVIN;
+file_data->menu = FALSE;
+file_data->label = g_strdup("MARVIN output");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "mot");
+file_data->ext = g_slist_prepend(file_data->ext, "mvout");
+file_data->ext = g_slist_prepend(file_data->ext, "mvnout");
+file_data->write_file = NULL;
+file_data->read_file = read_mvnout;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = NWCHEM;
+file_data->group = NWCHEM;
+file_data->menu = TRUE;
+file_data->label = g_strdup("NWChem");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "nwin");
+file_data->write_file = NULL;
+file_data->read_file = read_nw;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = NWCHEM_OUT;
+file_data->group = NWCHEM;
+file_data->menu = TRUE;
+file_data->label = g_strdup("NWChem output");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "nwout");
+file_data->ext = g_slist_prepend(file_data->ext, "nwot");
+file_data->write_file = NULL;
+file_data->read_file = read_nwout;
+file_data->read_frame = read_nwout_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = PICTURE;
+file_data->group = PICTURE;
+file_data->menu = FALSE;
+/* TODO - check supported image types with glib - add to string */
+file_data->label = g_strdup("Picture (jpg)");
+file_data->ext = rlist;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = RIETICA;
+file_data->group = RIETICA;
+file_data->menu = TRUE;
+file_data->label = g_strdup("Rietica");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "inp");
+file_data->write_file = NULL;
+file_data->read_file = read_rietica;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = PDB;
+file_data->group = PDB;
+file_data->menu = TRUE;
+file_data->label = g_strdup("PDB");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "pdb");
+file_data->write_file = write_pdb;
+file_data->read_file = read_pdb;
+file_data->read_frame = read_pdb_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = CASTEP;
+file_data->group = CASTEP;
+file_data->menu = TRUE;
+file_data->label = g_strdup("CASTEP");
+file_data->ext = NULL;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = CASTEP_OUT;
+file_data->group = CASTEP;
+file_data->menu = FALSE;
+file_data->label = g_strdup("CASTEP output");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "castep");
+file_data->write_file = NULL;
+file_data->read_file = read_castep_out;
+file_data->read_frame = read_castep_out_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = GAUSS;
+file_data->group = GAUSS;
+file_data->menu = TRUE;
+file_data->label = g_strdup("GAUSSIAN");
+file_data->ext = NULL;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = GAUSS_OUT;
+file_data->group = GAUSS;
+file_data->menu = FALSE;
+file_data->label = g_strdup("GAUSSIAN output");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "log");
+file_data->write_file = NULL;
+file_data->read_file = read_gauss_out;
+file_data->read_frame = read_gauss_out_frame;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = POVRAY;
+file_data->group = POVRAY;
+file_data->menu = FALSE;
+file_data->label = g_strdup("POVRay");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "pov");
+file_data->write_file = write_povray;
+file_data->read_file = NULL;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = FDF;
+file_data->group = FDF;
+file_data->menu = TRUE;
+file_data->label = g_strdup("SIESTA");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "fdf");
+file_data->write_file = write_fdf;
+file_data->read_file = read_fdf;
+file_data->read_frame = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = SIESTA_OUT;
+file_data->group = FDF;
+file_data->menu = FALSE;
+file_data->label = g_strdup("SIESTA output");
+file_data->ext = NULL;
+file_data->ext = g_slist_prepend(file_data->ext, "sout");
+file_data->ext = g_slist_prepend(file_data->ext, "sot");
+file_data->write_file = NULL;
+file_data->read_file = read_sout;
+file_data->read_frame = read_sout_frame;
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
@@ -302,21 +454,58 @@ file_data->ext = NULL;
 file_data->ext = g_slist_prepend(file_data->ext, "xml");
 file_data->write_file = write_xml;
 file_data->read_file = read_xml;
+file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
 /* supported file type */
 file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
-file_data->id = ABINIT_OUT;
-file_data->group = ABINIT;
-file_data->menu = FALSE;
-file_data->label = g_strdup("SIESTA Output");
+file_data->id = XTL;
+file_data->group = XTL;
+file_data->menu = TRUE;
+file_data->label = g_strdup("XTL");
 file_data->ext = NULL;
-file_data->ext = g_slist_prepend(file_data->ext, "about");
-file_data->ext = g_slist_prepend(file_data->ext, "abot");
-file_data->write_file = NULL;
-file_data->read_file = read_about;
+file_data->ext = g_slist_append(file_data->ext, "xtl");
+file_data->write_file = write_xtl;
+file_data->read_file = read_xtl;
+file_data->read_frame = NULL;                   /* frame reading */
 sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
 
+/* supported file type */
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = XYZ;
+file_data->group = XYZ;
+file_data->menu = TRUE;
+file_data->label = g_strdup("XYZ");
+file_data->ext = NULL;
+file_data->ext = g_slist_append(file_data->ext, "xyz");
+file_data->write_file = write_xyz;
+file_data->read_file = read_xyz;
+file_data->read_frame = NULL;                   /* frame reading */
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+/* supported file type */
+/*
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = BABEL;
+file_data->group = DATA;
+file_data->menu = TRUE;
+file_data->label = g_strdup("Filterable types");
+file_data->ext = NULL;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+
+file_data = (struct file_pak *) g_malloc(sizeof(struct file_pak));
+file_data->id = AUTO;
+file_data->group = BABEL;
+file_data->menu = TRUE;
+file_data->label = g_strdup("auto select filter");
+file_data->ext = NULL;
+file_data->write_file = NULL;
+file_data->read_file = NULL;
+sysenv.file_list = g_slist_prepend(sysenv.file_list, file_data);
+*/
+
 sysenv.file_list = g_slist_reverse(sysenv.file_list);
 }
 
@@ -346,16 +535,16 @@ GDir *dir;
 GSList *files=NULL;
 
 /* ensure we can go up a directory */
-files = g_slist_prepend(files, "..");
+files = g_slist_prepend(files, g_strdup(".."));
 
+/* build the directory list */
 dir = g_dir_open(path, 0, NULL);
-do
+name = g_dir_read_name(dir);
+while (name)
   {
+  files = g_slist_prepend(files, g_strdup(name));
   name = g_dir_read_name(dir);
-  if (name)
-    files = g_slist_prepend(files, g_strdup(name));
   }
-while (name);
 g_dir_close(dir);
 
 if (sort)
@@ -369,7 +558,7 @@ return(files);
 /************************************************/
 /* returns pointer to file info if found, NULL otherwise */
 #define DEBUG_GET_FILE_INFO 0
-struct file_pak *get_file_info(const gpointer *ptr, gint type)
+struct file_pak *get_file_info(gpointer ptr, gint type)
 {
 gint code=-1;
 gchar *text=NULL, *ext;
@@ -383,7 +572,7 @@ g_return_val_if_fail(ptr != NULL, NULL);
 switch(type)
   {
   case BY_LABEL:
-    text = g_strdup((gchar *) ptr);
+    text = (gchar *) ptr;
 #if DEBUG_GET_FILE_INFO
 printf("Searching for type [%s]\n", text);
 #endif
@@ -395,7 +584,7 @@ printf("Searching for type [%s]\n", text);
     if (!ext)
       return(NULL);
     else
-      text = g_strdup(++ext); /* skip the '.' */
+      text = ++ext; /* skip the '.' */
 #if DEBUG_GET_FILE_INFO
 printf("Searching for extension [%s]\n", text);
 #endif
@@ -470,6 +659,50 @@ printf("Matched: %s\n", file_data->label);
 return(NULL);
 }
 
+/**************************/
+/* detect valid filetypes */
+/**************************/
+gint file_extension_valid(gchar *name)
+{
+gint len;
+gchar *ext, *text;
+GSList *item, *list;
+struct file_pak *file;
+
+/* locate the extension */
+ext = find_char(name, '.', LAST);
+if (!ext)
+  return(FALSE);
+else
+  ext++; /* skip the '.' */
+len = strlen(ext);
+
+/* compare against all types, since we now permit duplicate extensions */ 
+for (list=sysenv.file_list ; list ; list=g_slist_next(list))
+  {
+  file = list->data;
+
+  for (item=file->ext ; item ; item=g_slist_next(item))
+    {
+    text = item->data;
+    if (strlen(text) == len)
+      {
+      if (g_ascii_strcasecmp(text, ext) == 0)
+        {
+        if (sysenv.file_type == DATA)
+          return(TRUE);
+        else
+          {
+          if (sysenv.file_type == file->group)
+            return(TRUE);
+          }
+        }
+      }
+    }
+  }
+return(FALSE);
+}
+
 /*******************************************************/
 /* get an unused BASENAME (of supplied extension type) */
 /*******************************************************/
@@ -622,6 +855,34 @@ g_string_free(buff, FALSE);
 return(line);
 }
 
+
+/**********************************************************/
+/* format value as a float printed to dp places plus unit */
+/**********************************************************/
+gchar *format_value_and_units(gchar *string, gint dp)
+{
+gint num_tokens;
+GString *text, *format_string;
+gchar *text_str, *units, **buff;
+gdouble float_val;
+
+float_val = str_to_float(string);
+buff = tokenize(string, &num_tokens);
+if (num_tokens < 2)
+  units = "";
+else
+  units = *(buff+1);
+format_string = g_string_new("");
+g_string_append_printf(format_string, "%%.%df %%s", dp);
+text = g_string_new("");
+g_string_append_printf(text, format_string->str, float_val, units);
+text_str = text->str;
+g_string_free(format_string, TRUE); 
+g_string_free(text, FALSE); 
+g_strfreev(buff);
+return (text_str);
+}
+
 /********************/
 /* BABEL conversion */
 /********************/
@@ -709,10 +970,13 @@ gint i, flag=99;
 gchar *filename;
 fpos_t *offset;
 FILE *fp2;
+struct file_pak *file;
 
 g_assert(fp != NULL);
 g_assert(model != NULL);
 
+file = get_file_info(GINT_TO_POINTER(model->id), BY_FILE_ID);
+
 /* position the file pointer */
 if (model->id != GULP)
   {
@@ -724,26 +988,66 @@ if (model->id != GULP)
     printf("Error positioning file pointer.\n");
     return(2);
     }
+/* use supplied routine (if available) */
+  if (file->read_frame)
+    flag = file->read_frame(fp, model);
+  else
+    show_text(ERROR, "No animation read routine.\n");
   }
-
-/* perform the read */
-switch(model->id)
+else
   {
-  case BIOSYM:
-    flag = read_arc_frame(fp, model);
-    break;
+/* GULP exception */
+  if (!n)
+    {
+/* NEW - 0th frame exception - load original GULP coords */
+    if (model->cores)
+      g_assert(model->gulp.orig_cores != NULL);
+    if (model->shels)
+      g_assert(model->gulp.orig_shells != NULL);
+
+/* free connectivity lists */
+    free_slist(model->bonds);
+    free_slist(model->moles);
+    model->bonds = NULL;
+    model->moles = NULL;
+
+/* restore coords */
+    free_slist(model->cores);
+    free_slist(model->shels);
+    model->cores = dup_core_list(model->gulp.orig_cores);
+    model->shels = dup_shell_list(model->gulp.orig_shells);
+
+/* restore lattice */
+/*
+      model->fractional = model->gulp.orig_fractional;
+*/
+/* FIXME - I don't know why this works (and the above doesn't)... */
+    model->fractional = TRUE;
+    model->construct_pbc = model->gulp.orig_construct_pbc;
+    memcpy(model->latmat, model->gulp.orig_latmat, 9*sizeof(gdouble));
+    memcpy(model->pbc, model->gulp.orig_pbc, 6*sizeof(gdouble));
+
+/* turn space group lookup on */
+    model->sginfo.lookup = TRUE;
+ 
+    flag = 0;
+    }
+  else
+    {
+/* turn off space group lookup */
+/* trg files contain ALL atoms & symmetry may be broken */
+    model->sginfo.lookup = FALSE;
 
-  case GULP:
 /* FIXME - trj files resist every attempt at random access... why??? */
-/* FIXME - ugly hack to get the nth frame */
     filename = g_strdup_printf("%s/%s", sysenv.cwd, model->gulp.trj_file);
     fp2 = fopen(filename, "r");
     if (!fp2)
       {
       printf("Failed to open: %s\n", filename);
       g_free(filename);
-      break;
+      return(3);
       }
+/* FIXME - ugly hack to get the nth frame */
     read_trj_header(fp2, model);
     for (i=0 ; i<n-1 ; i++)
       read_trj_frame(fp2, model, FALSE);
@@ -751,25 +1055,8 @@ switch(model->id)
     fclose(fp2);
     g_free(filename);
     flag = 0;
-    break;
-
-  case SIESTA_OUT:
-    flag = read_sout_frame(fp, model);
-    break;
-
-  case GAMESS_OUT:
-    flag = read_gms_out_frame(fp, model);
-    break;
-
-  case ABINIT_OUT:
-    flag = read_about_frame(fp, model);
-    break;
-    
-  case PDB:
-    flag = read_pdb_frame(fp, model);
-    break;
+    }
   }
-
 return(flag);
 }
 
@@ -779,14 +1066,30 @@ return(flag);
 #define DEBUG_FILE_LOAD 0
 void file_load(gchar *filename, struct model_pak *mdata)
 {
-gint i, j;
-gint flag, model, status=-1;
+gint j;
+gint flag, status=-1;
 gchar *fullname;
+GSList *list;
 struct model_pak *data;
 struct file_pak *file_data;
 
-/* get the model type */
-file_data = get_file_info((gpointer *) filename, BY_EXTENSION);
+#if DEBUG_FILE_LOAD
+printf("loading: [%s] into: %p\n", filename, mdata);
+#endif
+
+/* get the file structure required to parse the file */
+switch (sysenv.file_type)
+  {
+/* cases for which duplicate extensions occur */
+  case RIETICA:
+  case GAMESS:
+    file_data = get_file_info(GINT_TO_POINTER(sysenv.file_type), BY_FILE_ID);
+    break;
+
+/* cases for which extensions must be used to determine type (eg .gin/.got) */
+  default:
+    file_data = get_file_info((gpointer *) filename, BY_EXTENSION);
+  }
 if (!file_data)
   return;
 
@@ -794,35 +1097,39 @@ if (!file_data)
 printf("Using file load routine for: [%s] files.\n", file_data->label);
 #endif
 
-if (mdata == NULL)
+if (mdata)
+  data = mdata;
+else
   {
 /* get the new model number */
-  model = sysenv.num_models;
-  data = model_ptr(model, ASSIGN);
+  data = model_new();
   if (!data)
     {
     show_text(ERROR, "Model memory allocation failed.\n");
     return;
     }
   }
-else
-  {
-  data = mdata;
-  model = mdata->number;
-  }
-  
+
 /* group babel types together for this */
 if (file_data->group == BABEL)
   data->id = file_data->group;
 else
   data->id = file_data->id;
 
-/* NEW - cleaner file read/write */
+/* read file if exists, else try prepending current working directory */
 if (file_data->read_file)
   {
-  fullname =  g_build_filename(sysenv.cwd, filename, NULL);
-  status = file_data->read_file(fullname, data);
-  g_free(fullname);
+  property_add_ranked(1, "File Type", file_data->label, data);
+  if (g_path_is_absolute(filename))
+    {
+    status = file_data->read_file(filename, data);
+    }
+  else
+    {
+    fullname = g_build_filename(sysenv.cwd, filename, NULL);
+    status = file_data->read_file(fullname, data);
+    g_free(fullname);
+    }
   }
 else
   show_text(ERROR, "No read routine for this type. ");
@@ -835,25 +1142,32 @@ if (status)
   printf("Load failed, error code: %d\n",status);
 #endif
 
-  model_ptr(model, RELEASE);
+  model_delete(data);
   canvas_shuffle();
   }
 else
   {
 /* we don't know how many new models were loaded so */
 /* scan through them all & check for initialization */
-  for (i=0 ; i<sysenv.num_models ; i++)
+  for (list=sysenv.mal ; list ; list=g_slist_next(list))
     {
-    data = model_ptr(i, RECALL);
+    data = list->data;
 
 /* skip if already on the tree */
     if (data->grafted)
       continue;
+
+/* NEW */
+if (data->num_atoms > 1000)
+  {
+  core_render_mode_set(STICK, data->cores);
+  }
+
 /* surfaces are always conv by default */
     if (data->periodic == 2)
       data->gulp.method = CONV;
 /* not on tree - must have just been loaded */
-    new_tree_item(data->number, APPEND);
+    tree_model_add(data);
 
 /* create gulp supercells */
     flag=0;
@@ -862,19 +1176,36 @@ else
       if (data->gulp.super[j] > 1)
         {
         data->image_limit[2*j+1] = data->gulp.super[j];
+        data->expected_cores *= data->gulp.super[j];
+        data->expected_shells *= data->gulp.super[j];
         flag++;
         }
       }
     if (flag)
       {
-      update_images(CREATE, data);
-      make_supercell(NULL, data);
+      space_make_images(CREATE, data);
+      space_make_supercell(data);
       }
+
+/* NEW - store initial frame, as trajectory animation will overwrite */
+if (data->gulp.trj_file && !data->gulp.orig_cores)
+  {
+/* FIXME - saving fractional type here instead of at the end of read_gulp() stuffs things up  - why? */
+/* NB: we need to save the coords here (and not earlier) in case a supercell was created */
+
+  memcpy(data->gulp.orig_pbc, data->pbc, 6*sizeof(gdouble));
+  memcpy(data->gulp.orig_latmat, data->latmat, 9*sizeof(gdouble));
+  data->gulp.orig_cores = dup_core_list(data->cores);
+  data->gulp.orig_shells = dup_shell_list(data->shels);
+  }
+
+    sysenv.active_model = data;
     }
   }
 
 /* finished - only now we destroy the file dialog */
-close_dialog_type(FILE_SELECT);
+dialog_destroy_type(FILE_SELECT);
+tree_select_active();
 }
 
 /****************/
@@ -882,7 +1213,7 @@ close_dialog_type(FILE_SELECT);
 /***************/
 void file_load_dialog(void)
 {
-file_dialog("Load file", NULL, (gpointer) file_load, sysenv.file_type);
+file_dialog("Load file", NULL, FILE_LOAD, (gpointer) file_load, sysenv.file_type);
 }
 
 /**********************************/
@@ -896,21 +1227,41 @@ struct file_pak *file_data;
 
 /* setup & checks */
 if (!model)
-  model = model_ptr(sysenv.active, RECALL);
+  model = sysenv.active_model;
 
 if (!filename || !model)
   return(1);
 
 file_data = get_file_info((gpointer *) filename, BY_EXTENSION);
-if (!file_data)
-  return(2);
 
-id = file_data->id;
+if (file_data)
+  {
+/* use extension */
+  id = file_data->id;
+  strcpy(model->filename, filename);
+  g_free(model->basename);
+  model->basename = strdup_basename(filename);
+  }
+else
+  {
+  gchar *ext;
 
-/* overwrite the old filenames with the new */
-strcpy(model->filename, filename);
-g_free(model->basename);
-model->basename = strdup_basename(filename);
+/* no extension - attempt to use filter type */
+  file_data = get_file_info(GINT_TO_POINTER(sysenv.file_type), BY_FILE_ID);
+  if (!file_data)
+    return(2);
+  ext = g_slist_nth_data(file_data->ext, 0);
+  if (!ext)
+    return(2);
+
+  id = file_data->id;
+  g_free(model->basename);
+  model->basename = strdup_basename(filename);
+
+  name = g_strdup_printf("%s.%s", model->basename, ext);
+  strcpy(model->filename, name);
+  g_free(name);
+  }
 
 /* file info indicates routine to call */
 if (file_data->write_file == NULL)
@@ -932,8 +1283,8 @@ if (ret)
 else
   {
   show_text(STANDARD, "Saved model.\n");
-  close_dialog_type(FILE_SELECT);
-  new_tree_item(model->number, REPLACE);
+  dialog_destroy_type(FILE_SELECT);
+  tree_model_refresh(model);
   redraw_canvas(SINGLE);
   }
 
@@ -948,7 +1299,7 @@ void file_save(void)
 gchar *name;
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 if (!model)
   return;
 
@@ -966,10 +1317,11 @@ void file_save_dialog(void)
 gchar *text=NULL;
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 if (model)
   text = model->basename;
 
-file_dialog("File save", text, (gpointer) file_save_as, sysenv.file_type);
+file_dialog("File save", text, FILE_SAVE, (gpointer) file_save_as,
+                                                sysenv.file_type);
 }
 
