@@ -46,6 +46,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "file.h"
 #include "parse.h"
 #include "morph.h"
+#include "model.h"
 #include "matrix.h"
 #include "space.h"
 #include "surface.h"
@@ -110,7 +111,7 @@ switch(data->morph_type)
 for (plist=data->planes ; plist ; plist=g_slist_next(plist))
   {
   pdata = (struct plane_pak *) plist->data;
-  if (!pdata->primary || !pdata->present)
+  if (!pdata->primary)
     continue;
 
   fprintf(fp,"miller:  %3d %3d %3d\n", 
@@ -139,13 +140,14 @@ return(0);
 gint load_planes(gchar *filename, struct model_pak *data)
 {
 gint i, h, k, l, num_tokens;
+gint cflag=FALSE, sflag=FALSE, gflag=FALSE;
 gint *sym_hkl;
 gdouble m[3];
 gchar **buff;
-GSList *plist;
+GSList *list, *new_planes;
 GSList *sym_equiv, *sym_list;
-struct plane_pak *pdata=NULL, *pnew=NULL;
-struct shift_pak *sdata=NULL;
+struct plane_pak *plane=NULL, *new_plane=NULL;
+struct shift_pak *shift=NULL;
 FILE *fp;
 
 fp = fopen(filename, "rt");
@@ -153,6 +155,7 @@ if (!fp)
   return(1);
 
 /* get next line */
+new_planes = NULL;
 for (;;)
   {
   buff = get_tokenized_line(fp, &num_tokens);
@@ -168,6 +171,7 @@ for (;;)
       {
       if (num_tokens >= 7)
         {
+        cflag=TRUE;
         data->pbc[0] = str_to_float(*(buff+1));
         data->pbc[1] = str_to_float(*(buff+2));
         data->pbc[2] = str_to_float(*(buff+3));
@@ -185,9 +189,9 @@ for (;;)
 /* space group */
     if (g_ascii_strncasecmp(*buff,"space",5) == 0)
       {
-      g_free(data->sginfo.spacename);
+      sflag=TRUE;
       data->sginfo.spacename = g_strjoinv(" ", buff+1);
-      genpos(data);
+      data->sginfo.spacenum = 0;
       }
 
 /* default morphology type */
@@ -219,121 +223,132 @@ for (;;)
 /* process miller line */
   if (g_ascii_strncasecmp(*buff,"miller",6) == 0)
     {
+/* init space group (latmat? - if so, remove the make_latmat() in cell parse) */
+    if (cflag && sflag)
+      {
+      if (!gflag)
+        {
+        if (space_lookup(data))
+          printf("Error in space group lookup.\n");
+        gflag = TRUE;
+        }
+      }
+    else
+      {
+      if (data->id == MORPH)
+        {
+        printf("load_planes() error: miller encountered before space or cell.\n");
+        return(2);
+        }
+      }
+
     if (num_tokens >= 4)
       {
       h = (gint) str_to_float(*(buff+1));
       k = (gint) str_to_float(*(buff+2));
       l = (gint) str_to_float(*(buff+3));
 #if DEBUG_LOAD_PLANES
-printf("creating plane: %d %d %d\n", h, k, l);
+printf("read plane: %d %d %d\n", h, k, l);
 #endif
       VEC3SET(m, h, k, l);
-      pdata = fn_seek_plane(m, data);
-      if (!pdata)
+      plane = plane_find(m, data);
+      if (!plane)
         {
-        pdata = fn_make_plane(m, data);
-/* append to preserve order (eg import on existing plane set) */
-        if (pdata)
-          data->planes = g_slist_append(data->planes, pdata);
+        plane = plane_new(m, data);
+        if (plane)
+          new_planes = g_slist_prepend(new_planes, plane);
         }
       }
     }
 
 /* potential data */
-  if (num_tokens >= 8 && pdata)
+  if (num_tokens >= 8 && plane)
     {
 /* NB: use create_shift(), as it sets some important defaults */
-    sdata = create_shift(0.0);
-    if (sdata)
+    shift = shift_new(0.0);
+    if (shift)
       {
-      sdata->shift = str_to_float(*(buff+0));
-      sdata->region[0] = str_to_float(*(buff+1));
-      sdata->region[1] = str_to_float(*(buff+2));
-      sdata->esurf[0] = str_to_float(*(buff+3));
-      sdata->eatt[0] = str_to_float(*(buff+4));
-      sdata->esurf[1] = str_to_float(*(buff+5));
-      sdata->eatt[1] = str_to_float(*(buff+6));
-      sdata->gnorm = str_to_float(*(buff+7));
+      shift->shift = str_to_float(*(buff+0));
+      shift->region[0] = str_to_float(*(buff+1));
+      shift->region[1] = str_to_float(*(buff+2));
+      shift->esurf[0] = str_to_float(*(buff+3));
+      shift->eatt[0] = str_to_float(*(buff+4));
+      shift->esurf[1] = str_to_float(*(buff+5));
+      shift->eatt[1] = str_to_float(*(buff+6));
+      shift->gnorm = str_to_float(*(buff+7));
 #if DEBUG_LOAD_PLANES
-printf("adding shift: %f\n", sdata->shift);
+printf("adding shift: %f\n", shift->shift);
 #endif
 /* append to preserve order (eg import on existing plane set) */
-      pdata->shifts = g_slist_append(pdata->shifts, sdata);
+      plane->shifts = g_slist_append(plane->shifts, shift);
       }
     }
   g_strfreev(buff);
   }
+data->planes = g_slist_concat(data->planes, g_slist_reverse(new_planes));
 
 /* compute dhkl's for the plane's list */
-plist = data->planes;
-while(plist != NULL)
+for (list=data->planes ; list ; list=g_slist_next(list))
   {
-  pdata = (struct plane_pak *) plist->data;
+  plane = list->data;
 
 /* create default shift if none found */
-  if (!pdata->shifts)
+  if (!plane->shifts)
     {
-    sdata = create_shift(0.0);
-    if (sdata)
-      pdata->shifts = g_slist_append(pdata->shifts, sdata);
+    shift = shift_new(0.0);
+    if (shift)
+      plane->shifts = g_slist_append(plane->shifts, shift);
     }
-
 /* get best energy for the plane */
-  update_plane_energy(pdata, data);
-
-  plist = g_slist_next(plist);
-  i++;
+  update_plane_energy(plane, data);
   }
 
 /* compute symmetry related faces */
 i=0;
-plist = data->planes;
-while (plist != NULL)
+new_planes = NULL;
+for (list=data->planes ; list ; list=g_slist_next(list))
   {
-  pdata = (struct plane_pak *) plist->data;
-/* once we reach the non-primary planes, we're done */
-  if (!pdata->primary)
-    break;
+  plane = (struct plane_pak *) list->data;
 
-  sym_list = sym_equiv = get_facet_equiv(data, pdata->index);
-/* skip the 1st, we've already assigned it */
-  sym_list = g_slist_next(sym_list);
-  while(sym_list != NULL)
+  sym_equiv = get_facet_equiv(data, plane->index);
+
+#if DEBUG_LOAD_PLANES
+printf(" *** {%d %d %d}\n", plane->index[0], plane->index[1], plane->index[2]);
+#endif
+
+/* skip the 1st, as this is just the original hkl (ie plane->index) */
+  sym_list = g_slist_next(sym_equiv);
+  while (sym_list)
     {
 /* get the (normalized) hkl's */
-    sym_hkl = (gint *) sym_list->data;
+    sym_hkl = sym_list->data;
+    ARR3SET(m, sym_hkl);
 
-ARR3SET(m, sym_hkl);
-
-    pnew = fn_make_plane(m, data);
-    if (pnew)
+    new_plane = plane_new(m, data);
+    if (plane)
       {
-      pnew->primary = FALSE;
-      pnew->dhkl = pdata->dhkl;
-      pnew->esurf[0] = pdata->esurf[0];
-      pnew->esurf[1] = pdata->esurf[1];
-      pnew->eatt[0] = pdata->eatt[0];
-      pnew->eatt[1] = pdata->eatt[1];
+      new_plane->primary = FALSE;
+      new_plane->dhkl = plane->dhkl;
+      new_plane->esurf[0] = plane->esurf[0];
+      new_plane->esurf[1] = plane->esurf[1];
+      new_plane->eatt[0] = plane->eatt[0];
+      new_plane->eatt[1] = plane->eatt[1];
 
-      data->planes = g_slist_append(data->planes, pnew);
+#if DEBUG_LOAD_PLANES
+printf("   > (%d %d %d)\n",
+       new_plane->index[0], new_plane->index[1], new_plane->index[2]);
+#endif
+
+      new_planes = g_slist_prepend(new_planes, new_plane);
       }
     sym_list = g_slist_next(sym_list);
     }
-
-/* wipe the old list */
-  g_slist_free(sym_equiv);
-  plist = g_slist_next(plist);
+  free_slist(sym_equiv);
   }
 
-i=0;
-plist = data->planes;
-while (plist != NULL)
-  {
-  pdata = (struct plane_pak *) plist->data;
-  plist = g_slist_next(plist);
-  i++;
-  }
-data->num_planes = i;
+/* update list with the new (symmetry related) planes */
+data->planes = g_slist_concat(data->planes, new_planes);
+data->num_planes = g_slist_length(data->planes);
 
 fclose(fp);
 return(0);
@@ -353,7 +368,7 @@ gdouble vec[3];
 printf("[%d %d %d] and (%d %d %d) are ", f1[0], f1[1], f1[2], f2[0], f2[1], f2[2]);
 #endif
 
-if (data->sginfo.spacenum)
+if (data->sginfo.spacenum > 0)
   {
   for (i=0 ; i<data->sginfo.order ; i++)
     {
@@ -387,18 +402,26 @@ return(FALSE);
 GSList *get_facet_equiv(struct model_pak *data, gint *f1)
 {
 gint i, flag, *f2, index[3];
-gdouble vec[3];
+gdouble d1, d2, vec[3];
 GSList *flist=NULL, *list=NULL;
 
 g_return_val_if_fail(data != NULL, NULL);
 g_return_val_if_fail(f1 != NULL, NULL);
 
+/* NEW */
+ARR3SET(vec, f1);
+vecmat(data->rlatmat, vec);
+d1 = 1.0/VEC3MAG(vec);
+
 #if DEBUG_GET_FACET_EQUIV
-printf("search for equiv faces to (%d %d %d)\n", f1[0], f1[1], f1[2]);
+printf("search for equiv faces to (%d %d %d) (Dhkl = %f)\n",
+        f1[0], f1[1], f1[2], d1);
 #endif
 
 /* add the supplied face to the list (needed to eliminate repetitions) */
-flist = g_slist_prepend(flist, (gpointer) f1);
+f2 = g_malloc(3*sizeof(gint));
+ARR3SET(f2, f1);
+flist = g_slist_prepend(flist, (gpointer) f2);
 
 if (data->sginfo.spacenum)
   {
@@ -409,9 +432,24 @@ if (data->sginfo.spacenum)
     ARR3SET(vec, f1);
     vecmat(*(data->sginfo.matrix+i), vec);
     ARR3SET(index, vec);
+
+/* NEW - weed out symop generated faces with different Dhkl values */
+/* FIXME - why does this happen??? */
+vecmat(data->rlatmat, vec);
+d2 = 1.0/VEC3MAG(vec);
+
 #if DEBUG_GET_FACET_EQUIV
-printf("candidate: (%d %d %d) ", index[0], index[1], index[2]);
+printf("candidate: (%3d %3d %3d) (Dhkl=%7.4f)", index[0], index[1], index[2], d2);
 #endif
+
+if (fabs(d2-d1) > FRACTION_TOLERANCE)
+  {
+#if DEBUG_GET_FACET_EQUIV
+printf("[NO : dhkl mis-match]\n");
+#endif
+  continue;
+  }
+
 /* add new hkl if not found in the list */
     flag = 0;
     list = flist;
@@ -426,15 +464,16 @@ printf("candidate: (%d %d %d) ", index[0], index[1], index[2]);
       {
       f2 = g_malloc(3*sizeof(gint));
       ARR3SET(f2, index);
-      flist = g_slist_prepend(flist, (gpointer) f2);
+      flist = g_slist_prepend(flist, f2);
+
 #if DEBUG_GET_FACET_EQUIV
-printf("[YES]\n");
+printf("[YES : symop %d]\n", i);
 #endif
       }
 #if DEBUG_GET_FACET_EQUIV
     else
       {
-printf("[NO]\n");
+printf("[NO : already exists]\n");
       }
 #endif
     }
@@ -527,6 +566,10 @@ gint facet_visible(struct model_pak *data, struct plane_pak *plane)
 {
 gdouble a, norm[3], vec[3];
 
+/* NEW - an edge (2 vertices) is not to be treated as visible */
+if (g_slist_length(plane->vertices) < 3)
+  return(FALSE);
+
 /* get (unrotated!) normal */
 ARR3SET(norm, plane->norm);
 
@@ -548,13 +591,13 @@ if (a < PI/2.0)
 #if DEBUG_FACET
 printf("view: %5.2f,%5.2f,%5.2f (%5.1f) YES\n",vec[0],vec[1],vec[2],180.0*a/PI);
 #endif
-  return(1);
+  return(TRUE);
   }
 /* else... */
 #if DEBUG_FACET
 printf("view: %5.2f,%5.2f,%5.2f (%5.1f) NO\n",vec[0],vec[1],vec[2],180.0*a/PI);
 #endif
-return(0);
+return(FALSE);
 }
 
 /***********************************/
@@ -589,22 +632,19 @@ return(0);
 #define DEBUG_READ_GMF 0
 gint read_gmf(gchar *filename, struct model_pak *data)
 {
-gint model;
 gdouble len;
 gdouble norm[3];
 GSList *plist;
 struct plane_pak *plane;
 
 /* checks */
-g_return_val_if_fail(data != NULL, 1);
-g_return_val_if_fail(filename != NULL, 1);
+g_assert(data != NULL);
+g_assert(filename != NULL);
 
-sysenv.active = model = data->number;
 data->mode = FREE;
 /* setup model parameters */
 data->id = MORPH;
 data->axes_type = OTHER;
-data->hpr = TRUE;
 data->morph_label = TRUE;
 strcpy(data->filename, filename);
 g_free(data->basename);
@@ -613,8 +653,8 @@ data->basename = strdup_basename(data->filename);
 /* read from a gmf file */
 if (load_planes(filename, data))
   {
-  printf("File load failed, check %s.\n",data->filename);
-  delete_model(model);
+  printf("File load failed, check %s.\n", data->filename);
+  model_delete(data);
   return(1);
   }
 
@@ -640,14 +680,15 @@ while (plist != NULL)
     {
 /* make it so gdis safely ignores such a thing */
     printf("Warning: 0 length face found.\n");
-    return(1);
+    return(2);
     }
   plist = g_slist_next(plist);
   }
 
 /* construct the hull (convert to H representation) */
 data->num_vertices=0;
-exec_cdd(data);
+if (exec_cdd(data))
+  return(3);
 
 #if DEBUG_READ_GMF
 printf("-------------------------\n");
@@ -658,10 +699,7 @@ printf("-------------------------\n");
 
 /* test success */
 if (!g_slist_length(data->planes))
-  {
-  delete_model(model);
-  return(1);
-  }
+  return(4);
 
 /* vertices are cartesian - so a fudge is needed */
 prep_model(data);
