@@ -26,8 +26,11 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "edit.h"
 #include "file.h"
+#include "model.h"
 #include "parse.h"
+#include "spatial.h"
 #include "matrix.h"
 
 /* main structures */
@@ -39,7 +42,10 @@ enum
 XML_INACTIVE,
 XML_SYSTEM, XML_SYMMETRY, XML_CRYSTAL, XML_CELL, XML_CORE, XML_SHELL,
 XML_CELL_A, XML_CELL_B, XML_CELL_C,
-XML_CELL_ALPHA, XML_CELL_BETA, XML_CELL_GAMMA
+XML_CELL_ALPHA, XML_CELL_BETA, XML_CELL_GAMMA,
+XML_SPATIAL, XML_VERTEX,
+XML_TRANSFORM, XML_MATRIX, XML_VECTOR, XML_SCALAR,
+XML_PICTURE
 };
 
 gint xml_context = XML_INACTIVE;
@@ -62,7 +68,7 @@ while ((line = file_read_line(fp)))
   {
   type = g_strstrip(g_strndup(line, 6));
 
-  if (elem_type(type))
+  if (elem_symbol_test(type))
     {
 /* tokenize everything after the atom type */
     buff = tokenize(&line[6], &n);
@@ -118,7 +124,7 @@ printf("initial system: %s\n", *(values+i));
         {
 printf("adding system: %s\n", *(values+i));
 /* id not found - allocate new model & add to hash table */
-        xml_model = model_ptr(sysenv.num_models, ASSIGN);
+        xml_model = model_new();
         key = g_strdup(*(values+i));
         g_hash_table_insert(xml_system_table, key, xml_model);
         }
@@ -148,9 +154,7 @@ while (*(names+i))
 /* only init the cell parsing trigger if it's the full cell's data */
   if (g_ascii_strncasecmp(*(names+i), "spacegroup", 10) == 0)
     {
-    g_free(xml_model->sginfo.spacename);
     xml_model->sginfo.spacename = g_strdup(*(values+i));
-printf(" space group: %s\n", xml_model->sginfo.spacename);
     }
   i++;
   }
@@ -165,6 +169,7 @@ gint i;
 
 /* process attributes */
 i=0;
+xml_context = XML_INACTIVE;
 while (*(names+i))
   {
   if (g_ascii_strncasecmp(*(names+i), "dictRef\0", 8) == 0)
@@ -175,8 +180,13 @@ while (*(names+i))
       xml_context = XML_CELL;
       xml_model->periodic = 3;
       }
-    else
-      xml_context = XML_INACTIVE;
+
+    if (g_ascii_strncasecmp(*(values+i), "gulp:surface\0", 13) == 0)
+      {
+      xml_context = XML_CELL;
+      xml_model->periodic = 2;
+      }
+
     }
   i++;
   }
@@ -346,6 +356,105 @@ while (*(names+i))
   }
 }
 
+/*******************/
+/* spatial parsing */
+/*******************/
+struct spatial_pak *xml_spatial;
+
+/************************/
+/* create a new spatial */
+/************************/
+void xml_parse_spatial(const gchar **names, const gchar **values)
+{
+gint i;
+
+g_assert(xml_model != NULL);
+
+xml_spatial = g_malloc(sizeof(struct spatial_pak));
+xml_spatial->type = -1;
+xml_spatial->method = -1;
+xml_spatial->data = NULL;
+xml_model->spatial = g_slist_prepend(xml_model->spatial, xml_spatial);
+
+i=0;
+while (*(names+i))
+  {
+  if (g_ascii_strncasecmp(*(names+i), "type", 4) == 0)
+    xml_spatial->type = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "method", 6) == 0)
+    xml_spatial->method = str_to_float(*(values+i));
+
+  i++;
+  }
+}
+
+/********************/
+/* populate spatial */
+/********************/
+void xml_parse_vertex(const gchar **names, const gchar **values)
+{
+gint i=0;
+struct vec_pak *vec;
+
+g_assert(xml_spatial != NULL);
+
+vec = g_malloc(sizeof(struct vec_pak));
+xml_spatial->data = g_slist_prepend(xml_spatial->data, vec);
+
+while (*(names+i))
+  {
+  if (g_ascii_strncasecmp(*(names+i), "red", 3) == 0)
+    vec->colour[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "green", 5) == 0)
+    vec->colour[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "blue", 4) == 0)
+    vec->colour[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "x3", 2) == 0)
+    vec->x[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "y3", 2) == 0)
+    vec->x[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "z3", 2) == 0)
+    vec->x[2] = str_to_float(*(values+i));
+
+  if (g_ascii_strncasecmp(*(names+i), "nx", 2) == 0)
+    vec->n[0] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "ny", 2) == 0)
+    vec->n[1] = str_to_float(*(values+i));
+  if (g_ascii_strncasecmp(*(names+i), "nz", 2) == 0)
+    vec->n[2] = str_to_float(*(values+i));
+
+  i++;
+  }
+}
+
+/**************************/
+/* transformation parsing */
+/**************************/
+struct transform_pak *xml_transform=NULL;
+
+/*******************************/
+/* create a new transformation */
+/*******************************/
+void xml_parse_transform(const gchar **names, const gchar **values)
+{
+gint i=0;
+
+g_assert(xml_model != NULL);
+
+xml_transform = g_malloc(sizeof(struct transform_pak));
+xml_model->transform_list = g_slist_append(xml_model->transform_list, xml_transform);
+
+while (*(names+i))
+  {
+  if (g_ascii_strncasecmp(*(names+i), "id", 2) == 0)
+    xml_transform->id = str_to_float(*(values+i));
+
+  i++;
+  }
+}
+
 /****************/
 /* process text */
 /****************/
@@ -355,6 +464,8 @@ void xml_parse_text(GMarkupParseContext *context,
                     gpointer user_data,
                     GError **error)
 {
+gint i;
+gchar **buff;
 
 switch (xml_context)
   {
@@ -389,6 +500,43 @@ switch (xml_context)
 /* revert to general cell value context */
     xml_context = XML_CELL;
     break;
+
+  case XML_PICTURE:
+    xml_model->picture_list = g_slist_append(xml_model->picture_list,
+                                         g_strstrip(g_strdup(text)));
+    break;
+
+  case XML_MATRIX:
+/* FIXME - bit of a hack here */
+    if (xml_transform)
+      {
+      buff = g_strsplit(text, " ", 10);
+      for (i=0 ; i<9 ; i++)
+        xml_transform->matrix[i] = str_to_float(*(buff+i));
+      g_strfreev(buff);
+      }
+    break;
+ 
+  case XML_VECTOR:
+/* FIXME - bit of a hack here */
+    if (xml_transform)
+      {
+      buff = g_strsplit(text, " ", 4);
+      for (i=0 ; i<3 ; i++)
+        xml_transform->vector[i] = str_to_float(*(buff+i));
+      g_strfreev(buff);
+      }
+    break;
+
+  case XML_SCALAR:
+/* FIXME - bit of a hack here */
+    if (xml_transform)
+      {
+      buff = g_strsplit(text, " ", 2);
+      xml_transform->scalar = str_to_float(*(buff+0));
+      g_strfreev(buff);
+      }
+    break;
   }
 }
 
@@ -408,25 +556,29 @@ model = (struct model_pak *) current_model;
 
 /* context switching control */
 if (g_ascii_strncasecmp(element_name, "system\0", 7) == 0)
-  {
   xml_context = XML_SYSTEM;
-  }
 if (g_ascii_strncasecmp(element_name, "crystal\0", 8) == 0)
-  {
   xml_context = XML_CRYSTAL;
-  }
 if (g_ascii_strncasecmp(element_name, "symmetry\0", 9) == 0)
-  {
   xml_context = XML_SYMMETRY;
-  }
 if (g_ascii_strncasecmp(element_name, "atom\0", 5) == 0)
-  {
   xml_context = XML_CORE;
-  }
 if (g_ascii_strncasecmp(element_name, "particle\0", 9) == 0)
-  {
   xml_context = XML_SHELL;
-  }
+if (g_ascii_strncasecmp(element_name, "spatial\0", 8) == 0)
+  xml_context = XML_SPATIAL;
+if (g_ascii_strncasecmp(element_name, "vertex\0", 7) == 0)
+  xml_context = XML_VERTEX;
+if (g_ascii_strncasecmp(element_name, "picture\0", 8) == 0)
+  xml_context = XML_PICTURE;
+if (g_ascii_strncasecmp(element_name, "transform\0", 10) == 0)
+  xml_context = XML_TRANSFORM;
+if (g_ascii_strncasecmp(element_name, "matrix\0", 7) == 0)
+  xml_context = XML_MATRIX;
+if (g_ascii_strncasecmp(element_name, "vector\0", 7) == 0)
+  xml_context = XML_VECTOR;
+if (g_ascii_strncasecmp(element_name, "scalar\0", 7) == 0)
+  xml_context = XML_SCALAR;
 
 /* context attribute parsing */
 switch (xml_context)
@@ -454,6 +606,18 @@ switch (xml_context)
   case XML_SYMMETRY:
     xml_parse_symmetry(attribute_names, attribute_values);
     return;
+
+  case XML_SPATIAL:
+    xml_parse_spatial(attribute_names, attribute_values);
+    return;
+
+  case XML_VERTEX:
+    xml_parse_vertex(attribute_names, attribute_values);
+    return;
+
+  case XML_TRANSFORM:
+    xml_parse_transform(attribute_names, attribute_values);
+    return;
   }
 }
 
@@ -485,6 +649,29 @@ if (g_ascii_strncasecmp(element_name, "particle\0", 5) == 0)
   xml_shell = NULL;
   xml_context = XML_CORE;
   }
+if (g_ascii_strncasecmp(element_name, "spatial\0", 8) == 0)
+  {
+  xml_context = XML_INACTIVE;
+  xml_spatial->data = g_slist_reverse(xml_spatial->data);
+  xml_spatial = NULL;
+  }
+if (g_ascii_strncasecmp(element_name, "vertex\0", 7) == 0)
+  xml_context = XML_SPATIAL;
+if (g_ascii_strncasecmp(element_name, "picture\0", 8) == 0)
+  xml_context = XML_INACTIVE;
+if (g_ascii_strncasecmp(element_name, "transform\0", 10) == 0)
+  {
+  xml_model->num_frames++;
+  xml_context = XML_INACTIVE;
+  xml_transform = NULL;
+  }
+
+if (g_ascii_strncasecmp(element_name, "matrix\0", 7) == 0)
+  xml_context = XML_INACTIVE;
+if (g_ascii_strncasecmp(element_name, "vector\0", 7) == 0)
+  xml_context = XML_INACTIVE;
+if (g_ascii_strncasecmp(element_name, "scalar\0", 7) == 0)
+  xml_context = XML_INACTIVE;
 }
 
 /******************************/
@@ -535,6 +722,7 @@ return(0);
 /*****************************/
 /* the main reading function */
 /*****************************/
+#define DEBUG_READ_XML 0
 gint read_xml(gchar *filename, struct model_pak *model)
 {
 gint i, n, num_models=0;
@@ -551,18 +739,26 @@ num_models++;
 
 fclose(fp);
 
+#if DEBUG_READ_XML
 printf("read_xml() : found %d models.\n", num_models);
+#endif
 
+/*
 n = sysenv.num_models;
 for (i=0 ; i<num_models ; i++)
   {
   n--;
   model = model_ptr(n, RECALL);
 
+#if DEBUG_READ_XML
 printf("read_xml() : prep'ing model %p (%d)\n", model, n);
+#endif
 
   prep_model(model);
   }
+*/
+
+prep_model(model);
 
 return(0);
 }
@@ -572,9 +768,13 @@ return(0);
 /*****************************/
 gint write_xml(gchar *filename, struct model_pak *model)
 {
+gint i;
 gdouble vec[3];
-GSList *list;
+GSList *list, *list2;
 struct core_pak *core;
+struct vec_pak *v;
+struct spatial_pak *spatial;
+struct transform_pak *transform;
 FILE *fp;
 
 fp = fopen(filename, "wt");
@@ -586,7 +786,17 @@ fprintf(fp, "<system>\n");
 /* periodicity */
 if (model->periodic)
   {
-  fprintf(fp, "  <crystal dictRef=\"gulp:fullcell\">\n");
+/* TODO - 1D */
+  switch (model->periodic)
+    {
+    case 2:
+      fprintf(fp, "  <crystal dictRef=\"gulp:surface\">\n");
+      break;
+
+    default:
+      fprintf(fp, "  <crystal dictRef=\"gulp:fullcell\">\n");
+      break;
+    }
   fprintf(fp, "    <cell dictRef=\"cml:a\"> %f </cell>\n", model->pbc[0]);
   fprintf(fp, "    <cell dictRef=\"cml:b\"> %f </cell>\n", model->pbc[1]);
   fprintf(fp, "    <cell dictRef=\"cml:c\"> %f </cell>\n", model->pbc[2]);
@@ -608,8 +818,64 @@ for (list=model->cores ; list ; list=g_slist_next(list))
   fprintf(fp, "  <atom elementType=\"%s\" x3=\"%f\" y3=\"%f\" z3=\"%f\">",
           core->label, vec[0], vec[1], vec[2]);
   fprintf(fp, "</atom>\n");
+/* TODO - shells as particles */
   }
 
+/* spatial data */
+for (list=model->spatial ; list ; list=g_slist_next(list))
+  {
+  spatial = (struct spatial_pak *) list->data;
+
+  fprintf(fp, "<spatial Type=\"%d\" Method=\"%d\">\n",
+                      spatial->type, spatial->method);
+
+  for (list2=spatial->data ; list2 ; list2=g_slist_next(list2))
+    {
+    v = (struct vec_pak *) list2->data;
+
+    fprintf(fp, "  <vertex red=\"%f\" green=\"%f\" blue=\"%f\"",
+                 v->colour[0], v->colour[1], v->colour[2]);
+ 
+    fprintf(fp, " x3=\"%f\" y3=\"%f\" z3=\"%f\"",
+                 v->x[0], v->x[1], v->x[2]);
+
+    fprintf(fp, " nx=\"%f\" ny=\"%f\" nz=\"%f\"></vertex>\n",
+                 v->n[0], v->n[1], v->n[2]);
+    }
+
+  fprintf(fp, "</spatial>\n");
+  }
+
+/* transform data */
+for (list=model->transform_list ; list ; list=g_slist_next(list))
+  {
+  transform = (struct transform_pak *) list->data;
+
+  fprintf(fp, "  <transform id=\"%d\">\n", transform->id);
+
+  fprintf(fp, "    <matrix>");
+  for (i=0 ; i<9 ; i++)
+    fprintf(fp, "%f ", transform->matrix[i]);
+  fprintf(fp, "</matrix>\n");
+
+  fprintf(fp, "    <vector>");
+  for (i=0 ; i<3 ; i++)
+    fprintf(fp, "%f ", transform->vector[i]);
+  fprintf(fp, "</vector>\n");
+
+  fprintf(fp, "    <scalar>");
+  fprintf(fp, "%f ", transform->scalar);
+  fprintf(fp, "</scalar>\n");
+  
+  fprintf(fp, "</transform>\n");
+  }
+
+
+/* pictures */
+for (list=model->picture_list ; list ; list=g_slist_next(list))
+  fprintf(fp, "<picture> %s </picture>\n", (gchar *) list->data);
+
+
 fprintf(fp, "</system>\n");
 
 fclose(fp);
