@@ -123,6 +123,7 @@ for (list=data->cores ; list ; list=g_slist_next(list))
     if (invert)
       vecmat(tmat, vec);
 
+/* FIXME - do the scattering column properly */
     fprintf(fp,"%4s %10.5f %10.5f %10.5f  0.0000  0.0000  1.0000   C 0+\n",
             core->label, vec[0], vec[1], vec[2]);
     }
@@ -140,7 +141,7 @@ return(0);
 #define DEBUG_READ_XYZ 0
 gint read_xtl(gchar *filename, struct model_pak *data)
 {
-gint i, num_tokens;
+gint i, offset, num_tokens;
 gchar **buff;
 struct core_pak *core;
 FILE *fp;
@@ -163,8 +164,12 @@ while(!fgetline(fp, line))
 for (;;)
   {
   buff = get_tokenized_line(fp, &num_tokens);
+/*
   if (!buff)
     break;
+*/
+  if (!num_tokens)
+    break;
 
 /* periodicity search */
   if (g_ascii_strncasecmp("dimension", *buff, 9) == 0)
@@ -181,7 +186,6 @@ for (;;)
 /* seek space group label */
       if (g_ascii_strncasecmp("label", *(buff+i), 5) == 0)
         {
-        g_free(data->sginfo.spacename);
         data->sginfo.spacename = g_strdup(*(buff+i+1));
 /* indicate that name should used in lookup */
         data->sginfo.spacenum = -1;
@@ -196,37 +200,43 @@ for (;;)
 /* pbc search */
   if (g_ascii_strncasecmp("cell", *buff, 4) == 0)
     {
-    g_strfreev(buff);
-/* get next line (if possible) */
-    buff = get_tokenized_line(fp, &num_tokens);
-    if (!buff)
-      return(2);
+/* use this line if it has more tokens, otherwise get the next */
+    offset = 1;
+    if (num_tokens == 1)
+      {
+      g_strfreev(buff);
+      buff = get_tokenized_line(fp, &num_tokens);
+      if (!buff)
+        return(2);
+      offset = 0;
+      }
+
     if (num_tokens > 1)
       {
-      data->pbc[0] = str_to_float(*(buff+0));
-      data->pbc[1] = str_to_float(*(buff+1));
+      data->pbc[0] = str_to_float(*(buff+offset));
+      data->pbc[1] = str_to_float(*(buff+offset+1));
       }
     else
-      printf("insufficient tokens.\n");
+      printf("Insufficient CELL tokens.\n");
 
     switch(data->periodic)
       {
       case 2:
         if (num_tokens > 2)
-          data->pbc[5] = D2R * str_to_float(*(buff+2));
+          data->pbc[5] = D2R * str_to_float(*(buff+offset+2));
         else
-          printf("insufficient tokens.\n");
+          printf("Insufficient CELL tokens.\n");
         break;
       case 3:
         if (num_tokens > 5)
           {
-          data->pbc[2] = str_to_float(*(buff+2));
-          data->pbc[3] = D2R * str_to_float(*(buff+3));
-          data->pbc[4] = D2R * str_to_float(*(buff+4));
-          data->pbc[5] = D2R * str_to_float(*(buff+5));
+          data->pbc[2] = str_to_float(*(buff+offset+2));
+          data->pbc[3] = D2R * str_to_float(*(buff+offset+3));
+          data->pbc[4] = D2R * str_to_float(*(buff+offset+4));
+          data->pbc[5] = D2R * str_to_float(*(buff+offset+5));
           }
         else
-          printf("insufficient tokens.\n");
+          printf("Insufficient CELL tokens.\n");
         break;
       }
     }
@@ -236,6 +246,7 @@ for (;;)
     {
     for (;;)
       {
+      g_strfreev(buff);
       buff = get_tokenized_line(fp, &num_tokens);
       if (!buff)
         break;
@@ -246,7 +257,7 @@ for (;;)
       if (num_tokens > 3)
         {
 /* add new atom if first item is a valid atom type */
-        if (elem_type(*buff))
+        if (elem_symbol_test(*buff))
           {
           core = new_core(*buff, data);
           data->cores = g_slist_prepend(data->cores, core);
@@ -265,7 +276,6 @@ for (;;)
 strcpy(data->filename, filename);
 g_free(data->basename);
 data->basename = strdup_basename(filename);
-
 prep_model(data);
 
 return(0);
