@@ -20,21 +20,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-#define VERSION 0.81
+#define VERSION 0.86
 #define PATCH 0
 
 #define MAX_ELEMENTS 110
 #define MAX_TOKENS 100
-#define MAX_ELEM_TOKENS 7
-#define MAX_PATH_ITEMS 50
 #define MAX_DISPLAYED 9
 #define MAX_MODELS 14
-/* NB: allowing sof's require making this unrealistically large */
-#define MAX_BONDS 16
-#define MAX_MOL_SIZE 100
-#define MAX_POTENTIALS 256
-#define MAX_MODEL_DIALOGS 10
-#define MAX_DIALOGS 15
 #define ELEM_LABEL_SIZE 7
 #define LABEL_SIZE 7
 #define TAIL_SIZE 80
@@ -43,7 +35,6 @@ The GNU GPL can also be found at http://www.gnu.org
 #define DEFAULT_SS 10
 #define BOND_FUDGE 0.1
 #define RMAX_FUDGE 1.1
-#define RMAX_SCALE 1.1
 #define ROTATE_SCALE 1.0
 #define SCALE_MAG 10
 #define MESH_POINTS_MIN 2
@@ -57,7 +48,6 @@ The GNU GPL can also be found at http://www.gnu.org
 /* parsing */
 #define FILELEN 150
 #define LINELEN  300
-#define ITEMLEN  50
 
 /* maths */
 #define PI 3.1415926535897932384626433832795
@@ -129,7 +119,7 @@ MOLE, MOLMEC, MOLQ, NOBUILD,
 NAME, CELL, CART, FRAC, S_FRAC,
 SURFACE_CELL, SURFACE_VECTORS, POLYMER_CELL, POLYMER_VECTOR, LATTICE_VECTORS,
 SPACE, ORIGIN, SPECIES, MAXCYC, ENERGY, TOTAL_ENERGY, SBULK_ENERGY,
-HARMONIC, MORSE, BUCKINGHAM, LENNARD,
+HARMONIC, MORSE, BUCKINGHAM, LENNARD, BRENNER,
 THREE, UREYBRADLEY, FOUR, TORSION, OUTOFPLANE, SPRING,
 SWITCH_OFF, SWITCH_ON, CYCLE, GNORM, BFGS_OPT, RFO_OPT, CONJ_OPT, UNIT_HESSIAN,
 TEMPERATURE, ENSEMBLE, SHRINK, QEQ, ZSISA, COMPARE, NOSYM, FIX, PHONON, EIGEN,
@@ -137,6 +127,7 @@ KPOINTS, GULP_LIBRARY,
 TITLE, END, GULP_OBSERVABLES, TEMP_FILE, DUMP_FILE, LIB_FILE, OUTPUT, 
 SUPER_CELL, NVE, NVT, NPT, 
 NO_ESURF, NO_EATT, D_HKL,
+KEYWORD,
 /* LABEL is new */
 ELEMENT, SYMBOL, NUMBER, LABEL, WEIGHT, COVALENT, IONIC, VDW, CHARGE, COLOUR,
 LABEL_NORMAL, LABEL_GHOST,
@@ -165,7 +156,7 @@ enum {CHECK, CONDENSE};
 enum {EXEC_PATH};
 enum {SELECTED, ANY};
 enum {BY_FILE_ID, BY_EXTENSION, BY_LABEL};
-enum {STANDARD, WARNING, ERROR};
+enum {STANDARD, WARNING, ERROR, BOLD, ITALIC};
 
 /* surface types */
 enum {SOLSURF, MOLSURF};
@@ -176,11 +167,12 @@ enum {BLACK, WHITE, RED, GREEN, BLUE, YELLOW, GOLD, ORANGE,
 
 /* data type specifiers */
 enum {MODEL, CORE, BOND, UBOND, HBOND, MOL, SHELL, ELEM,
-      ATOM_LABEL, ELEM_MOL, REGION,
+      ATOM_LABEL, ELEM_MOL, FRAGMENT, REGION,
       GEOM, DIST, ANGLE, DIHEDRAL, SELECT,
-      POLYHEDRAL, ZEOLITE, RIBBON,
+      ZEOLITE, RIBBON,
       SPATIAL_VECTOR, SPATIAL_VECTORS, SPATIAL_PLANE,
-      SPATIAL_TRIANGLE, SPATIAL_POLYGON, SPATIAL_MOLSURF};
+      SPATIAL_TRIANGLE_STRIP, SPATIAL_TRIANGLE, SPATIAL_POLYGON,
+      SPATIAL_MOLSURF, SPATIAL_HIRSHFELD, SPATIAL_POLYHEDRA};
 
 /* init_objs() call modes */
 enum 
