@@ -209,7 +209,7 @@ if (!sysenv.gamess_path)
 printf("output file: %s\n", data->gamess.out_file);
 #endif
 
-submit_task("Gamess", &exec_gamess_task, data, &proc_gamess_task, data, data);
+task_new("Gamess", &exec_gamess_task, data, &proc_gamess_task, data, data);
 }
 
 /*********************************/
@@ -328,7 +328,7 @@ g_return_val_if_fail(w != NULL, FALSE);
 /* ascertain type of modification required */
 id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "id"));
 
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 sysenv.moving = FALSE;
 
 switch(id)
@@ -402,15 +402,15 @@ gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 /* radio buttons for execution type */
 new_radio_group(0, vbox, TT);
-button = add_radio_button("run", (gpointer) gamess_exe_run, data);
+button = add_radio_button("Run", (gpointer) gamess_exe_run, data);
 if (data->gamess.exe_type == GMS_RUN)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("check", (gpointer) gamess_exe_check, data);
+button = add_radio_button("Check", (gpointer) gamess_exe_check, data);
 if (data->gamess.exe_type == GMS_CHECK)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("debug", (gpointer) gamess_exe_debug, data);
+button = add_radio_button("Debug", (gpointer) gamess_exe_debug, data);
 if (data->gamess.exe_type == GMS_DEBUG)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
@@ -423,19 +423,19 @@ gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 /* radio buttons for run type */
 new_radio_group(0, vbox, TT);
-button = add_radio_button("single point", (gpointer) gamess_run_single, data);
+button = add_radio_button("Single point", (gpointer) gamess_run_single, data);
 if (data->gamess.run_type == GMS_ENERGY)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("gradient", (gpointer) gamess_run_gradient, data);
+button = add_radio_button("Gradient", (gpointer) gamess_run_gradient, data);
 if (data->gamess.run_type == GMS_GRADIENT)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("hessian", (gpointer) gamess_run_hessian, data);
+button = add_radio_button("Hessian", (gpointer) gamess_run_hessian, data);
 if (data->gamess.run_type == GMS_HESSIAN)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-button = add_radio_button("optimize", (gpointer) gamess_run_optimize, data);
+button = add_radio_button("Optimize", (gpointer) gamess_run_optimize, data);
 if (data->gamess.run_type == GMS_OPTIMIZE)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
