@@ -43,6 +43,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "matrix.h"
 #include "opengl.h"
 #include "numeric.h"
+#include "module.h"
 #include "interface.h"
 
 /* main data structures */
@@ -60,10 +61,11 @@ free_slist((GSList *) sfc_list);
 /******************/
 /* system cleanup */
 /******************/
+/* TODO - other things to free? */
 void sys_free(void)
 {
-/* TODO - other things to free? */
 g_hash_table_destroy(sysenv.sfc_table);
+module_free();
 }
 
 /**********************************/
@@ -71,7 +73,7 @@ g_hash_table_destroy(sysenv.sfc_table);
 /**********************************/
 gint read_gdisrc(void)
 {
-gint i, element_flag;
+gint i, element_flag, num_tokens;
 gchar line[LINELEN], **buff;
 gdouble version;
 FILE *fp;
@@ -83,12 +85,12 @@ fp = fopen(sysenv.init_file, "r");
 if (fp)
   {
 /* scan the 1st line */
-  fgetline(fp, line);
-/* test for right format */
-  buff = get_tokens(line, 2);
+  buff = get_tokenized_line(fp, &num_tokens);
   if (g_ascii_strncasecmp(*buff,"gdis",4) == 0)
     {
 /* test for right version */
+    if (num_tokens < 2)
+      return(1);
     version = str_to_float(*(buff+1));
     sscanf(*(buff+1),"%lf",&version);
 /* 0.75 was when the new element scheme was implemented */
@@ -105,51 +107,65 @@ else
 /* read until EOF, or (failsafe) reached elements[] array allocation */
 element_flag=0;
 i=0;
-while(!fgetline(fp,line))
+for (;;)
   {
-  buff = get_tokens(line, 11);
+/* NB: we need line[] to get the OpenGL font */
+  if (fgetline(fp, line))
+    break;
+  buff = tokenize(line, &num_tokens);
+  if (!buff)
+    break;
 
 /* decide what to read */
   if (g_ascii_strncasecmp("size",*buff,4) == 0)
     {
-    sysenv.width = (gint) str_to_float(*(buff+1));
-    sysenv.height = (gint) str_to_float(*(buff+2));
-    if (sysenv.width < MIN_WIDTH)
-      sysenv.width = MIN_WIDTH;
-    if (sysenv.height < MIN_HEIGHT)
-      sysenv.height = MIN_HEIGHT;
+    if (num_tokens > 2)
+      {
+      sysenv.width = str_to_float(*(buff+1));
+      sysenv.height = str_to_float(*(buff+2));
+      if (sysenv.width < MIN_WIDTH)
+        sysenv.width = MIN_WIDTH;
+      if (sysenv.height < MIN_HEIGHT)
+        sysenv.height = MIN_HEIGHT;
+      }
     } 
 
 /* model pane width */
   if (g_ascii_strncasecmp("pane",*buff,4) == 0)
     {
-    if (strlen(*(buff+1)))
-      sysenv.tree_width = (gint) str_to_float(*(buff+1));
-    if (strlen(*(buff+2)))
-      sysenv.tray_height = (gint) str_to_float(*(buff+2));
+    if (num_tokens > 2)
+      {
+      sysenv.tree_width = str_to_float(*(buff+1));
+      sysenv.tray_height = str_to_float(*(buff+2));
+      }
+    }
+
+/* model tree/property divider */
+  if (g_ascii_strncasecmp("divide",*buff,6) == 0)
+    {
+    if (num_tokens > 1)
+      sysenv.tree_divider = (gint) str_to_float(*(buff+1));
     }
 
 /* povray executable */
   if (g_ascii_strncasecmp("povray_exe",*buff,10) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       {
       g_free(sysenv.povray_exe);
       sysenv.povray_exe = g_strdup(*(buff+1));
       }
     }
-
 /* auto unfrag? */
   if (g_ascii_strncasecmp("unfragment",*buff,10) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       sysenv.unfragment = str_to_float(*(buff+1));
     }
-
 /* animation creation executable */
   if (g_ascii_strncasecmp("convert_exe",*buff,11) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       {
       g_free(sysenv.convert_exe);
       sysenv.convert_exe = g_strdup(*(buff+1));
@@ -158,7 +174,7 @@ while(!fgetline(fp,line))
 /* image viewing executable */
   if (g_ascii_strncasecmp("viewer_exe",*buff,10) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       {
       g_free(sysenv.viewer_exe);
       sysenv.viewer_exe = g_strdup(*(buff+1));
@@ -167,7 +183,7 @@ while(!fgetline(fp,line))
 /* gulp executable */
   if (g_ascii_strncasecmp("gulp",*buff,4) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       {
       g_free(sysenv.gulp_exe);
       sysenv.gulp_exe = g_strdup(*(buff+1));
@@ -176,39 +192,52 @@ while(!fgetline(fp,line))
 /* file conversion executable */
   if (g_ascii_strncasecmp("babel_exe",*buff,9) == 0)
     {
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       {
       g_free(sysenv.babel_exe);
       sysenv.babel_exe = g_strdup(*(buff+1));
       }
     }
-/* gtk text font (NB: due to pango bug, name can't be only whitespace) */
-  if (g_ascii_strncasecmp("gtk_font",*buff,8) == 0)
-    if (strlen(*(buff+1)))
-      strcpy(sysenv.gfont_name, g_strstrip(&line[9]));
 /* OpenGL drawing font */
   if (g_ascii_strncasecmp("gl_font",*buff,7) == 0)
-    if (strlen(*(buff+1)))
-      strcpy(sysenv.dfont_name, g_strstrip(&line[8]));
+    if (num_tokens > 1)
+      strcpy(sysenv.gl_fontname, g_strstrip(&line[8]));
+
+/* model tree box */
+  if (g_ascii_strncasecmp("mtb",*buff,3) == 0)
+    if (num_tokens > 1)
+      sysenv.mtb_on = (gint) str_to_float(*(buff+1));
+
+/* model properties box */
+  if (g_ascii_strncasecmp("mpb",*buff,3) == 0)
+    if (num_tokens > 1)
+      sysenv.mpb_on = (gint) str_to_float(*(buff+1));
+
+/* model symmetry box */
+  if (g_ascii_strncasecmp("msb",*buff,3) == 0)
+    if (num_tokens > 1)
+      sysenv.msb_on = (gint) str_to_float(*(buff+1));
 
 /* atom properties box */
   if (g_ascii_strncasecmp("apb",*buff,3) == 0)
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       sysenv.apb_on = (gint) str_to_float(*(buff+1));
 
 /* rotation adjustment box */
+/*
   if (g_ascii_strncasecmp("rab",*buff,3) == 0)
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       sysenv.rab_on = (gint) str_to_float(*(buff+1));
+*/
 
 /* halo type */
   if (g_ascii_strncasecmp("halo",*buff,4) == 0)
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       sysenv.render.halos = (gint) str_to_float(*(buff+1));
 
 /* low quality rotation */
   if (g_ascii_strncasecmp("fast",*buff,4) == 0)
-    if (strlen(*(buff+1)))
+    if (num_tokens > 1)
       sysenv.render.fast_rotation = (gint) str_to_float(*(buff+1));
 
   g_strfreev(buff);
@@ -247,8 +276,11 @@ fprintf(fp,"gdis %f\n", VERSION);
 fprintf(fp,"canvas %d\n", sysenv.canvas);
 fprintf(fp,"size %d %d\n", sysenv.width,sysenv.height);
 fprintf(fp,"pane %d %d\n", sysenv.tree_width, sysenv.tray_height);
-fprintf(fp,"gtk_font %s\n", sysenv.gfont_name);
-fprintf(fp,"gl_font %s\n", sysenv.dfont_name);
+fprintf(fp,"divider %d\n", sysenv.tree_divider);
+fprintf(fp,"gl_font %s\n", sysenv.gl_fontname);
+fprintf(fp,"mtb %d\n", sysenv.mtb_on);
+fprintf(fp,"mpb %d\n", sysenv.mpb_on);
+fprintf(fp,"msb %d\n", sysenv.msb_on);
 fprintf(fp,"apb %d\n", sysenv.apb_on);
 fprintf(fp,"rab %d\n", sysenv.rab_on);
 fprintf(fp,"unfragment %d\n", sysenv.unfragment);
@@ -275,37 +307,43 @@ return(0);
 #define DEBUG_SYS_INIT 0
 void sys_init(gint argc, gchar *argv[])
 {
-gint i;
 gchar *temp;
 const gchar *ctemp;
 struct light_pak *light;
 FILE *fp;
 
 /* top level structure initialization */
-sysenv.num_tasks = 0;
-sysenv.running_tasks = 0;
-sysenv.max_running_tasks = 1;
-sysenv.tasks = NULL;
-sysenv.num_models = 0;
+sysenv.max_threads = 1;
+sysenv.task_list = NULL;
+sysenv.dialog_list = NULL;
 sysenv.stereo = FALSE;
+sysenv.stereo_windowed = FALSE;
+sysenv.stereo_fullscreen = FALSE;
 sysenv.canvas_list = NULL;
 sysenv.canvas_rows = 1;
 sysenv.canvas_cols = 1;
 sysenv.width = START_WIDTH;
 sysenv.height = START_HEIGHT;
 sysenv.tree_width = START_WIDTH/4;
+sysenv.tree_divider = -1;
 sysenv.tray_height = 60;
 sysenv.mpane = NULL;
 sysenv.moving = FALSE;
+sysenv.roving = FALSE;
 sysenv.redraw = TRUE;
+sysenv.mtb_on = TRUE;
+sysenv.mpb_on = TRUE;
+sysenv.msb_on = TRUE;
 sysenv.apb_on = TRUE;
-sysenv.rab_on = TRUE;
+sysenv.rab_on = FALSE;
+sysenv.lmb_on = FALSE;
+sysenv.pib_on = FALSE;
 /* default to single model display */
 sysenv.displayed[0] = -1;
-sysenv.active = 0;
+sysenv.active_model = NULL;
 sysenv.canvas = TRUE;
 sysenv.select_mode = CORE;
-sysenv.unfragment = FALSE;
+sysenv.unfragment = TRUE;
 sysenv.calc_pbonds = TRUE;
 VEC3SET(sysenv.angle, 0.0, 0.0, 0.0);
 
@@ -340,16 +378,19 @@ sysenv.render.wire_model = FALSE;
 sysenv.render.wire_surface = FALSE;
 sysenv.render.shadowless = FALSE;
 sysenv.render.animate = FALSE;
+sysenv.render.no_exec_povray = FALSE;
+sysenv.render.no_keep_tempfiles = FALSE;
 sysenv.render.animate_type = ANIM_GIF;
 sysenv.render.animate_file = g_strdup("movie");
 sysenv.render.atype = FALSE;
 sysenv.render.axes = FALSE;
 sysenv.render.morph_finish = g_strdup("F_Glass4");
 sysenv.render.ref_index = 2.5;
-sysenv.render.delay = 20;
-sysenv.render.sphere_quality = 2;
-sysenv.render.cylinder_quality = 9;
-sysenv.render.ribbon_quality = 10;
+sysenv.render.delay = 20.0;
+sysenv.render.mpeg_quality = 95.0;
+sysenv.render.sphere_quality = 2.0;
+sysenv.render.cylinder_quality = 9.0;
+sysenv.render.ribbon_quality = 10.0;
 sysenv.render.halos = FALSE;
 sysenv.render.fast_rotation = FALSE;
 
@@ -357,12 +398,13 @@ sysenv.render.ambience = 0.2;
 sysenv.render.diffuse = 0.9;
 sysenv.render.specular = 0.9;
 sysenv.render.transmit = 0.9;
+sysenv.render.ghost_opacity = 0.5;
 sysenv.render.ball_rad = 0.3;
 sysenv.render.stick_rad = 0.1;
 sysenv.render.stick_thickness = GTKGL_LINE_WIDTH;
 sysenv.render.line_thickness = GTKGL_LINE_WIDTH;
 sysenv.render.frame_thickness = GTKGL_LINE_WIDTH;
-sysenv.render.geom_line_width = GTKGL_LINE_WIDTH;
+sysenv.render.geom_line_width = 3.0;
 sysenv.render.cpk_scale = 1.0;
 sysenv.render.fog_density = 0.35;
 sysenv.render.fog_start = 0.5;
@@ -376,7 +418,7 @@ sysenv.render.shl_size = 20;
 VEC3SET(sysenv.render.fg_colour, 1.0, 1.0, 1.0);
 VEC3SET(sysenv.render.bg_colour, 0.0, 0.0, 0.0);
 VEC3SET(sysenv.render.morph_colour, 0.1, 0.1, 0.8);
-VEC3SET(sysenv.render.rsurf_colour, 0.1, 0.5, 0.7);
+VEC3SET(sysenv.render.rsurf_colour, 0.0, 0.3, 0.6);
 VEC3SET(sysenv.render.label_colour, 1.0, 1.0, 0.0);
 VEC3SET(sysenv.render.title_colour, 0.0, 1.0, 1.0);
 VEC3SET(sysenv.render.ribbon_colour, 0.0, 0.0, 1.0);
@@ -441,8 +483,7 @@ sysenv.gulp_exe = g_strdup("gulp");
 sysenv.babel_exe = g_strdup("babel");
 sysenv.grace_exe = g_strdup("xmgrace");
 sysenv.gamess_exe = g_strdup("run_gms_for_gdis");
-strcpy(sysenv.gfont_name, GTK_FONT);
-strcpy(sysenv.dfont_name, GL_FONT);
+strcpy(sysenv.gl_fontname, GL_FONT);
 /* atomic scattering factor coefficients */
 sysenv.sfc_table = g_hash_table_new_full(&g_str_hash,
                                          &hash_strcmp,
@@ -489,13 +530,12 @@ sysenv.babel_path = g_find_program_in_path(sysenv.babel_exe);
 sysenv.grace_path = g_find_program_in_path(sysenv.grace_exe);
 sysenv.gamess_path = g_find_program_in_path(sysenv.gamess_exe);
 
-/* dialog init */
-for (i=0 ; i<MAX_DIALOGS ; i++)
-  sysenv.dialog[i].active = FALSE;
 /* tree init */
 sysenv.num_trees = 0;
 /* copied selection */
 sysenv.select_source = NULL;
+sysenv.module_list = NULL;
+sysenv.projects = NULL;
 
 /* file extension recognition stuff */
 file_init();
@@ -514,6 +554,7 @@ gint i;
 /* NBL read this 1st as it affects canvas type, but command */
 /* line arguments should be able to overide */
 sys_init(argc, argv);
+module_setup();
 sysenv.write_gdisrc = FALSE;
 
 /* command line? */
@@ -524,8 +565,9 @@ if (argc > 1)
 /* set up main window and event handlers (or start non-gui mode) */
 if (sysenv.canvas)
   {
-/* NB: threads must be initialized before calling gtk_init() */
-  init_task_thread();
+/* initialize threads and set up the queue */
+  task_queue_init();
+
   gtk_init(&argc, &argv);
   gtk_gl_init(&argc, &argv);
 
@@ -540,16 +582,14 @@ if (sysenv.canvas)
   connect_events();
 
 /* process arguments as files to load */
-for (i=1 ; i<argc ; i++)
-  file_load(argv[i], NULL);
+  for (i=1 ; i<argc ; i++)
+    file_load(argv[i], NULL);
 
 /* thread-safe handling */
   gdk_threads_enter();
   gtk_main();
   gdk_threads_leave();
   }
-else
-  cmd_init(argc, argv);
 
 return(0);
 }
