@@ -29,6 +29,7 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "edit.h"
 #include "matrix.h"
+#include "model.h"
 #include "morph.h"
 #include "opengl.h"
 #include "select.h"
@@ -131,7 +132,6 @@ memcpy(mat2,tmp,16*sizeof(gdouble));
 return;
 }
 
-
 /****************************************/
 /* transform a 3D vector by matrix mult */
 /****************************************/
@@ -418,6 +418,19 @@ ARR3SUB(vec2, a);
 crossprod(res, vec1, vec2);
 }
 
+/*********************************************/
+/* calculate the distance between two points */
+/*********************************************/
+gdouble calc_sep(gdouble *a, gdouble *b)
+{
+gdouble x[3];
+
+ARR3SET(x, a);
+ARR3SUB(x, b);
+
+return(VEC3MAG(x));
+}
+
 /*****************************/
 /* project vector onto plane */
 /*****************************/
@@ -459,9 +472,9 @@ ARR3MUL(vec1, tmp);
 return(vec1[0]+vec1[1]+vec1[2]);
 }
 
-/******************************************/
-/* compute volume, given a lattice matrix */
-/******************************************/
+/**********************************************/
+/* compute volume, assuming 3D lattice matrix */
+/**********************************************/
 gdouble calc_volume(gdouble *latmat)
 {
 gdouble vec[3], tmat[9];
@@ -477,18 +490,40 @@ ARR3MUL(vec, &tmat[6]);
 return(vec[0]+vec[1]+vec[2]);
 }
 
+/****************************************************/
+/* compute surface area, assuming 2D lattice matrix */
+/****************************************************/
+gdouble calc_area(gdouble *latmat)
+{
+gdouble a, b, c, tmat[9];
+
+/* NB: latmat has cell vectors in columns */
+memcpy(tmat, latmat, 9*sizeof(gdouble));
+transpose(tmat);
+
+/* get lengths and angle between */
+a = VEC3MAG(&tmat[0]);
+b = VEC3MAG(&tmat[3]);
+c = via(&tmat[0], &tmat[3], 3);
+
+/* return volume */
+return(a * b * sin(c));
+}
+
 /*******************************************************************/
 /* construct matrix for (cartesian space) reflection about a plane */
 /*******************************************************************/
-/* TODO - add another pointer to the operation type reflection/rotation etc. */
+/* TODO - pointer to the operation type reflection/rotation etc. */
 #define DEBUG_MAKE_SYMMAT 0
-void make_symmat(gdouble *mat, gdouble *latmat, gpointer *spatial, gint type,
-                                                               gdouble angle) 
+void make_symmat(gdouble *mat,
+                 gdouble *latmat,
+                 gpointer *spatial,
+                 gint type,
+                 gdouble angle) 
 {
 gint n;
 gdouble proj, len;
 gdouble a[3], b[3], c[3], vec[4], tmp[4], mop[16], rot[9];
-gchar *text;
 struct vec_pak *p[4];
 struct spatial_pak *sdata;
 GSList *list;
@@ -615,7 +650,6 @@ mat[11] -= tmp[2];
 switch(type)
   {
   case REFLECTION:
-    show_text(STANDARD, "Constructing reflection matrix.\n");
     VEC4SET(&mop[0],  1.0, 0.0, 0.0, 0.0);
     VEC4SET(&mop[4],  0.0, 1.0, 0.0, 0.0);
     VEC4SET(&mop[8],  0.0, 0.0, -1.0, 0.0);
@@ -623,9 +657,6 @@ switch(type)
     break;
 
   case PAXIS:
-    text = g_strdup_printf("Constructing rotation, angle: %f\n", angle);
-    show_text(STANDARD, text);
-    g_free(text);
     make_rotmat(rot, D2R*angle, ROLL);
     VEC4SET(&mop[0],  rot[0], rot[1], rot[2], 0.0);
     VEC4SET(&mop[4],  rot[3], rot[4], rot[5], 0.0);
@@ -872,11 +903,24 @@ make_cell(data);
 memcpy(data->ilatmat, data->latmat, 9*sizeof(gdouble));
 invmat(data->ilatmat);
 make_rlat(data);
+switch(data->periodic)
+  {
+  case 3:
+    data->volume = calc_volume(data->latmat);
+    break;
+  case 2:
+    data->area = calc_area(data->latmat);
+    break;
+  }
+
+/* NEW - store angles in degrees as well as radians */
+ARR3SET(data->cell_angles, &data->pbc[3]);
+VEC3MUL(data->cell_angles, R2D);
 
 #if DEBUG_XLAT
 printf("cell dimensions: [%6.2f %6.2f %6.2f] (%6.2f %6.2f %6.2f)\n",
        data->pbc[0], data->pbc[1], data->pbc[2],
-       data->pbc[3], data->pbc[4], data->pbc[5]);
+       data->cell_angles[0], data->cell_angles[1], data->cell_angles[2]);
 P3MAT("lattice matrix:",data->latmat);
 #endif
 return;
@@ -1025,7 +1069,7 @@ if (invmat(mat2))
   }
 
 /* transformation -> new model */
-dest = model_ptr(sysenv.num_models, ASSIGN);
+dest = model_new();
 dest->periodic = model->periodic;
 memcpy(dest->latmat, mat1, 9*sizeof(gdouble));
 copy_gulp_data(model, dest);
@@ -1083,6 +1127,7 @@ for (ic=0 ; ic<mc ; ic++)
         {
         core = dup_core(list->data);
         core->primary = TRUE;
+        core->primary_core = NULL;
         core->orig = TRUE;
         ARR3ADD(core->x, vec);
         vecmat(model->latmat, core->x);
@@ -1091,6 +1136,7 @@ for (ic=0 ; ic<mc ; ic++)
           {
           shel = core->shell;
           shel->primary = TRUE;
+          shel->primary_shell = NULL;
           shel->orig = TRUE;
           ARR3ADD(shel->x, vec);
           vecmat(model->latmat, shel->x);
@@ -1106,7 +1152,7 @@ for (ic=0 ; ic<mc ; ic++)
 dest->fractional = FALSE;
 dest->construct_pbc = TRUE;
 prep_model(dest);
-new_tree_item(dest->number, APPEND);
+tree_model_add(dest);
 redraw_canvas(SINGLE);
 }
 
@@ -1197,4 +1243,3 @@ while(i<17)
 return(0);
 }
 
-
