@@ -30,16 +30,23 @@ The GNU GPL can also be found at http://www.gnu.org
 
 #include "gdis.h"
 #include "coords.h"
+#include "matrix.h"
 #include "edit.h"
 #include "file.h"
+#include "graph.h"
 #include "morph.h"
-#include "matrix.h"
+#include "model.h"
+#include "measure.h"
+#include "project.h"
+#include "analysis.h"
+#include "render.h"
+#include "select.h"
 #include "space.h"
 #include "surface.h"
-#include "analysis.h"
+#include "opengl.h"
 #include "interface.h"
 #include "dialog.h"
-#include "opengl.h"
+#include "zone.h"
 
 /* the main pak structures */
 extern struct sysenv_pak sysenv;
@@ -72,6 +79,7 @@ static gint n=0;
 /* NB: all the important defines should go here */
 /* ie those values that might cause a core dump due to */
 /* code that requires they at least have a value assigned  */
+data->id = -1;
 data->locked = FALSE;
 data->protein = FALSE;
 data->num_atoms = 0;
@@ -107,9 +115,16 @@ data->ubonds = NULL;
 data->moles = NULL;
 data->planes = NULL;
 data->ghosts = NULL;
-data->geom = NULL;
 data->images = NULL;
 data->layer_list = NULL;
+data->measure_list = NULL;
+data->graph_list = NULL;
+data->picture_list = NULL;
+data->transform_list = NULL;
+data->frame_data_list = NULL;
+data->graph_active = NULL;
+data->picture_active = NULL;
+data->project = NULL;
 data->zones = NULL;
 data->num_zones = 0;
 VEC3SET(data->zone_min, 0.0, 0.0, 0.0);
@@ -123,8 +138,10 @@ data->show_title = TRUE;
 data->show_frame_number = TRUE;
 data->show_hbonds = FALSE;
 data->show_energy = FALSE;
-data->show_charges = FALSE;
+data->show_charge = FALSE;
+data->show_atom_charges = FALSE;
 data->show_atom_labels = FALSE;
+data->show_atom_types = FALSE;
 data->show_geom_labels = TRUE;
 data->show_cores = TRUE;
 data->show_shells = FALSE;
@@ -133,10 +150,15 @@ data->show_cell = TRUE;
 data->show_cell_images = TRUE;
 data->show_cell_lengths = FALSE;
 
-/* NEW - special object list */
+/* NEW */
+data->property_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, property_free);
+data->property_list = NULL;
+
 data->ribbons = NULL;
 data->spatial = NULL;
 data->phonons = NULL;
+data->ir_list = NULL;
+data->raman_list = NULL;
 data->phonon_slider = NULL;
 data->current_phonon = -1;
 data->pulse_count = 0;
@@ -148,12 +170,7 @@ data->gulp.num_kpoints = 0;
 VEC3SET(data->gulp.kpoints, 0.0, 0.0, 0.0);
 
 /* VdW surface calc */
-data->csurf.type = MS_MOLECULAR;
-data->csurf.colour_method = MS_TOUCH;
-data->csurf.dsize = 0.1;
-data->csurf.depth = 4;
-data->csurf.prad = 1.0;
-data->csurf.offset = 2.0;
+data->ms_colour_scale = FALSE;
 
 /* init flags */
 data->periodic = 0;
@@ -162,7 +179,6 @@ data->fractional = FALSE;
 data->done_pbonds = FALSE;
 data->axes_type = CARTESIAN;
 data->box_on = FALSE;
-data->csurf_on = FALSE;
 data->asym_on = FALSE;
 /* bonding modes */
 data->build_molecules = TRUE;
@@ -188,23 +204,27 @@ data->pbc[2] = 1.0;
 data->pbc[3] = 0.5*PI;
 data->pbc[4] = 0.5*PI;
 data->pbc[5] = 0.5*PI;
+data->volume = 0.0;
+data->area = 0.0;
 /* init lattice matrix as this (for safety) */
 make_latmat(data);
 
 /* space group info */
+data->sginfo.lookup = TRUE;
 data->sginfo.spacenum = 0;
 data->sginfo.lattice = 0;
 data->sginfo.pointgroup = 0;
 data->sginfo.cellchoice = 0;
 data->sginfo.order = 0;
 data->sginfo.centric = 0;
-data->sginfo.spacename = g_strdup("P1");
-data->sginfo.latticename = g_strdup("None");
-/* NEW - store the raw SgInfo data */
 /*
-data->sginfo.raw = g_malloc(1);
+data->sginfo.spacename = g_strdup("P 1");
+data->sginfo.latticename = g_strdup("None");
 */
-data->sginfo.raw = NULL;
+data->sginfo.spacename = NULL;
+data->sginfo.latticename = NULL;
+data->sginfo.matrix = NULL;
+data->sginfo.offset = NULL;
 
 /* symmetry info */
 data->symmetry.num_symops = 0;
@@ -241,23 +261,15 @@ data->num_vertices = 0;
 data->num_planes = 0;
 data->vertices = NULL;
 
-/* NEW - default to a (semi) unique name */
-data->basename = g_strdup_printf("model_%d", data->number);
+data->basename = g_strdup_printf("model_%d", g_slist_index(sysenv.mal, data));
 strcpy(data->filename, data->basename);
 n++;
 
-/* deprec. */
-/* model tree widget */
-data->tree_label=NULL;
-data->tree_name = NULL;
 /* default potential library */
-strcpy(data->gulplib,LIBRARY);
+strcpy(data->gulplib, LIBRARY);
 
 /* gulp template init */
 data->gulp.maxcyc=0;
-data->gulp.num_potentials=0;
-data->gulp.num_elem=0;
-data->gulp.num_species=0;
 data->gulp.energy = 0.0 ;
 data->gulp.no_esurf=FALSE;
 data->gulp.esurf[0]=0.0;
@@ -282,17 +294,23 @@ data->gulp.compare = FALSE;
 data->gulp.nosym = FALSE;
 data->gulp.fix = FALSE;
 data->gulp.method = CONP;
-
-data->gulp.potgrid = FALSE;
-VEC3SET(data->gulp.grid_pts, 10, 10, 10) ;
-VEC3SET(&data->gulp.grid_dim[0], 0.0, 0.0, 0.0);
-VEC3SET(&data->gulp.grid_dim[3], 1.0, 1.0, 1.0);
-/* deprec. */
-data->gulp.epot_grid = NULL;
+/* dynamics info */
+data->gulp.frame_time = 0.0;
+data->gulp.frame_ke = 0.0;
+data->gulp.frame_pe = 0.0;
+data->gulp.frame_temp = 0.0;
+/* NEW - original frame data (stored since reading in trajectory overwrites) */
+data->gulp.orig_fractional = FALSE;
+data->gulp.orig_construct_pbc = FALSE;
+data->gulp.orig_cores = NULL;
+data->gulp.orig_shells = NULL;
+data->gulp.ff_list = NULL;
+data->gulp.species_list = NULL;
+data->gulp.element_list = NULL;
 /* potgrid replacement */
+data->gulp.epot_autoscale = TRUE;
 data->gulp.epot_vecs = NULL;
 data->gulp.epot_vals = NULL;
-data->gulp.epot_autoscale = TRUE;
 data->gulp.epot_divisions = 0;
 data->gulp.epot_min = 999999999.9;
 data->gulp.epot_max = -999999999.9;
@@ -310,9 +328,9 @@ data->gulp.libfile = g_strdup(LIBRARY);
 data->gulp.srcfile = NULL;
 data->gulp.extra = NULL;
 data->gulp.temp_file = g_strdup("none");
-data->gulp.out_file = g_strdup("none");
 data->gulp.dump_file = g_strdup("none");
-data->gulp.trj_file = g_strdup("none");
+data->gulp.out_file = g_strdup("none");
+data->gulp.trj_file = NULL;
 data->gulp.mov_file = NULL;
 
 /* GAMESS defaults */
@@ -342,8 +360,15 @@ data->gamess.temp_file = g_strdup("none");
 data->gamess.out_file = g_strdup("none");
 data->gamess.energy = data->gamess.max_grad = data->gamess.rms_grad = 0.0;
 data->gamess.have_energy = data->gamess.have_max_grad = data->gamess.have_rms_grad = FALSE;
+
 data->abinit.energy = data->abinit.max_grad = data->abinit.rms_grad = 0.0;
 
+data->nwchem.energy = data->nwchem.max_grad = data->nwchem.rms_grad = 0.0;
+data->nwchem.have_energy = data->nwchem.have_max_grad = data->nwchem.have_rms_grad = data->nwchem.min_ok = FALSE;
+
+data->castep.energy = data->castep.max_grad = data->castep.rms_grad = 0.0;
+data->castep.have_energy = data->castep.have_max_grad = data->castep.have_rms_grad = data->castep.min_ok = FALSE;
+
 /* SIESTA defaults */
 data->siesta.energy = data->siesta.max_grad = 0.0;
 data->siesta.have_energy = data->siesta.have_max_grad = 0.0;
@@ -365,6 +390,7 @@ data->surface.include_polar = FALSE;
 data->surface.ignore_bonding = FALSE;
 data->surface.create_surface = FALSE;
 data->surface.true_cell = FALSE;
+data->surface.keep_atom_order = FALSE;
 data->surface.miller[0] = 1;
 data->surface.miller[1] = 0;
 data->surface.miller[2] = 0;
@@ -388,12 +414,15 @@ data->unique_atom_list = NULL;
 data->animation = FALSE;
 data->animating = FALSE;
 data->anim_fix = FALSE;
+data->anim_loop = FALSE;
 data->afp = NULL;
 data->anim_confine = PBC_CONFINE_ATOMS;
 data->anim_speed = 5.0;
-data->analysis = NULL;
 data->frame_list = NULL;
 data->title = NULL;
+
+/* NEW */
+data->analysis = analysis_new();
 }
 
 /************************************/
@@ -404,7 +433,6 @@ gint prep_model(struct model_pak *data)
 {
 g_return_val_if_fail(data != NULL, 1);
 
-
 #if DEBUG_PREP_MODEL
 printf("start prep: %d\n", data->number);
 #endif
@@ -419,35 +447,50 @@ make_latmat(data);
 if (!data->fractional)
   latmat_fudge(data);
 
+/* initialize the spatial partitioning */
+zone_init(data);
+
 /* get initial linkages */
-update_shells(data);
+/* NB: full calc is done after remove_duplicates, but this is still needed */
+/* so that a full cell of shells can be built from the asymmetric unit */
+shell_make_links(data);
 
 /* mark the largest region label */
 data->region_max = region_max(data);
 
+if (data->periodic == 3)
+  {
+  if (!data->sginfo.spacenum)
+    data->sginfo.spacenum=1;
+  if (space_lookup(data))
+    show_text(ERROR, "Error in Space Group lookup.\n");
+  }
+else
+  remove_duplicates(data);
+
+
+#if OLD_CRAP
 /* periodic setup */
 switch(data->periodic)
   {
   case 3:
+/*
   case 2:
+*/
     if (!data->sginfo.spacenum)
-      {
-#if DEBUG_PREP_MODEL
-printf("Initial space group lookup failed, retrying as P1.\n");
-#endif
       data->sginfo.spacenum=1;
-      }
-    if (genpos(data))
-      printf("Fatal error in Space Group lookup.\n");
+    if (space_lookup(data))
+      show_text(ERROR, "Error in Space Group lookup.\n");
     break;
 
   default:
 /* NEW - remove duplicate cores which can cause connectivity  */
 /* wierdness and stop the ribbon code (for eg) from working */
 /* also removes symmetry generated equivalents */
-/* NB: fn_fill_cell() in genpos() also calls this */
+/* NB: fn_fill_cell() in space_lookup() also calls this */
     remove_duplicates(data);
   }
+#endif
 
 /* multi-frame file is always an animation */
 if (data->num_frames > 1)
@@ -470,6 +513,12 @@ if (data->animation)
     data->expected_cores = g_slist_length(data->cores);
     data->expected_shells = g_slist_length(data->shels);
     n = data->expected_cores + data->expected_shells;
+/* cope with the supercell cell multiplication */
+    n *= data->gulp.super[0] * data->gulp.super[1] * data->gulp.super[2];
+
+/* NB: a bit messy, but the supercell isn't created until later */
+/* (see file_load() - after the entire file has been processed) */
+
 /* sizeof RECORD */
     frame_size = 14 * sizeof(int);
     frame_size += (4 + 6*n) * sizeof(double);
@@ -477,8 +526,27 @@ if (data->animation)
 if (data->gulp.ensemble == NPT)
   {
   frame_size += 4 * sizeof(int);
-/* TODO - ask julian how to figure out what nstrains is */
-  frame_size += 18 * sizeof(double);
+
+/* nstrains (cell velocities) */
+  switch (data->periodic)
+    {
+    case 3:
+      n=6;
+      break;
+    case 2:
+      n=3;
+      break;
+    case 1:
+      n=1;
+      break;
+    default:
+      n=0;
+    }
+
+/* cell vectors */
+  n += 9;
+
+  frame_size += n * sizeof(double);
   }
 
 /* stat the file (MUST have the full path) */
@@ -518,21 +586,6 @@ if ((num_frames - data->num_frames) > FRACTION_TOLERANCE)
   {
   printf("Bad GULP trajectory file.\n");
   }
-else
-  {
-/*
-  add_frame_offset(fp, data);
-  for (n=0 ; n<num_frames ; n++)
-    {
-    if (read_trj_frame(fp, data, FALSE))
-      {
-      printf("Error parsing frames...\n");
-      break;
-      }
-    add_frame_offset(fp, data);
-    }
-*/
-  }
 fclose(fp);
 
       }
@@ -550,14 +603,6 @@ printf(" num atoms: %d\n", data->num_atoms);
 printf("num shells: %d\n", data->num_shells);
 #endif
 
-/* faster animation - assume constant connectivity */
-if (data->animating && data->anim_fix)
-  {
-/* this seems to fix some problems with animation & periodic bonds */
-  unfragment(data);
-  return(0);
-  }
-
 /* display coords & connectivity */
 init_objs(INIT_COORDS, data);
 calc_bonds(data);
@@ -573,9 +618,12 @@ model_colour_scheme(data->colour_scheme, data);
 /* final surface setup */
 if (data->periodic == 2)
   {
-/* order by z */
-  sort_coords(data);
-  init_objs(CENT_COORDS, data);
+/* order by z, unless input core order should be kept */
+  if (!data->surface.keep_atom_order)
+    {
+    sort_coords(data);
+    init_objs(CENT_COORDS, data);
+    }
   }
 
 #if DEBUG_PREP_MODEL
@@ -597,42 +645,20 @@ calc_bonds(data);
 calc_mols(data);
 }
 
-#define DEBUG_WIPE_PLANE_LIST 0
-void free_plane_list(struct model_pak *data)
+/*********************************************************/
+/* convenience routine for duplicating a list of strings */
+/*********************************************************/
+GSList *slist_gchar_dup(GSList *src)
 {
-GSList *list;
-struct plane_pak *pdata;
+GSList *list, *dest=NULL;
 
-#if DEBUG_WIPE_PLANE_LIST
-printf("Freeing plane list");
-#endif
+for (list=src ; list ; list=g_slist_next(list))
+  dest = g_slist_prepend(dest, g_strdup(list->data));
 
-/* free lists contained within the plane pak structure */
-list = data->planes;
-while (list)
-  {
-#if DEBUG_WIPE_PLANE_LIST
-printf(".");
-#endif
-  pdata = (struct plane_pak *) list->data;
-  list = g_slist_next(list);
+if (dest)
+  dest = g_slist_reverse(dest);
 
-  if (pdata->shifts)
-    g_slist_free(pdata->shifts);
-
-  g_slist_free(pdata->vertices);
-
-  g_free(pdata);
-  }
-
-#if DEBUG_WIPE_PLANE_LIST
-printf("!\n");
-#endif
-
-if (data->planes)
-  g_slist_free(data->planes);
-data->planes = NULL;
-data->num_planes = 0;
+return(dest);
 }
 
 /*************************************************************/
@@ -663,6 +689,16 @@ g_list_free(list);
 list=NULL;
 }
 
+/*************************************************/
+/* TODO - put stuff like this in a gamess.c file */
+/*************************************************/
+void gamess_data_free(struct model_pak *model)
+{
+g_free(model->gamess.title);
+g_free(model->gamess.temp_file);
+g_free(model->gamess.out_file);
+}
+
 /***********************************/
 /* free the entire model structure */
 /***********************************/
@@ -677,17 +713,16 @@ printf("freeing string data...\n");
 #endif
 
 g_free(data->basename);
-g_free(data->sginfo.spacename);
-g_free(data->sginfo.latticename);
-g_free(data->sginfo.raw);
 g_free(data->symmetry.pg_name);
 g_free(data->title);
 
+space_free(&data->sginfo);
+
 #if DEBUG_FREE_MODEL
 printf("freeing zone data...\n");
 #endif
 
-con_free_zones(data);
+zone_free(data);
 
 #if DEBUG_FREE_MODEL
 printf("freeing coord data...\n");
@@ -698,6 +733,25 @@ free_core_list(data);
 free_slist(data->images);
 free_list(data->frame_list);
 free_slist(data->layer_list);
+free_slist(data->picture_list);
+graph_free_list(data);
+free_slist(data->phonons);
+free_slist(data->ir_list);
+free_slist(data->raman_list);
+
+/* destroy the property table and list */
+g_hash_table_destroy(data->property_table);
+g_slist_free(data->property_list);
+
+gamess_data_free(data);
+
+/* NEW - GULP files with associated trajectory only */
+if (data->gulp.orig_cores)
+  free_slist(data->gulp.orig_cores);
+if (data->gulp.orig_shells)
+  free_slist(data->gulp.orig_shells);
+
+free_gulp_data(data);
 
 #if DEBUG_FREE_MODEL
 printf("freeing symmetry data...\n");
@@ -708,36 +762,27 @@ g_strfreev(data->symmetry.items);
 #if DEBUG_FREE_MODEL
 printf("freeing measurements...\n");
 #endif
-meas_prune_all(data);
-meas_free_all(data);
+meas_prune_model(data);
+measure_free_all(data);
 
 #if DEBUG_FREE_MODEL
 printf("freeing vertices... (%d)\n", data->num_vertices);
 #endif
-free_plane_list(data);
+
+plane_data_free(data->planes);
+g_slist_free(data->planes);
+data->planes = NULL;
+data->num_planes = 0;
+
 /* TODO - free adj list (not data) for each vertices */
 free_slist(data->vertices);
 
 #if DEBUG_FREE_MODEL
 printf("freeing element data...\n");
 #endif
-free_gulp_data(data);
 if (data->num_elem)
   g_free(data->elements);
 
-#if DEBUG_FREE_MODEL
-printf("freeing electrostatic data...\n");
-#endif
-
-if (data->gulp.epot_vecs)
-  free_slist(data->gulp.epot_vecs);
-if (data->gulp.epot_vals)
-  free_slist(data->gulp.epot_vals);
-
-/* deprec. */
-if (data->gulp.epot_grid)
-  g_free(data->gulp.epot_grid);
-
 /*
 #if DEBUG_FREE_MODEL
 printf("closing animation stream...\n");
@@ -746,72 +791,62 @@ if (data->afp)
   fclose(data->afp);
 */
 
-free_analysis(data);
+analysis_free(data);
 }
 
 /************************/
 /* delete a given model */
 /************************/
 #define DEBUG_DELETE_MODEL 0
-void delete_model(gint model)
+void model_delete(struct model_pak *model)
 {
-gint i;
-struct model_pak *data;
+GSList *list;
+struct model_pak *m;
 
 /* checks */
-if (!sysenv.num_models)
+if (!model)
   return;
-data = model_ptr(model, RECALL);
-g_return_if_fail(data != NULL);
-if (data->locked)
+if (model->locked)
   {
   show_text(ERROR, "Model is locked.\n");
   return;
   }
 
+if (!g_slist_find(sysenv.mal, model))
+  {
+  printf("WARNING: unregistered model.\n");
+  }
+else
+  {
 #if DEBUG_DELETE_MODEL
 printf("shuffling model numbers...\n");
 #endif
 /* shuffle the data->number to account */
-/* for the deletion of sysenv.active */
-for (i=0 ; i<sysenv.num_models ; i++)
-  {
-  if (i <= model)
-    continue;
-  data = model_ptr(i, RECALL);
-  g_return_if_fail(data != NULL);
-  data->number--; 
+/* for the deletion of model */
+  for (list=sysenv.mal ; list ; list=g_slist_next(list))
+    {
+    m = list->data;
+    if (m == model)
+      break;
+    m->number--; 
+    }
+  sysenv.mal = g_slist_remove(sysenv.mal, model);
   }
 
-/* free the model's pointers */
-data = model_ptr(model, RECALL);
-g_return_if_fail(data != NULL);
-free_model(data);
-
 #if DEBUG_DELETE_MODEL
 printf("freeing assoc. dialogs...\n");
 #endif
 
-/* NB: do last - since some stuff in free_model() does dialog cleanup */
-close_model_dialogs(model);
+dialog_destroy_model(model);
 
-#if DEBUG_DELETE_MODEL
-printf("releasing slot...\n");
-#endif
-model_ptr(model, RELEASE);
+/* free the model's pointers */
+free_model(model);
+g_free(model);
 
-/* ensure active model number stays within upper bound */
-if (sysenv.active >= sysenv.num_models)
-  sysenv.active = sysenv.num_models-1;
-
-/* adjust the displayed models */
+/* update */
 canvas_shuffle();
-
-/* highlight the new active model */
-tree_select_active();
-
-/* redraw */
 redraw_canvas(ALL);
+
 #if DEBUG_DELETE_MODEL
 printf("delete_model() completed.\n");
 #endif
@@ -822,7 +857,21 @@ printf("delete_model() completed.\n");
 /**************************/
 void delete_active()
 {
-delete_model(sysenv.active);
+model_delete(sysenv.active_model);
+}
+
+/*******************************/
+/* replacement for ASSIGN call */
+/*******************************/
+struct model_pak *model_new(void)
+{
+struct model_pak *model;
+
+model = g_malloc(sizeof(struct model_pak));
+sysenv.mal = g_slist_append(sysenv.mal, model);
+
+template_model(model);
+return(model);
 }
 
 /*******************************************/
@@ -836,68 +885,126 @@ struct model_pak *ptr=NULL;
 /* how were we called? */
 switch (mode)
   {
-  case ASSIGN:
-/* allocate a new pointer */
-    ptr = g_malloc(sizeof(struct model_pak));
-/* FIXME - append is inefficient */
-    sysenv.mal = g_slist_append(sysenv.mal, ptr);
-#if DEBUG_PTR
-printf("Allocated: address=%p\n",ptr);
-#endif
-/* the model is new, so do some setup */
-    sysenv.active = model;
-    sysenv.num_models++;
-/* record the model number (needed before calling template_model() */
-    ptr->number = model;
-/* standard settings/allocations */
-    template_model(ptr);
-    break;
-
   case RECALL:
 /* FIXME - there are an awful lot of calls to this; check for routines */
 /* that pass the model number, when they could pass the model ptr instead */
 #if DEBUG_PTR
 printf("Request for model %d [%d,%d], ptr %p\n", model, 0, sysenv.num_models, ptr);
 #endif
-/* check for out of bounds request */
-    if (model >= 0 && model < sysenv.num_models)
-      ptr = (struct model_pak *) g_slist_nth_data(sysenv.mal, model);
-    break;
-
-/* delete a pointer */
-  case RELEASE:
-#if DEBUG_PTR
-printf("Requested release of model=%d\n",model);
-#endif
-    if (model >= 0 && model < sysenv.num_models)
-      {
-/* get the model pointer and free it */
-      ptr = (struct model_pak *) g_slist_nth_data(sysenv.mal, model);
-      g_free(ptr);
-/* remove it from the list */
-      sysenv.mal = g_slist_remove(sysenv.mal, ptr);
-      sysenv.num_models--;
-      }
+    ptr = (struct model_pak *) g_slist_nth_data(sysenv.mal, model);
     break;
   }
-
 return(ptr);
 }
 
-/**************************************************/
-/* search for the first model with the given name */
-/**************************************************/
-gint search_basename(const gchar *name)
-{
-gint i;
-struct model_pak *data;
+/***********************************************/
+/* model property list modification primitives */
+/***********************************************/
 
-for (i=0 ; i<sysenv.num_models ; i++)
+struct property_pak
+{
+guint rank;
+gchar *label;
+gchar *value;
+};
+
+/*************************/
+/* destruction primitive */
+/*************************/
+void property_free(gpointer ptr_property)
+{
+struct property_pak *p = ptr_property;
+
+/* NB: the label/key is free'd elsewhere */
+g_free(p->value);
+g_free(p);
+}
+
+/*********************/
+/* ranking primitive */
+/*********************/
+gint property_ranking(gpointer ptr_p1, gpointer ptr_p2)
+{
+struct property_pak *p1 = ptr_p1, *p2 = ptr_p2;
+
+if (p1->rank < p2->rank)
+  return(-1);
+if (p1->rank > p2->rank)
+  return(1);
+return(0);
+}
+
+/***************************************/
+/* add a pre-ranked new model property */
+/***************************************/
+/* NB: property is not displayed if rank = 0 */
+void property_add_ranked(guint rank,
+                         const gchar *key,
+                         const gchar *value,
+                         struct model_pak *model)
+{
+struct property_pak *p;
+
+g_assert(model != NULL);
+
+/* check if label already exists */
+p = g_hash_table_lookup(model->property_table, key);
+if (p)
   {
-  data = model_ptr(i, RECALL);
-  if (g_ascii_strcasecmp(data->basename, name) == 0)
-    return(i);
+/* update existing property */
+  g_free(p->value); 
+  p->value = g_strdup(value);
+  p->rank = rank;
+  model->property_list = g_slist_remove(model->property_list, p);
   }
-return(-1);
+else
+  {
+/* create new property */
+  p = g_malloc(sizeof(struct property_pak));
+
+  p->value = g_strdup(value);
+  p->label = g_strdup(key);
+  p->rank = rank;
+  g_hash_table_replace(model->property_table, p->label, p);
+  }
+
+/* update sorted property list */
+model->property_list = g_slist_insert_sorted(model->property_list, p, (gpointer) property_ranking);
+}
+
+/****************************************/
+/* property value extraction primitives */
+/****************************************/
+gchar *property_lookup(gchar *key, struct model_pak *model)
+{
+struct property_pak *p;
+
+g_assert(model != NULL);
+
+p = g_hash_table_lookup(model->property_table, key);
+if (p)
+  return(p->value);
+return(NULL);
+}
+
+guint property_rank(gpointer ptr_property)
+{
+struct property_pak *p = ptr_property;
+
+return(p->rank);
+}
+
+gchar *property_label(gpointer ptr_property)
+{
+struct property_pak *p = ptr_property;
+
+return(p->label);
+}
+
+gchar *property_value(gpointer ptr_property)
+{
+struct property_pak *p = ptr_property;
+
+return(p->value);
 }
 
