@@ -20,18 +20,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-/**************/
-/* dialog pak */
-/**************/
-struct dialog_pak
-{
-gint active; 
-gint model;       /* assoc. model */
-gint type;        /* identifier */
-GtkWidget *win;
-gpointer data;    /* assoc. data */
-};
-
 /**********************************/
 /* global unified rendering setup */
 /**********************************/
@@ -41,9 +29,8 @@ struct render_pak
 gint type;
 gdouble width;
 gdouble height;
-gint background;	/* TODO - make a colour, not a toggle */
 gint camera[3];
-gdouble vp_dist;         /* effectively the vanishing point for perspective proj */
+gdouble vp_dist;        /* vanishing point for perspective proj */
 gchar filename[FILELEN];
 /* flags & control */
 gint perspective;
@@ -57,8 +44,11 @@ gboolean wire_surface;
 gboolean wire_show_hidden;
 gint animate;
 gint animate_type;       /* file output */
+gint no_exec_povray;
+gint no_keep_tempfiles;
 gchar *animate_file;
 gdouble delay;
+gdouble mpeg_quality;
 /* composite object quality */
 gdouble sphere_quality;
 gdouble cylinder_quality;
@@ -100,6 +90,7 @@ gdouble ahl_size;
 /* material properties*/
 gdouble ref_index;
 gdouble transmit;
+gdouble ghost_opacity;
 gdouble fg_colour[3];
 gdouble bg_colour[3];
 gdouble morph_colour[3];
@@ -141,8 +132,12 @@ gint file_type;
 gint babel_type;
 gint write_gdisrc;
 
-/* NEW - centralized info on all supported file types */
+/* centralized info on all supported file types */
 GSList *file_list;
+/* NEW - dynamic module support */
+GSList *module_list;
+/* NEW - simulation list */
+GSList *projects;
 
 /* database allocation size */
 gint num_elements;
@@ -151,29 +146,29 @@ GHashTable *sfc_table;
 GtkTreeStore *tree_store;
 GtkWidget *tree;
 
-/* GTK text font */
-gchar gfont_name[LINELEN];
-GdkFont *gfont;
-gint gfontsize;
-/* GDK drawing font */
-gchar dfont_name[LINELEN];
-GdkFont *dfont;
-gint dfontsize;
-/* rotation adjustment box */
-/* atom properties box */
-gint rab_on;
+/* approximate GTK font width */
+gint gtk_fontsize;
+
+/* OpenGL font */
+gchar gl_fontname[LINELEN];
+GdkFont *gl_font;
+
+/* LEFT pane hideable boxes */
+gint mtb_on;
+gint mpb_on;
+gint msb_on;
 gint apb_on;
-GtkWidget *rab_frame;
-GtkWidget *apb_frame;
+gint pib_on;
+gint rab_on;
+gint lmb_on;
 
 /* model related */
-gint num_models;
+gpointer active_model;
 GSList *mal;
 gint num_trees;
 gint num_displayed;
 gint displayed[MAX_DISPLAYED];
 gint select_mode;
-gint active;
 gint unfragment;
 /* NEW - perform the periodic bond calc, slower when it's done, but */
 /* gives more info & speeds up subsequent stuff (eg unfrag) */
@@ -181,10 +176,13 @@ gint calc_pbonds;
 gdouble angle[3];
 
 /* drawing */
-gboolean stereo;
+gboolean stereo;                /* stereo on/off */
+gboolean stereo_windowed;       /* is capable of windowed stereo */
+gboolean stereo_fullscreen;     /* actual stereo mode (windowed/fullscreen) */
 gint canvas;
 gint redraw;
 gint moving;
+gint roving;
 gint canvas_rows;
 gint canvas_cols;
 GSList *canvas_list;
@@ -199,6 +197,7 @@ gint height;
 /* model pane dimensions */
 gint tree_width;
 gint tray_height;
+gint tree_divider;
 /* for OpenGL - the smaller of width & height */
 gint size;    /* pixel size */
 gdouble rsize; /* real world size */
@@ -219,24 +218,17 @@ gdouble fsubheight;
 gint subcenx[MAX_DISPLAYED];
 gint subceny[MAX_DISPLAYED];
 /* dialog list */
-struct dialog_pak dialog[MAX_DIALOGS];
+GSList *dialog_list;
 /* copy active select */
 struct model_pak *select_source;
 /* rendering */
 struct render_pak render;
 
 /* task handling */
-gint num_tasks;
-gint running_tasks;
-gint max_running_tasks;
-GSList *tasks;
-GtkWidget *task_label;
-/* gthread task implementation */
-GCond *task_cond;
-GMutex *task_mutex;
 GSList *task_list;
-GThread *task_thread;
-
+GtkWidget *task_label;
+gint max_threads;
+GThreadPool *thread_pool;
 };
 
 /********************************/
@@ -274,9 +266,11 @@ gdouble rn[3];
 /***************************/
 struct geom_pak
 {
-gint type;       /* atom, bond, angle etc. */
+gint type;           /* atom, bond, angle etc. */
+gint image[3];       /* iff dist -> image translation of the 2nd core */
 GSList *cores;
-gchar *text;    /* text */
+gdouble colour[3];
+gchar *text;        /* text */
 };
 
 /***********************/
@@ -289,19 +283,6 @@ gint type;          /* plane, vector, ribbon */
 gpointer *data;     /* object definition data */
 };
 
-/********************/
-/* VdW surface pack */
-/********************/
-struct molsurf_pak
-{
-gint type;            /* solvent acc. or molecular surface */
-gint colour_method;
-gdouble depth;        /* triangle subdivision */
-gdouble prad;         /* probe radius */
-gdouble dsize;        /* pairwise contribution weighting */
-gdouble offset;       /* distance above to plot the surface */
-};
-
 /**********************/
 /* general point pack */
 /**********************/
@@ -317,6 +298,7 @@ gdouble *x[3];
 /*****************************************************************/
 struct shift_pak
 {
+gint locked;
 gdouble shift;
 gint region[2];
 gdouble eatt[2];
@@ -368,6 +350,7 @@ gdouble rx[3];
 /********************/
 struct space_pak
 {
+gint lookup;
 gint spacenum;
 gint lattice;
 gint pointgroup;
@@ -380,9 +363,6 @@ gint order;
 gdouble **matrix;
 /* const offset to general positions */
 gdouble **offset;
-
-/* raw SGINFO data */
-gpointer *raw;
 };
 
 /*****************/
@@ -452,36 +432,37 @@ gdouble sdipole_tolerance;
 gdouble qsum;
 gdouble gnorm;
 gdouble temp;
-
-/* potgrid calc */
-/* deprec. - use potsites instead */
-gint potgrid;
-gint grid_pts[3];
-gdouble grid_dim[6];
-gdouble *epot_grid;
-
-/* NEW */
+/* dynamics info */
+gdouble frame_time;
+gdouble frame_ke;
+gdouble frame_pe;
+gdouble frame_temp;
+/* HACK - store original structure so we can recall after reading trajectory */
+gint orig_fractional;
+gint orig_construct_pbc;
+gdouble orig_pbc[6];
+gdouble orig_latmat[9];
+GSList *orig_cores;
+GSList *orig_shells;
+/* electrostatic potential calcs */
+gint epot_autoscale;
 GSList *epot_vecs;
 GSList *epot_vals;
-
-gint epot_autoscale;
 gint epot_divisions;
 gdouble epot_min;
 gdouble epot_max;
-
 /* widgets that need to be updated */
 GtkWidget *energy_entry;
 GtkWidget *sdipole_entry;
 GtkWidget *esurf_entry;
 GtkWidget *eatt_entry;
 GtkWidget *sbe_entry;
-/* element/FF data */
-gint num_potentials;
-gint num_elem;
-gint num_species;
-gchar **pot;
-gchar **elem;
-gchar **species;
+
+/* element/potentials data */
+GSList *ff_list;
+GSList *species_list;
+GSList *element_list;
+
 /* filename control */
 gchar *libfile;
 gchar *srcfile;
@@ -594,12 +575,40 @@ gdouble max_grad;
 gdouble rms_grad;
 };
 
+/***************/
+/* NWChem data */
+/***************/
+struct nwchem_pak
+{
+gdouble energy;
+gdouble max_grad;
+gdouble rms_grad;
+gint min_ok;
+gint have_energy;
+gint have_max_grad;
+gint have_rms_grad;
+};
+
+/***************/
+/* CASTEP data */
+/***************/
+struct castep_pak
+{
+gdouble energy;
+gdouble max_grad;
+gdouble rms_grad;
+gint min_ok;
+gint have_energy;
+gint have_max_grad;
+gint have_rms_grad;
+};
+
 /**********************/
 /* surface generation */
 /**********************/
 struct surface_pak
 {
-gint model;
+struct model_pak *model;
 gint optimise;         /* relaxed/unrelaxed */
 gint converge_eatt;    /* T - eatt, F - esurf */
 gint converge_r1;
@@ -608,6 +617,7 @@ gint include_polar;    /* calc shifts - keep polar surfaces */
 gint ignore_bonding;   /* gensurf - ignore bonding */
 gint create_surface;
 gint true_cell;
+gint keep_atom_order;
 gdouble miller[3];
 gdouble shift;
 gdouble dspacing;
@@ -624,6 +634,7 @@ struct diffract_pak
 gint radiation;
 gdouble wavelength;
 gdouble theta[3];
+gdouble dhkl_min;
 gint broadening;
 gdouble asym;
 gdouble u, v, w;
@@ -644,6 +655,7 @@ gint state;              /* the mode state (always reset when changing modes) */
 gint grafted;            /* indicates if it's been put on the model tree yet */
 gint redraw;             /* indicates a redraw is required */
 gint redraw_count;       /* current number of redraws */
+gint redraw_current;     /* current redraw time (micro-sec) */
 gint redraw_cumulative;  /* cumulative redraw time (micro-sec) */
 gint redraw_time;        /* average redraw time (micro-sec) */
 gint fractional;
@@ -680,8 +692,10 @@ gint show_title;
 gint show_hbonds;
 gint show_frame_number;
 gint show_energy;
-gint show_charges;
+gint show_charge;
+gint show_atom_charges;
 gint show_atom_labels;
+gint show_atom_types;
 gint show_geom_labels;
 gint show_cores;
 gint show_shells;
@@ -690,10 +704,6 @@ gint show_cell;
 gint show_cell_images;
 gint show_cell_lengths;
 
-/* deprec. */
-GtkWidget *tree_label;
-GtkWidget *tree_name;
-
 /* ghosts */
 gint num_ghosts;
 GSList *ghosts;
@@ -709,11 +719,22 @@ GSList *shels;
 GSList *bonds;    /* radii based connectivity */
 GSList *ubonds;   /* user modified bond list (overlaid on the above) */
 GSList *moles;
-GSList *geom;
 GSList *images;
 GSList *selection;
 GSList *unique_atom_list;
+GSList *measure_list;
 GSList *layer_list;
+GSList *graph_list;
+GSList *picture_list;
+GSList *transform_list;
+GSList *frame_data_list;
+gpointer graph_active;
+gpointer picture_active;
+gpointer project;
+
+/* property hash table and ranked list */
+GHashTable *property_table;
+GSList *property_list;
 
 /* spatial partioning data */
 gint zone_div[3];
@@ -727,6 +748,8 @@ gpointer *zones;
 GSList *ribbons;
 GSList *spatial;
 GSList *phonons;
+GSList *ir_list;
+GSList *raman_list;
 gint num_phonons;
 gint current_phonon;
 gint pulse_count;
@@ -740,6 +763,9 @@ gdouble angle[3];
 gdouble da;
 gdouble pbc[6];
 gdouble centroid[3];
+gdouble cell_angles[3];
+gdouble area;
+gdouble volume;
 /* translation matrix/vectors & its inverse */
 gdouble latmat[9];
 gdouble ilatmat[9];
@@ -748,9 +774,13 @@ gdouble rlatmat[9];
 /* cummulative rotation matrix & its inverse */
 gdouble rotmat[9];
 gdouble irotmat[9];
+/* rotation/translation transformation matrix */
+gdouble display_lattice[16];
 /* special object flags */
 gint axes_type;
-gint mesh_on, box_on, csurf_on, asym_on, solsurf_on, molsurf_on;
+gint mesh_on, box_on, asym_on;
+gboolean ms_colour_scale;
+gint ms_colour_method;
 
 /* special object data */
 struct vec_pak xlat[6];
@@ -758,11 +788,8 @@ struct vec_pak axes[6];
 struct vec_pak cell[8];
 gint select_box[4];
 
-/* surface info */
-struct molsurf_pak csurf;
 /* morphology data */
 gint morph_type;                /* BFDH, etc. */
-gint hpr;                       /* flag - hidden plane removal */
 gint morph_label;               /* flag - hkl labelling */
 gint num_vertices;
 gint num_planes;                /* total number of faces */
@@ -794,6 +821,8 @@ struct gulp_pak gulp;
 struct gamess_pak gamess;
 struct siesta_pak siesta;
 struct abinit_pak abinit;
+struct nwchem_pak nwchem;
+struct castep_pak castep;
 struct diffract_pak diffract;
 /* surface config */
 struct surface_pak surface;
@@ -805,6 +834,7 @@ gint animation;
 gint animating;
 gint anim_confine;
 gboolean anim_fix;
+gboolean anim_loop;
 gdouble anim_speed;
 gpointer analysis;
 GList *frame_list;
@@ -823,6 +853,7 @@ gchar *label;                 /* text info for the user */
 GSList *ext;                  /* extension matching list */
 gint (*read_file) (gchar *, struct model_pak *);     /* read routine */
 gint (*write_file) (gchar *, struct model_pak *);    /* write routine */
+gint (*read_frame) (FILE *, struct model_pak *);     /* animation read */
 };
 
 /******************/
@@ -903,7 +934,6 @@ void write_dat(struct model_pak *);
 
 gdouble ran2(void);
 gint fill(void);
-gint make_pov(gint, gchar *);
 
 void run_pov(void);
 void exec_pov(gchar *);
@@ -918,29 +948,24 @@ void animate_setup_widget(void);
 gint get_frame(struct model_pak *data, gint);
 
 void create_new_model(void);
-void delete_model(gint);
 void delete_active(void);
 void free_model(struct model_pak *);
 
 void mem_shrink(struct model_pak *);
 
-void update_shells(struct model_pak *);
 void template_model(struct model_pak *);
 gint prep_model(struct model_pak *);
 
-gint elem_type(const gchar *);
-gint pdb_elem_type(const gchar *);
-gint copy_elem_data(struct model_pak *, struct model_pak *);
+gint elem_symbol_test(const gchar *);
+gint elem_number_test(const gchar *);
+gint elem_test(const gchar *);
 
-void energetics_dialog(GtkWidget *, struct model_pak *);
+gint pdb_elem_type(const gchar *);
 
 void save_apd(void);
 
 gint geom_label(gint, struct model_pak *, gint *, gchar *);
 void geom_info(void);
-void meas_graft_all(struct model_pak *);
-void meas_prune_all(struct model_pak *);
-void meas_free_all(struct model_pak *);
 void update_creator(void);
 
 void set_rgb_colour(GdkGC *, gint *);
@@ -979,18 +1004,18 @@ void make_rotmat(gdouble *, gdouble , gint);
 void gdk_draw(gint);
 void force_view(gint);
 
-void overlay_models_dialog();
+void overlay_models_dialog(void);
 
 void delay(gint);
 
 void expose_canvas(void);
 
-void update_render_dialog(void);
-
 gint read_elem_data(FILE *, gint);
 gint write_elem_data(FILE *);
 
 void cmd_init(gint, gchar **);
 
-void free_slist(GSList *list);
+GSList *slist_gchar_dup(GSList *);
+void free_slist(GSList *);
+void free_list(GList *);
 
