@@ -59,17 +59,14 @@ temp = g_path_get_basename(name);
 /* get the rightmost '.' */
 for (i=strlen(temp) ; i-- ; )
   if (*(temp+i) == '.')
-    break;
+    {
+    base = g_strndup(temp, i);
+    free(temp);
+    return(base);
+    }
 
 /* not found - return the whole thing */
-if (!i)
-  return(temp);
-
-/* if a '.' was found then malloc a truncated string, otherwise */
-base = g_strndup(temp, i);
-free(temp);
-
-return(base);
+return(temp);
 }
 
 /**************************************/
@@ -92,6 +89,44 @@ switch(search_type)
 return(NULL);
 }
 
+/********************************************/
+/* test if a text string looks like a float */
+/********************************************/
+#define DEBUG_STR_IS_FLOAT 0
+gint str_is_float(const gchar *text)
+{
+gint i;
+
+#if DEBUG_STR_IS_FLOAT
+printf("testing [%s] ", text);
+#endif
+
+for (i=0 ; i<strlen(text) ; i++)
+  {
+  switch(text[i])
+    {
+/* exceptions */
+/* TODO - cope with exp notation chars */
+    case '+':
+    case '-':
+    case '.':
+      break;
+
+    default:
+      if (g_ascii_isdigit(text[i]))
+        break;
+#if DEBUG_STR_IS_FLOAT
+printf(" = not a number (contains %c)\n", text[i]);
+#endif
+      return(FALSE);
+    }
+  }
+#if DEBUG_STR_IS_FLOAT
+printf(" = a number.\n");
+#endif
+return(TRUE);
+}
+
 /*******************************************************/
 /* get a float from a string (even in fraction format) */
 /*******************************************************/
@@ -146,15 +181,15 @@ return(val);
 #define DEBUG_GET_KEYWORDS_ANYWHERE 0
 GSList *get_keywords_anywhere(gchar *str)
 {
-gint i, j=0, n;
+gint i, j=0;
 GSList *list=NULL;
 
 #if DEBUG_GET_KEYWORD_ANYWHERE
 printf("extracted: ");
 #endif
 
-n = num_keys();
-for(i=0 ; i<n ; i++)
+i = 0;
+while (keywords[i].code != -1)
   {
   if (strstr(str, keywords[i].label) != NULL)
     {
@@ -164,6 +199,7 @@ printf(" %d",keywords[i].code);
     list = g_slist_prepend(list, GINT_TO_POINTER(keywords[i].code));
     j++;
     }
+  i++;
   }
 list = g_slist_reverse(list);
 
@@ -247,9 +283,6 @@ printf("extracted: ");
 #endif
 
 list = g_malloc((max+1) * sizeof(gint));
-/*
-buff = get_tokens(str, MAX_TOKENS);
-*/
 buff = tokenize(str, &num_tokens);
 
 n=1;
@@ -319,7 +352,7 @@ return(-1);
 /*********************/
 /* tokenize a string */
 /*********************/
-/* replacement routine for get_tokens */
+/* replacement routine for get_tokens() */
 /* will get as many tokens as available (no more messing with MAX_TOKENS) */
 #define DEBUG_TOKENIZE 0
 gchar **tokenize(const gchar *src, gint *num)
@@ -329,7 +362,9 @@ gchar *tmp, *ptr;
 gchar **dest;
 GSList *list=NULL, *item=NULL;
 
-g_assert(src != NULL);
+/* checks */
+if (!src)
+  return(NULL);
 
 /* duplicate & replace all whitespace with a space */
 tmp = g_strdup(src);
@@ -341,18 +376,10 @@ for (i=0 ; i<strlen(tmp) ; i++)
 g_strstrip(tmp);
 
 #if DEBUG_TOKENIZE
-printf("tokenizing [%s]:\n", tmp);
+printf("tokenizing [%s]\n", tmp);
 #endif
 
 len = strlen(tmp);
-
-/* NB: most problems have occured by making MAX_TOKENS too small */
-/* for some reason it can need many more than it would apparently seem */
-/* problem is multiple separators - giving too many tokens */
-/*
-buff = g_strsplit(tmp, " ", MAX_TOKENS);
-*/
-
 i=n=0;
 while(i<len)
   {
@@ -364,7 +391,7 @@ while(i<len)
 /* assign token */
   ptr = g_strndup(tmp+i, j-i);
 
-  list = g_slist_prepend(list, (gpointer *) ptr);
+  list = g_slist_prepend(list, ptr);
   n++;
 
 /* find start of new token */
@@ -375,14 +402,11 @@ while(i<len)
   }
 list = g_slist_reverse(list);
 
-*num = n;
-#if DEBUG_TOKENIZE
-printf("Found %d tokens: ", n);
-#endif
-
 /* return a NULL if no tokens were found */
 if (!n)
   {
+  *num = 0;
+  g_free(tmp);
   free_slist(list);
   return(NULL);
   }
@@ -397,9 +421,14 @@ while (i<n)
   {
   if (item != NULL)
     {
-    *(dest+i) = g_strdup((gchar *) item->data);
+/* comment character - ignore all subsequent tokens */
+    ptr = item->data;
+    if (*ptr == '#')
+      break;
+
+    *(dest+i) = g_strdup(ptr);
 #if DEBUG_TOKENIZE
-printf("%s:", *(dest+i));
+printf(" (%s)", ptr);
 #endif
     item = g_slist_next(item);
     }
@@ -408,19 +437,19 @@ printf("%s:", *(dest+i));
 /* fake item */
     *(dest+i) = g_strdup(" ");;
 #if DEBUG_TOKENIZE
-printf("(empty token):");
+printf(" (empty token)");
 #endif
     }
   i++;
   }
 
-g_assert (i == n);
-
 /* terminate */
-*(dest+n) = NULL;
+*(dest+i) = NULL;
+*num = i;
 
 #if DEBUG_TOKENIZE
-printf("%p\n",*(dest+n));
+printf(" %p",*(dest+i));
+printf(": found %d tokens\n", *num);
 #endif
 
 /* done */
@@ -441,8 +470,10 @@ gchar **buff, line[LINELEN];
 do
   {
   if (fgetline(fp, line))
+    {
+    *num_tokens = 0;
     return(NULL);
-
+    }
   buff = tokenize(line, num_tokens);
   }
 while (!buff);
