@@ -11,7 +11,7 @@
 /* unified render (povray & openGL) */
 enum 
 {
-STICK, BALL_STICK, CPK, LIQUORICE,
+STICK, BALL_STICK, CPK, LIQUORICE, POLYHEDRAL,
 LIGHT_TYPE, DIRECTIONAL, POSITIONAL,
 SCALE, HALO_QUALITY, FAST_ROTATION,
 ANTIALIAS, SHADOWLESS, DRAW_AXES, PERSPECTIVE, WIRE_FRAME, FOG,
@@ -52,9 +52,13 @@ gdouble transmit;
 gchar morph_finish[LINELEN];
 };
 
-
 /* prototypes */
-GSList *render_get_pipes(gint, struct model_pak *);
+GSList *render_get_pipes(gint, gint, gint, struct model_pak *);
 
+void render_make_pipes(GSList **, struct model_pak *);
 
+GSList *render_sort_pipes(GSList *);
+
+void create_polyhedra(struct model_pak *);
+void destroy_polyhedra(struct model_pak *);
 
