@@ -28,14 +28,18 @@ The GNU GPL can also be found at http://www.gnu.org
 #include "coords.h"
 #include "matrix.h"
 #include "spatial.h"
+#include "measure.h"
 #include "opengl.h"
 #include "select.h"
+#include "zone.h"
 #include "interface.h"
 #include "gtkshorts.h"
 
 extern struct sysenv_pak sysenv;
 extern struct elem_pak elements[];
 
+gint backbone = 0;
+
 /**********************/
 /* set selection mode */
 /**********************/
@@ -60,6 +64,13 @@ switch(len)
   case 11:
     sysenv.select_mode = ATOM_LABEL;
     break;
+
+/* NEW */
+  case 19:
+    sysenv.select_mode = FRAGMENT;
+    switch_mode(SELECT_FRAGMENT);
+    return;
+
   case 20:
     sysenv.select_mode = ELEM_MOL;
     break;
@@ -81,7 +92,7 @@ GSList *list;
 struct core_pak *core;
 struct model_pak *data;
 
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -104,7 +115,7 @@ GSList *list;
 struct core_pak *core;
 struct model_pak *data;
 
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -122,7 +133,9 @@ redraw_canvas(SINGLE);
 /* change selection rendering mode */
 /***********************************/
 /* NB: mode = -1 means atom will follow the global rendering mode */
+/*
 void set_selection_rendering(gint mode, struct model_pak *model)
+void render_mode_set(gint mode, struct model_pak *model)
 {
 GSList *list;
 struct core_pak *core;
@@ -130,7 +143,34 @@ struct core_pak *core;
 for (list=model->selection ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
-  core->render = mode;
+  core->render_mode = mode;
+  }
+}
+*/
+
+/* TODO - same for shells */
+
+void core_render_mode_set(gint mode, GSList *cores)
+{
+GSList *list;
+struct core_pak *core;
+
+for (list=cores ; list ; list=g_slist_next(list))
+  {
+  core = (struct core_pak *) list->data;
+  core->render_mode = mode;
+  }
+}
+
+void core_render_wire_set(gint wire, GSList *cores)
+{
+GSList *list;
+struct core_pak *core;
+
+for (list=cores ; list ; list=g_slist_next(list))
+  {
+  core = (struct core_pak *) list->data;
+  core->render_wire = wire;
   }
 }
 
@@ -143,7 +183,7 @@ GSList *list;
 struct model_pak *data;
 struct core_pak *core;
 
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -165,19 +205,16 @@ redraw_canvas(SINGLE);
 /******************/
 void select_copy(void)
 {
-gint model;
-struct model_pak *data;
+struct model_pak *model;
 
 /* setup & check */
-model = sysenv.active;
-data = model_ptr(model, RECALL);
-if (data == NULL)
+model = sysenv.active_model;
+if (!model)
   return;
-if (!data->selection)
+if (!model->selection)
   return;
 
-/* save ptr as this won't change if models (other than itself) are deleted */
-sysenv.select_source = data;
+sysenv.select_source = model;
 }
 
 /*********************/
@@ -185,14 +222,12 @@ sysenv.select_source = data;
 /*********************/
 void select_paste(void)
 {
-gint model;
 struct model_pak *data, *src;
 struct core_pak *core, *orig;
 GSList *clist;
 
 /* checks */
-model = sysenv.active;
-data = model_ptr(model, RECALL);
+data = sysenv.active_model;
 src = sysenv.select_source;
 if (data == NULL || src == NULL)
   return;
@@ -225,10 +260,8 @@ for (clist=src->selection ; clist ; clist=g_slist_next(clist))
   }
 
 /* updates */
-/*
-con_init_zones(data);
-*/
 init_objs(INIT_COORDS, data);
+zone_init(data);
 calc_bonds(data);
 calc_mols(data);
 
@@ -246,7 +279,7 @@ GSList *list;
 struct core_pak *core;
 struct model_pak *model;
 
-model = model_ptr(sysenv.active, RECALL);
+model = sysenv.active_model;
 if (!model)
   return;
 
@@ -273,11 +306,9 @@ redraw_canvas(SINGLE);
 /********************/
 void select_colour(void)
 {
-/* exit if nothing is loaded */
-if (!sysenv.num_models)
-  return;
-
-new_csd("Selection colour", (gpointer) cb_select_colour);
+/* no action if nothing is loaded */
+if (sysenv.active_model)
+  new_csd("Selection colour", (gpointer) cb_select_colour);
 }
 
 /********************/
@@ -289,12 +320,10 @@ GSList *list;
 struct model_pak *data;
 struct core_pak *core;
 
-/* exit if nothing is loaded */
-if (!sysenv.num_models)
-  return;
-
 /* deletion for the active model only */
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
+if (!data)
+  return;
 
 /* delete */
 list = data->selection;
@@ -321,11 +350,8 @@ GSList *list;
 struct model_pak *data;
 struct core_pak *core;
 
-/* exit if nothing is loaded */
-if (!sysenv.num_models)
-  return;
 /* deletion for the active model only */
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 /* exit if nothing selected */
@@ -360,7 +386,7 @@ struct core_pak *core;
 struct model_pak *data;
 
 /* deletion for the active model only */
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -387,7 +413,7 @@ struct core_pak *core;
 struct model_pak *data;
 
 /* deletion for the active model only */
-data = model_ptr(sysenv.active, RECALL);
+data = sysenv.active_model;
 if (!data)
   return;
 
@@ -462,17 +488,19 @@ for (list=model->cores ; list ; list=g_slist_next(list))
 /**********************************************************/
 /* select all elements of given type from a list of cores */
 /**********************************************************/
-void select_all_elem(gint code, struct model_pak *data)
+void select_all_elem(gint code, struct model_pak *model)
 {
 GSList *list;
 struct core_pak *core;
 
-for (list=data->cores ; list ; list=g_slist_next(list))
+g_assert(model != NULL);
+
+for (list=model->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
  
   if (core->atom_code == code)
-    select_add_core(core, data);
+    select_add_core(core, model);
   }
 }
 
@@ -484,6 +512,9 @@ void select_all_elem_in_mol(struct core_pak *core, struct model_pak *model)
 GSList *list;
 struct core_pak *comp;
 
+g_assert(model != NULL);
+g_assert(core != NULL);
+
 for (list=model->cores ; list ; list=g_slist_next(list))
   {
   comp = (struct core_pak *) list->data;
@@ -495,18 +526,135 @@ for (list=model->cores ; list ; list=g_slist_next(list))
 /**********************************************/
 /* add a particular molecule to the selection */
 /**********************************************/
-void select_add_mol(struct mol_pak *mol, struct model_pak *data)
+void select_add_mol(struct mol_pak *mol, struct model_pak *model)
 {
 GSList *list;
 struct core_pak *core;
 
+g_assert(model != NULL);
+if (!mol)
+  return;
+
 for (list=mol->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
-  select_add_core(core, data);
+  select_add_core(core, model);
   }
 }
 
+/**********************************************/
+/* add a particular molecule to the selection */
+/**********************************************/
+#define DEBUG_ADD_FRAGMENT 0
+void select_add_fragment(struct core_pak *core, struct model_pak *model)
+{
+static struct core_pak *first=NULL, *core1, *core2;
+GSList *list, *item1, *item2, *branch_all, *branch_new;
+
+/* test for first or second call */
+if (model->state)
+  {
+  g_assert(first != NULL);
+
+/* clear the pruned flag */
+  for (item1=model->cores ; item1 ; item1=g_slist_next(item1))
+    {
+    core1 = (struct core_pak *) item1->data;
+    core1->status &= ~PRUNED;
+    }
+
+/* get initial branches */
+  branch_new = exit_branches(first, core);
+  item1 = branch_all = g_slist_prepend(branch_new, core);
+
+#if DEBUG_ADD_FRAGMENT
+printf("seeking branches for: %p -> %p : ", first, core);
+printf("initial branches: %d\n", g_slist_length(branch_new));
+for (item3=branch_new ; item3 ; item3 = g_slist_next(item3))
+  {
+  core3 = (struct core_pak *) item3->data;
+  printf(" - %p (%s)\n", core3, core3->label);
+  }
+#endif
+
+/* iterate through upper branch level */
+  while (item1)
+    {
+    core1 = (struct core_pak *) item1->data;
+    item1 = g_slist_next(item1);
+    g_assert(core1 != NULL);
+
+/* iterate through lower branch level and aquire next level of branches */
+    item2 = branch_new;
+    branch_new = NULL;
+    while (item2)
+      {
+      core2 = (struct core_pak *) item2->data;
+      g_assert(core2 != NULL);
+      item2 = g_slist_next(item2);
+      if (core2->status & PRUNED)
+        continue;
+
+/* prune this core, so its branches are not reexamined */
+      core2->status |= PRUNED;
+
+      list = exit_branches(core1, core2);
+
+#if DEBUG_ADD_FRAGMENT
+{
+struct core_pak *core3;
+GSList *item3;
+
+printf("branches: %p (%s) -> %p (%s)", core1, core1->label, core2, core2->label);
+printf(" : found %d.\n", g_slist_length(list));
+for (item3=list ; item3 ; item3 = g_slist_next(item3))
+  {
+  core3 = (struct core_pak *) item3->data;
+  printf(" - %p (%s)\n", core3, core3->label);
+  }
+}
+#endif
+
+/* aquire all branches at current level */
+      branch_new = g_slist_concat(branch_new, list);
+      }
+
+/* test if we arrive back at first core (from a different direction) */
+    if (g_slist_find(branch_new, first))
+      {
+      show_text(ERROR, "Bad fragment.\n");
+      g_slist_free(branch_all);
+      model->state = 0;
+      first = NULL;
+      return;
+      }
+
+/* accumlate new branches into main list */
+    branch_all = g_slist_concat(branch_all, branch_new);
+    }
+
+/* add all cores to the selection */
+/* FIXME - why is the first core's selection getting wiped??? */
+  select_add_core(first, model);
+  select_add_core(core, model);
+  for (item1=branch_all ; item1 ; item1=g_slist_next(item1))
+    {
+    core1 = (struct core_pak *) item1->data;
+    select_add_core(core1, model);
+    }
+
+/* done */
+  g_slist_free(branch_all);
+  model->state = 0;
+  first = NULL;
+  }
+else
+  {
+  first = core;
+  select_add_core(core, model);
+  model->state++;
+  }
+}
 
 /******************************************/
 /* add all cores of the same region label */
@@ -516,6 +664,9 @@ void select_add_region(struct core_pak *core, struct model_pak *model)
 GSList *list;
 struct core_pak *comp;
 
+g_assert(model != NULL);
+g_assert(core != NULL);
+
 /*
 printf("select region: %d in: %p\n", core->region, model);
 */
@@ -531,58 +682,13 @@ for (list=model->cores ; list ; list=g_slist_next(list))
 /******************************************/
 /* remove an atom from the selection list */
 /******************************************/
-void select_del_core(struct core_pak *core, struct model_pak *data)
+void select_del_core(struct core_pak *core, struct model_pak *model)
 {
+g_assert(model != NULL);
+g_assert(core != NULL);
+
 core->status &= ~SELECT;
-data->selection = g_slist_remove(data->selection, core);
-}
-
-/*********************************************************************/
-/* put in the selection all atoms/mols participating in a geom label */
-/*********************************************************************/
-void select_model_labels(void)
-{
-GSList *list1, *list2;
-struct model_pak *data;
-struct geom_pak *geom;
-struct core_pak *core;
-
-/* checks */
-data = model_ptr(sysenv.active, RECALL);
-if (!data)
-  return;
-
-switch(sysenv.select_mode)
-  {
-  case MOL:
-/* go through all labels */
-    for (list1=data->geom ; list1 ; list1=g_slist_next(list1))
-      {
-      geom = (struct geom_pak *) list1->data;
-
-/* append associated molecules of the cores to the selection */
-      for (list2=geom->cores ; list2 ; list2=g_slist_next(list2))
-        {
-        core = (struct core_pak *) list2->data;
-        select_add_mol(core->mol, data); 
-        }
-      }
-    break;
-
-  default:
-/* go through all labels */
-    for (list1=data->geom ; list1 ; list1=g_slist_next(list1))
-      {
-      geom = (struct geom_pak *) list1->data;
-
-/* copy the measurement core list and append it to the selection */
-      data->selection = g_slist_concat(data->selection, g_slist_copy(geom->cores));
-      }
-    break;
-  }
-
-/* update */
-redraw_canvas(SINGLE);
+model->selection = g_slist_remove(model->selection, core);
 }
 
 /********************************/
@@ -657,7 +763,7 @@ if (data->cbu)
   calc_mols(data);
   }
 
-meas_graft_all(data);
+meas_graft_model(data);
 }
 
 /*************************/
@@ -688,7 +794,7 @@ if (data->cbu)
   calc_mols(data);
   }
 
-meas_graft_all(data);
+meas_graft_model(data);
 }
 
 /************************************/
@@ -714,9 +820,9 @@ for (list=core->bonds ; list ; list=g_slist_next(list))
   if (exit->status & PRUNED)
     continue;
 
-/* CURRENTLY - carbon only backbone */
-  if (exit->atom_code == 6)
-    branches++;
+  if (backbone)
+    if (exit->atom_code == backbone)
+      branches++;
   }
 
 return(branches);
@@ -732,20 +838,17 @@ struct bond_pak *bond;
 struct core_pak *exit;
 
 /* checks */
-if (core2->atom_code != 6)
-  return(NULL);
-
-/* NB: core1 may be NULL ie no entry point */
-/*
-if (core1)
-  g_assert(core1->atom_code == 6);
-*/
+if (backbone)
+  if (core2->atom_code != backbone)
+    return(NULL);
 
 for (list1=core2->bonds ; list1 ; list1=g_slist_next(list1))
   {
   bond = (struct bond_pak *) list1->data;
   if (bond->periodic == BOND_SPLIT)
     continue;
+  if (bond->type == BOND_HBOND)
+    continue;
 
 /* get exits */
   if (core2 == bond->atom1)
@@ -757,13 +860,18 @@ for (list1=core2->bonds ; list1 ; list1=g_slist_next(list1))
   if (exit == core1)
     continue;
 
-/* preprocess has removed all the dead end branches */
+/* ignore marked (pruned) branches */
   if (exit->status & PRUNED)
     continue;
 
-/* CURRENT - carbon only backbone */
-  if (exit->atom_code == 6)
-    exits = g_slist_prepend(exits, exit);
+/* if constructing ribbon first atom is backbone type */
+  if (backbone)
+    {
+    if (exit->atom_code != backbone)
+       continue;
+    }
+
+  exits = g_slist_prepend(exits, exit);
   }
 
 return(exits);
@@ -1086,7 +1194,6 @@ for (list=core->bonds ; list ; list=g_slist_next(list))
 /*********************************************/
 /* eliminate (prune) all dead end type links */
 /*********************************************/
-/* FIXME - currently quite inefficient... */
 #define DEBUG_PREPROCESS 0
 void preprocess(struct core_pak *start, struct model_pak *data)
 {
@@ -1096,13 +1203,18 @@ struct core_pak *core;
 
 type = start->atom_code;
 
-/* clear the pruned flag */
+/* clear the pruned flag for backbone & prune all others */
 for (list=data->cores ; list ; list=g_slist_next(list))
   {
   core = (struct core_pak *) list->data;
-  core->status &= ~PRUNED;
+
+  if (core->atom_code == type)
+    core->status &= ~PRUNED;
+  else
+    core->status |= PRUNED;
   }
 
+/* prune "dangling" backbone type links */
 flag=1;
 while (flag)
   {
@@ -1111,15 +1223,12 @@ while (flag)
     {
     core = (struct core_pak *) list->data;
 
-/* only prune the backbone type */
-    if (core->atom_code != type)
-      continue;
 /* only prune once */
     if (core->status & PRUNED)
       continue;
 
 /* prune off dead-ends */
-    if (total_branches(core) == 1)
+    if (total_branches(core) < 2)
       {
       core->status |= PRUNED;
       flag++;
@@ -1165,12 +1274,17 @@ struct node_pak *current_node, *repeat_node, *temp_node;
 struct link_pak *current_link;
 struct core_pak *core, *path;
 
-/* prune the model first (ie remove dangling chains) */
-/* to make tracing the connectivity easier */
-preprocess(start, data);
+/* checks */
+g_assert(start != NULL);
+g_assert(data != NULL);
 
 /* clicked atom */
 core = start;
+backbone = start->atom_code;
+
+/* prune the model first (ie remove dangling chains) */
+/* to make tracing the connectivity easier */
+preprocess(start, data);
 
 /* attempt to set up initial node */
 switch(total_branches(core))
@@ -1261,7 +1375,7 @@ while (current_node)
   if (core == current_node->core)
     num_exits = 2;
 
-  switch(num_exits)
+  switch (num_exits)
     {
 /* pre-process should remove all such possibilities */
     case 0:
@@ -1403,6 +1517,7 @@ for (tmp_list=data->cores ; tmp_list ; tmp_list=g_slist_next(tmp_list))
 
 /* use cyclic/single linkages to construct ribbon */
 construct_ribbon(node_list, data);
+backbone = 0;
 
 /* recalc coords (ie the ribbon coords) & draw */
 init_objs(REDO_COORDS, data);
