@@ -20,11 +20,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 The GNU GPL can also be found at http://www.gnu.org
 */
 
-gint genpos(struct model_pak *);
-void update_images(gint, struct model_pak *);
-void make_supercell(GtkWidget *, struct model_pak *);
-void make_cartesian(struct model_pak *);
-void make_p1(struct model_pak *);
-void space_info(struct model_pak *);
+gint space_lookup(struct model_pak *);
+void space_free(gpointer);
 
+void space_make_p1(struct model_pak *);
+void space_make_images(gint, struct model_pak *);
+void space_make_supercell(struct model_pak *);
+
+void space_image_widget_setup(GtkWidget *);
+void space_image_widget_redraw(void);
+void space_image_widget_reset(void);
 
