@@ -1,6 +1,6 @@
 
 /* task modes */
-enum {QUEUED, RUNNING, SUSPENDED};
+enum {RUNNING, QUEUED, COMPLETED, KILLED};
 
 /************/
 /* task pak */
@@ -20,7 +20,12 @@ gint min;
 gint hour;
 gchar *label;
 gchar *message;
+
+#ifdef __WIN32
+gpointer start_time;
+#else
 time_t start_time;
+#endif
 
 gchar *time;
 gdouble pcpu;
@@ -42,13 +47,13 @@ gint init_task_thread(void);
 gint update_tasks(void);
 gint update_task_info(void);
 
-void submit_task(gchar *, gpointer, gpointer, gpointer, gpointer, gpointer);
+void submit_task(const gchar *, gpointer, gpointer, gpointer, gpointer, gpointer);
 
 void task_dialog(void);
 
 gint task_sync(const gchar *);
 
-gint exec_gulp(gchar *, gchar *);
+gint exec_gulp(const gchar *, const gchar *);
 
 /* work functions */
 void exec_gulp_task(gpointer *);
@@ -56,6 +61,10 @@ void proc_gulp_task(gpointer *);
 void exec_pov_task(gpointer *);
 void exec_img_task(gpointer *);
 
-
-
+void task_queue_init(void);
+void task_queue_free(void);
+void task_new(const gchar *,
+              gpointer, gpointer,
+              gpointer, gpointer,
+              gpointer );
 
